//---------------------------------------------------------------
// Small RTCUCSP.DLL sample program
//---------------------------------------------------------------
#include <windows.h>
#include <process.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <rtcucsp.h>

//---------------------------------------------------------------
// Connection handle
//---------------------------------------------------------------
HRMONCON hCon;

//---------------------------------------------------------------
// Receive any incoming Debug messages from RTCU
//---------------------------------------------------------------
static void thDebug(void *arg) {
  char buffer[512];
  int rc;
  for (;;) {
    // Wait for any debug messages from unit
    rc=rmonReceiveDebugMsg(hCon, buffer, sizeof(buffer));
    if (rc==0) {
      // Just print the debug message
      printf("Debug::[%s]\n", buffer);
    } else {
      Sleep(50);
    }
  }
}

//---------------------------------------------------------------
// Callback function that will be called by rmonFirmwareUpload()
// to report progress in the upload process
//---------------------------------------------------------------
static int RMONCC cbfuncFW(void* uptr,int percentage) {
  printf("Firmware upload finished: %3i\r", percentage);
  return 0;
}

//---------------------------------------------------------------
// Callback function that will be called by rmonApplicationUpload()
// to report progress in the upload process
//---------------------------------------------------------------
static int RMONCC cbfuncApp(void* uptr,int percentage) {
  printf("Application upload finished: %3i\r", percentage);
  return 0;
}

//---------------------------------------------------------------
// Callback function that will be called by rmonVoiceUpload()
// to report progress in the upload process
//---------------------------------------------------------------
static int RMONCC cbfuncVoice(void* uptr,int percentage) {
  printf("Voice upload finished: %3i\r", percentage);
  return 0;
}

//---------------------------------------------------------------
// the main program
//---------------------------------------------------------------
int main(int argc,char** argv) {  
  int rc;


  puts("The RTCU Demo project is running\n\r");

  // Open RTCUCSP library
  rmonOpen();

  // Open a connection
  hCon = rmonOpenConnection();

  // Select which comports to use for local and remote connections. (Use COM0 if no port is to be used)
  rmonSetComport(hCon, "USB1", "COM0");

  // Connect to unit via cable
  rmonConnect(hCon, "");

  // Start the listener thread for incoming Debug messages
  _beginthread(thDebug, 0, NULL);

  // Wait for a connection to a RTCU unit
  while (1) {
    // Check and wait for connection (can be RMONCON_LOCAL, RMONCON_REMOTE, RMONCON_GW or RMONCON_NONE)
    if (rmonConnected(hCon) != RMONCON_NONE)
      break;
    Sleep(300);
  }
  printf("Connected to unit.\n");

  // Try to authenticate with an empty password:
  rc=rmonAuthenticate(hCon, "");
  switch (rc) {
    case rmonDenied: printf("Authenticate with empty password denied. Use correct password !\n"); break;
    case rmonOK: printf("Authenticate with empty password accepted !\n"); break;
  }

  if (rc==rmonDenied) {
    printf("Not able to logon to unit !\n");
    return 1;
  }

  // Get information about the unit
  int targetid, firmwareversion;
  rmonGetTargetInfo(hCon, &targetid, &firmwareversion);
  printf("Target ID=%i, Firmware version=0x%X\n", targetid, firmwareversion);

  // Get the units serial number
  unsigned long SerialNumber;
  rmonGetSerialNumber(hCon, &SerialNumber);
  printf("Serialnumber of unit is %09i\n", SerialNumber);
  

  // Use this to upload new firmware to the unit:
  //printf("\nrc=%i\n", rmonFirmwareUpload(hCon, "D:\\FirmwareFile.bin", cbfuncFW, (void*)0));


  // Use the following to upload a new application and voice messages:
  /*
  rmonHalt(hCon);
  printf("\nrc=%i\n", rmonApplicationUpload(hCon, "D:\\APP\\APP.VSX", cbfuncApp, NULL));
  printf("\nrc=%i\n", rmonVoiceUpload(hCon, "C:\\APP\\APP.PRJ", cbfuncVoice, NULL));
  rmonReset(hCon);
  */


  printf("\n\nPress any key to end program...\n\n");
  for (;;) {
    if (_getch()) break;
  }
  // Close connection after use
  rmonCloseConnection(hCon);

 return 0;
}
 

