/***************************** C HEADER FILE *********************************
**
**  Project:    Logic IO RTCU Communication Support Package Library
**  Filename:   RTCUCSP.H 
**  Version:    4.50
**  Date:       April, 2022
**  
*****************************************************************************/
#ifndef _RTCUCSP_H_INCLUDED
#define _RTCUCSP_H_INCLUDED

#ifdef __cplusplus
  extern "C" {
#endif


/*****************************************************************************
**
**    MODULES USED 
**
*****************************************************************************/


/*****************************************************************************
**
**    DEFINITIONS AND MACROS
**
*****************************************************************************/
#define RMONCC                      __stdcall
#define HRMONCON                    void*
#define HRMONCON_ILLEGAL            NULL

#define RMON_MAXPORTS               16

typedef enum {
   rmonOK=0,                        // Success
   rmonError,                       // General error
   rmonComError,                    // Error communicating with RTCU unit
   rmonTargetError,                 // Function failed in RTCU unit
   rmonIllegalHandle,               // Not a valid connection handle
   rmonIllegalTarget,               // Target does not support function
   rmonOnlyGateway,                 // The function can only be used over a gateway
   rmonNotGateway,                  // The function can not be used over a gateway
   rmonDenied,                      // Access to RTCU unit denied
   rmonNoData,                      // There are no data to read
   rmonNoMoreData,                  // There are no more data
   rmonNotInit,                     // Library or Datalog is not initialized
   rmonInit,                        // The RTCUCSP library is already initialized
   rmonConnection,                  // There is a connection established already
   rmonGatewayNotFound,             // Gateway not found
   rmonFileNotFound,                // File not found
   rmonIllegalFile,                 // File specified is illegal (corrupt/non existent)
   rmonOldFormat,                   // Old file format, no longer supported.
   rmonNoMonitorMode,               // Not able to enter monitor mode
   rmonErrorReset,                  // Not able to reset RTCU unit
   rmonErrorHalt,                   // Not able to halt RTCU unit
   rmonNoBackground,                // Background transfer is not supported by RTCU unit.
   rmonInterrupted,                 // Background transfer was interrupted.
   rmonCancelled,                   // Data transfer has been cancelled
   rmonNotProgrammable,             // Attempt to program a Micro unit with a VSX file
   rmonNoModem,                     // It is not possible to use Modem connections.
   rmonNoCable,                     // It is not possible to use Cable connections.
   rmonMemoryConfig,                // The memory configuration is out of sync between Project and Unit.
   rmonImageTooLarge,               // There are not room for the image in the unit. (Voice messages, etc.)
   rmonNotModem,                    // The function can not be used over a modem.
   rmonIllegalAccess,               // The function is not accessible.
   rmonDowngrade,                   // The unit does not support this firmware version
   rmonSize,                        // The buffer is not large enough
   rmonImageSupport                 // The device do not support the image type
} rmonRet;

#define RMONCON_NONE                0
#define RMONCON_LOCAL               1
#define RMONCON_REMOTE              2
#define RMONCON_GW                  3

#define RMON_IOTYPE_DIN             1
#define RMON_IOTYPE_DOUT            2
#define RMON_IOTYPE_AIN             3
#define RMON_IOTYPE_AOUT            4
#define RMON_IOTYPE_LED             5
#define RMON_IOTYPE_DIPSW           6
#define RMON_IOTYPE_FLEX_DIN        7
#define RMON_IOTYPE_FLEX_AIN        8

#define RMONSMS_TEXT                0
#define RMONSMS_BINARY              1

#define RMONLOGGET_NEXT             1
#define RMONLOGGET_PREV             2

#define RMONLOGGOTOLINSEC_BACK      0
#define RMONLOGGOTOLINSEC_FORWARD   1

#define RMON_TYPE_NA                0
#define RMON_TYPE_SDCARD            1
#define RMON_TYPE_INTERNAL          2

#define RMONTGT_ICP002              1
#define RMONTGT_ICP003              2
#define RMONTGT_ICP004              4
#define RMONTGT_ICP005              5
#define RMONTGT_ICP006              6
#define RMONTGT_ICP007              7
#define RMONTGT_ICP009              9
#define RMONTGT_ICP010              10
#define RMONTGT_ICP011              11
#define RMONTGT_ICP102              102
#define RMONTGT_ICP103              103
#define RMONTGT_ICP104              104
#define RMONTGT_ICP105              105
#define RMONTGT_ICP106              106
#define RMONTGT_ICP109              109
#define RMONTGT_ICP122              122
#define RMONTGT_ICP129              129
#define RMONTGT_ICP192              192
#define RMONTGT_ICP202              202
#define RMONTGT_ICP204              204
#define RMONTGT_ICP209              209
#define RMONTGT_ICP302              302
#define RMONTGT_ICP304              304
#define RMONTGT_ICP305              305

#define RMONFS_INVALIDDRIVE         101
#define RMONFS_NOTFOUND             105
#define RMONFS_DUPLICATED           106
#define RMONFS_NOMOREENTRY          107
#define RMONFS_NOTOPEN              108
#define RMONFS_LOCKED               112
#define RMONFS_NOTEMPTY             114
#define RMONFS_CARDREMOVED          116
#define RMONFS_ONDRIVE              117
#define RMONFS_BUSY                 122
#define RMONFS_WRITEPROTECT         123
#define RMONFS_FILEACCESS           138
#define RMONFS_EXTENSION            139

#define RMONFS_TYPE_FILE            0
#define RMONFS_TYPE_FOLDER          1

#define RMONFS_FMTFLAG_FULL         1

#define RMONCT_APP_DIRECT           1
#define RMONCT_APP_BACKGROUND       2
#define RMONCT_FW_DIRECT            3
#define RMONCT_FW_BACKGROUND        4
#define RMONCT_VOICE                5

#define RMONCT_REP_VOICE            1
#define RMONCT_REP_FW300            2
#define RMONCT_REP_DENIED           3

#define RMON_STATE_RUN              1
#define RMON_STATE_HALT             2
#define RMON_STATE_FAULT            3

#define RMON_TYPE_FRAM              1
#define RMON_TYPE_FLASH             2
#define RMON_TYPE_XFLASH            3
#define RMON_PERSIST_TEXT           1
#define RMON_PERSIST_BINARY         2

#define RMON_NET_DHCP               0x0001
#define RMON_NET_AUTODNS            0x0002

#define RMONOBJ_TABLE_SYSTEM        1
#define RMONOBJ_TABLE_USER          2
#define RMONOBJ_FLAG_OVERWRITE      0x0001
#define RMONOBJ_FLAG_IGNORE         0x0002
#define RMONOBJ_TYPE_BOOL           1
#define RMONOBJ_TYPE_INT            2
#define RMONOBJ_TYPE_STRING         3
#define RMONOBJ_TYPE_DATA           4
#define RMONOBJ_TYPE_FLOAT          5

#define RMON_SUPPLY_BAT             1
#define RMON_SUPPLY_DC              2
#define RMON_SUPPLY_AC              3
#define RMON_BAT_NONE               0
#define RMON_BAT_NONCHG             1
#define RMON_BAT_LOW                2
#define RMON_BAT_HIGH               3
#define RMON_BAT_SUPER              4


/****************************************************************************
**
**    TYPEDEFS AND STRUCTURES
**
****************************************************************************/
typedef int  (RMONCC *rmoncbprogress)(void* uptr, int percent);
typedef void (RMONCC *rmoncbserialport)(void *arg, const char *name, const char *desc);
typedef int  (RMONCC *rmoncbdirentry)(void* uptr, const char* filename, unsigned char type, long timestamp, unsigned long size);
typedef void (RMONCC *rmoncbunitstate)(void* uptr, int state);
typedef int  (RMONCC *rmoncbcertificate)(void* uptr, const char* name, unsigned char type, unsigned char key, const char* subject, const char* issuer, long linsec_from, long linsec_to);
typedef int  (RMONCC *rmoncbobject)(void* uptr, const char* name, int type, int size, unsigned char* data, unsigned long flags, int min_val, int max_val, const char* description);
typedef int  (RMONCC *rmoncbobjectx)(void* uptr, const char* name, int type, int size, unsigned char* data, unsigned long flags, int min_size, unsigned char* min_val, int max_size, unsigned char* max_val, const char* description);

struct rmonBGReport {
  unsigned long       extSeg;                  // External segment written
  unsigned long       intSeg;                  // Internal segment written
  unsigned long       headSeg;                 // Header segment written
};

#ifndef RMONRTCTIMEDEFINED
#define RMONRTCTIMEDEFINED
struct rmonRTCTime {                 
   unsigned short year;                        // 1980..2048
   unsigned char  month;                       // 01..12
   unsigned char  date;                        // 01..31
   unsigned char  reserved;                    // was day.
   unsigned char  hour;                        // 00..23
   unsigned char  minute;                      // 00..59
   unsigned char  second;                      // 00..59
};
#endif

struct rmonGPRSSettings {
  // general TCP/IP parameters:
  unsigned long       ip_address;              // ip-address
  unsigned long       subnet_mask;             // subnet mask
  unsigned long       gateway;                 // gateway-address
  unsigned long       dns_1;                   // DNS-address 1
  unsigned long       dns_2;                   // DNS-address 2

  // PPP parameters:
  char                username[34];            // username.
  char                password[34];            // username.

  // Dialup/GPRS parameters:
  char                APN[34];                 // APN
  unsigned short      authentication;          // Authentication type, 0=none, 1=pap, 2=chap, 3=pap/chap (Input to SGAUTH)
  char                modem_init[254];         // Modem init string
};

struct rmonGWSettings {
  // Gateway parameters:
  unsigned short      gw_enabled;              // 0=disabled, 1=enabled.
  char                gw_ip[42];               // gateway address (dotted / symbolic)
  unsigned short      gw_port;                 // gateway port (def: 5001)
  char                gw_key[10];              // gateway key
  char                phonenumber_sms[22];     // phonenumber for SMS messages.
  unsigned char       crypt_key[16];           // encryption key used to encrypt/decrypt data to/from the gateway.

  // advanced settings (modification not recommended):
  unsigned short      max_connection_attempt;  // max number of connection attempts before GPRS re-connects. 
                                               // interval: 1..60. (default: 3)
  unsigned short      max_send_req_attempt;    // max number of send-request attempt before send fails.
                                               // interval: 1..60 (default: 3)
  unsigned short      response_timeout;        // time waiting for response in seconds
                                               // interval: 5..60 (def: 30 seconds).
  unsigned short      alive_freq;              // frequency for sending self-transactions in seconds
                                               // interval: 0..60000 (def: 300 seconds). 
                                               // 0 (zero) will disable self-trans.
};

struct rmonRCHConfig {
  unsigned long       size;                    // The size of the structure.

  unsigned char       enabled;                 // Enable/Disable configuration
  unsigned char       secure;                  // 0=Use standard connection, 1=Use secure connection
  char                host[51];                // The address of the server. (dotted / symbolic)
  unsigned char       iface;                   // The network interface to use.
  unsigned char       crypt_enable;            // Enable/Disable encryption of data to/from server.
                                               // Only used for standard connection.
  unsigned short      port;                    // The port of the server (def: 5001)
  char                login_key[10];           // The server login key
  unsigned char       crypt_key[16];           // The encryption key used to encrypt/decrypt data to/from the server.
                                               // Only used for standard connection.
  unsigned short      max_connection_attempt;  // max number of connection attempts before network interface re-connect.
                                               // interval: 1..60. (default: 3)
  unsigned short      max_send_req_attempt;    // max number of send-request attempt before send fails.
                                               // interval: 1..60 (default: 3)
  unsigned short      response_timeout;        // time waiting for response in seconds
                                               // interval: 5..60 (def: 30 seconds).
  unsigned short      alive_freq;              // frequency for sending self-transactions in seconds
                                               // interval: 0..60000 (def: 300 seconds). 
                                               // 0 (zero) will disable self-trans.
};

struct rmonNetwork {
  unsigned short             flags;            // Network flags: RMON_NET_DHCP, RMON_NET_AUTODNS
  unsigned long              addr;             // ip-address
  unsigned long              subnet;           // subnet mask
  unsigned long              gateway;          // gateway-address
  unsigned long              dns_1;            // DNS-address 1
  unsigned long              dns_2;            // DNS-address 2
};

struct rmonWLANSecurityWPA {
  char                       phrase[64];       // Passphrase. String, 64 Hex digits or 8-63 ASCII characters + zero termination.
};

struct rmonWLANSettings {
  unsigned char              ssid[32];         // SSID - 32 octets.
  unsigned short             security;         // 0 = None, 1 = WPA/WPA2 - PSK
  union u_secure_t {
    struct rmonWLANSecurityWPA wpa;              // WPA security parameters
  } sec;
  struct rmonNetwork         ipv4;             // IPv4 parameters
};

struct rmonFaultRecord {
  unsigned short             year;             // 2000..99
  unsigned char              month;            // 01..12
  unsigned char              date;             // 01..31
  unsigned char              hour;             // 00..23
  unsigned char              minute;           // 00..59
  unsigned char              second;           // 00..59
  unsigned char              Code;
};

struct rmonFault {
  unsigned char              NumRecords;       // Number of records
  unsigned char              NextIn;           // Index of next record that is being inserted
  struct rmonFaultRecord     Record[32];
};

struct rmonFaultRecordX {
  unsigned short      year;                    // 2000..99
  unsigned char       month;                   // 01..12
  unsigned char       date;                    // 01..31
  unsigned char       hour;                    // 00..23
  unsigned char       minute;                  // 00..59
  unsigned char       second;                  // 00..59
  unsigned char       code;
  char                filename[16];            // Filename of fault location
  unsigned short      line;                    // Line number of fault location
  unsigned short      threadid;                // The ID of the thread that faulted.
  unsigned char       data[38];                // Reserved for future use
};

struct rmonFaultX {
  unsigned char              NumRecords;       // Number of records
  unsigned char              NextIn;           // Index of next record that is being inserted
  struct rmonFaultRecordX    Record[16];
};

struct rmonComStatistics {
   int                       RXBytes;          // Number of bytes sent
   int                       TXBytes;          // Number of bytes received.
   int                       RXPackets;        // Number of packets received.
   int                       TXPackets;        // Number of packets sent.
   int                       RXNAK;            // Number of frames NAK'ed by transmitter.
   int                       TXNAK;            // Number of frames NAK'ed by receiver.
   int                       RXTimeout;        // Number of receive timeouts.
   int                       TXTimeout;        // Number of TX packets with timeout.
   int                       RXCRC;            // Number of packets with CRC error.
};

struct rmonMediaInfo {
  unsigned char              Type;             // Type of media
  unsigned long              Size;             // Size of the media, lower 32bit
  unsigned long              SizeHi;           // Size of the media, upper 32bits
};

struct rmonIOCount {
  // Onboard I/O's
  unsigned char              NumberOfAI;       // Number of Analog inputs mounted
  unsigned char              NumberOfAO;       // Number of Analog outputs mounted
  unsigned char              NumberOfDI;       // Number of Digital inputs mounted
  unsigned char              NumberOfDO;       // Number of Digital outputs mounted
  unsigned char              NumberOfDIPSW;    // Number of DIP switches mounted
  unsigned char              NumberOfLED;      // Number of LED's mounted
  // Extension I/O's
  unsigned short             NumberOfExtAI;    // Number of extended Analog inputs mounted
  unsigned short             NumberOfExtAO;    // Number of extended Analog outputs mounted
  unsigned short             NumberOfExtDI;    // Number of extended Digital inputs mounted
  unsigned short             NumberOfExtDO;    // Number of extended Digital outputs mounted
  unsigned short             NumberOfExtDIPSW; // Number of extended DIP switches mounted
  unsigned short             NumberOfExtLED;   // Number of extended LED's mounted
};

struct rmonUnitStatistics {
  short               size;                    // The size of the structure
  short               max_temp;                // Maximum temperature measured
  short               avg_temp;                // Average temperature measured
  short               min_temp;                // Minimum temperature measured
  unsigned long       num_boot;                // The number of times the unit have booted
  unsigned long       time_run;                // The number of minutes the unit have run
  unsigned long       time_bat;                // The number of minutes the unit have run from battery
  unsigned long       time_chg;                // The number of minutes the unit have charged the battery
  unsigned long       gw_connect;              // The number of times the unit have tried to connect to a gateway
  unsigned long       gw_incoming;             // Number of incoming transaction from a gateway
  unsigned long       gw_outgoing;             // Number of outgoing transactions to a gateway
  unsigned long       gw_alive;                // The number of gateway selftransactions performed
  unsigned long       gprs_connect;            // The number of times GPRS connection is established
  unsigned long       gprs_send;               // The number of Bytes send over GPRS
  unsigned long       gprs_receive;            // The number of Bytes received from GPRS
  unsigned long       gps_nofix;               // The number of No fix from GPS
  unsigned long       gps_2dfix;               // The number of 2D fix from GPS
  unsigned long       gps_3dfix;               // The number of 3D fix from GPS
};

#pragma pack(1)
struct rmonPersistEntry {
  unsigned char       type;                    // The type of entry, 1 = text entry, 2 = binary entry
  short               length;                  // The number of bytes in the buffer
  unsigned char       data[255];               // The contents of the persistent entry
};
#pragma pack()

struct rmonObjectInfo {
  char                name[255];               // The name of the object
  int                 type;                    // The type of object
  int                 size;                    // The size of the data in bytes
  unsigned char*      data;                    // Pointer to buffer with the data
  unsigned long       flags;                   // Flags for object, reserved for future use. set to zero
  int                 min_val;                 // The lower limit for integer values.
  int                 max_val;                 // The upper limit for integer values, maximum length for string and data.
  char                desc[81];                // Desription of the object
};

struct rmonObjectInfoX {
  char                name[255];               // The name of the object
  int                 type;                    // The type of object
  int                 size;                    // The size of the data in bytes
  unsigned char*      data;                    // Pointer to buffer with the data
  unsigned long       flags;                   // Flags for object, reserved for future use. set to zero
  int                 size_min;                // The size of the lower limit buffer.
  unsigned char*      min_val;                 // Pointer to buffer with the lower limit for integer values.
  int                 size_max;                // The size of the upper limit buffer.
  unsigned char*      max_val;                 // Pointer to buffer with the upper limit for integer values, maximum length for string and data.
  char                desc[81];                // Desription of the object
};

struct rmon_power_info_t {
  short                      sof;              // The size of the structure
  short                      temp;             // The temperature
  unsigned short             sup_volt;         // The power supply voltage
  unsigned char              sup_type;         // The power supply type
  unsigned char              batt_type;        // The battery type
  unsigned char              batt_level;       // The battery power level
  unsigned char              batt_charging;    // The battery is charging (Yes/No)
  unsigned char              chg_enable;       // The charger is enabled (Yes/No)
  unsigned char              run_on_batt;      // The device will continue to run from battery (Yes/No
  char                       ext_disable;      // The external power supply is disabled (1 = Yes, 0 = No, -1 = Not Available)
  char                       s0_enable;        // S0 input is enabled (1 = Yes, 0 = No, -1 = Not Available)
  char                       dcout1_enable;    // DC-OUT is enabled  (1 = Yes, 0 = No, -1 = Not Available)
  char                       dcout2_enable;    // DC_OUT2 is enabled (1 = Yes, 0 = No, -1 = Not Available)
  char                       lcd_power;        // The display power is enabled  (1 = Yes, 0 = No, -1 = Not Available)
  char                       usb_host1;        // The USB host1 power is enabled  (1 = Yes, 0 = No, -1 = Not Available)
  char                       usb_host2;        // The USB host2 power is enabled  (1 = Yes, 0 = No, -1 = Not Available)
};


/****************************************************************************
** 
**    EXPORTED VARIABLES
**
****************************************************************************/


/****************************************************************************
**
**    EXPORTED FUNCTIONS
**
****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// Open RTCUCSP library.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonError, rmonInit
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonOpen(void);

/////////////////////////////////////////////////////////////////////////////
// Close RTCUCSP library.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonNotInit
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonClose(void);

/////////////////////////////////////////////////////////////////////////////
// Set parameters for a gateway connection.
//
// Input:
//      Port               - The TCP/IP port of the gateway.
//      MyNodeID           - The Node ID of the RTCUCSP library when connected
//                           to the gateway. (0 indicates a dynamic id)
//      IP                 - The TCP/IP address of the gateway.
//      Key                - The logon key for the gateway.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonInit
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetGWParameters(
                                   const unsigned short Port, 
                                   const unsigned long  MyNodeID,
                                   const char*          IP,
                                   const char*          Key
                                  );

/////////////////////////////////////////////////////////////////////////////
// Set advanced parameters for a gateway connection.
//
// Input:
//      CryptKey                  - The key used to encrypt/decrypt data to/
//                                  from the gateway. If this is empty or all
//                                  16 bytes are 0 (zero) the standard key is
//                                  used.
//      gw_max_connection_attempt - The maximum number of tries to connect
//                                  to the gateway
//      gw_max_send_req_attempt   - The maximum number of send-request
//                                  attempts before send fails.
//                                  interval: 1..60 (default: 3)
//      gw_response_timeout       - The timeout, in seconds, before a
//                                  transaction failes.
//                                  interval: 5..60 (default: 30)
//      gw_alive_freq             - The time between keep alive messages to
//                                  the gateway in seconds.
//                                  interval: 0..60000. (default: 300). 
//                                  0 (zero) will disable self-trans.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonInit
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetGWParametersAdv(
                                      unsigned char  CryptKey[16],
                                      unsigned char  gw_max_connection_attempt,
                                      unsigned char  gw_max_send_req_attempt,     
                                      unsigned short gw_response_timeout,
                                      unsigned short gw_alive_freq
                                     );

/////////////////////////////////////////////////////////////////////////////
// Set parameters for RTCU Communication Hub secure connection.
//
// Input:
//      server                     - The path to the Root CA certificate file
//                                   used to verify the server certificate.
//      cert                       - The client certificate to present to
//                                   the server.
//      key                        - The private key of the client.
//      password                   - The password of the private key.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonInit, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetRCHParametersSecure(
                                          char* server,
                                          char* cert,
                                          char* key,
                                          char* password
                                         );

/////////////////////////////////////////////////////////////////////////////
// Set maximum number of simultaneous sessions.
//
// Input:
//      max_sessions       - The maximum number of sessions. (Default: 100)
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonInit
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetMaxConnections(int max_sessions);

/////////////////////////////////////////////////////////////////////////////
// Retrieve the number of comports, and their names.
//
// Input:
//      None.
//
// Output:
//      name               - Pointer to a table of string where the names of
//                           the ports is stored.
//      size               - The number of ports stored in table.
//
// Returns:
//      rmonOK, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetPortList(char name[RMON_MAXPORTS][9], int* size);

/////////////////////////////////////////////////////////////////////////////
// Enumerate the serial port.
// The name and description of the serial ports are returned with the
// callback function.
//
// Input:
//      cbFunc             - The callback function.
//      arg                - An user defined argument for callback function.
//
// Output:
//      count              - The number of serial ports enumerated.
//
// Returns:
//      rmonOK
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonEnumeratePorts(rmoncbserialport cbFunc, void *arg, int *count);

/////////////////////////////////////////////////////////////////////////////
// Retrieve the version number of the RTCUCSP library
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//      version scaled by 100.
//
/////////////////////////////////////////////////////////////////////////////
int RMONCC rmonGetVer(void);

/////////////////////////////////////////////////////////////////////////////
// Open a connection.
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//      Handle to open connection.
//      HRMONCON_ILLEGAL if no connection is opened.
//
/////////////////////////////////////////////////////////////////////////////
HRMONCON RMONCC rmonOpenConnection(void);

/////////////////////////////////////////////////////////////////////////////
// Close a connection.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonCloseConnection(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Sets the comports used for direct and remote connection.
//
// Input:
//      hCon               - Handle for connection.
//      localPort          - ASCIIZ string with the name of the port to use.
//      remotePort         - ASCIIZ string with the name of the port to use.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonConnection, rmonIllegalHandle, rmonOnlyGateway,
//      rmonNoCable, rmonNoModem
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetComport(
                              HRMONCON    hCon,
                              const char* localPort,
                              const char* remotePort
                             );

/////////////////////////////////////////////////////////////////////////////
// Sets the init string used for remote connection.
//
// Input:
//      hCon               - Handle for connection.
//      atcmd              - ASCIIZ string with AT commands to init. modem.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonConnection, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetModemInit(HRMONCON hCon, const char* atcmd);

/////////////////////////////////////////////////////////////////////////////
// Sets the baud used for remote connection.
//
// Input:
//      hCon               - Handle for connection.
//      baud               - The baud rate used over the modem.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonConnection, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetRemoteBaudrate(HRMONCON hCon, int baud);

/////////////////////////////////////////////////////////////////////////////
// Controls the use of large data frames on medias that supports them.
//
// Input:
//      hCon               - Handle for connection.
//      enable             - 0=Disable large data blocks,
//                           1=Enable large data blocks.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonEnableLargeData(HRMONCON hCon, unsigned char enable);

/////////////////////////////////////////////////////////////////////////////
// This function is used to establish a connection to a RTCU unit.
// The connection can be either over cable, thru a modem, or thru a gateway.
//
// If the remote RTCU is to be contacted thru a modem, simply use the
// telephone number of the SIM card in the RTCU, if connection is thru a
// gateway, then the units serial number (nodeid) is to be used, prefixed
// with a "@" character! To establish a cabel connection leave the
// phonenumber empty.
//
// Input:
//      hCon               - Handle for connection.
//      phonenumber        - ASCIIZ string that contains the unit contact info.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonConnection, rmonDenied, rmonComError,
//      rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonConnect(HRMONCON hCon, const char* phonenumber);

/////////////////////////////////////////////////////////////////////////////
// Disconnect connection, local and remote.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonError
//      
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDisconnect(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Determine the type of connection that is currently (if any) established
// with the RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      RMONCON_NONE       - Not connected.
//      RMONCON_LOCAL      - Connected over a cable.
//      RMONCON_REMOTE     - Connected over a datacall.
//      RMONCON_GW         - Connected over a gateway.
//
/////////////////////////////////////////////////////////////////////////////
int RMONCC rmonConnected(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Determine the type of cable connection currently established
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      type               - The type of connection.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonConnectionType(HRMONCON hCon, int* type);

/////////////////////////////////////////////////////////////////////////////
// Get communication statistics.
//
// Input:
//      hCon               - Handle for connection.
//      pStat              - Pointer to structure where the statistics is stored.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonConnection, rmonNotGateway, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetComStatistics(
                                    HRMONCON                  hCon,
                                    struct rmonComStatistics* pStat
                                   );

/////////////////////////////////////////////////////////////////////////////
// Reset communication statistics.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonConnection, rmonNotGateway
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonResetComStatistics(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// If there is established a (new) connection with a RTCU unit, either local
// or remote, the first thing to do, is for the application to authenticate
// itself for the RTCU unit.
// This is done using this function, and must be done, BEFORE any other
// communication with the unit can take place. If there is no password set
// in the RTCU unit, this function must still be called, just with a empty
// string "" as the password.
//
// Input:
//      hCon               - Handle for connection.
//      password           - ASCIIZ string with the password.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonIllegalTarget,
//      rmonTargetError, rmonDenied
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonAuthenticate(HRMONCON hCon, const char password[21]);

/////////////////////////////////////////////////////////////////////////////
// Fetches RTCU type and firmware version from the RTCU.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      targetID           - The ID number of the unit.
//      firmwareVer        - Firmware version scaled by 100.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetTargetInfo(
                                 HRMONCON hCon,
                                 int*     targetID,
                                 int*     firmwareVer  
                                );

/////////////////////////////////////////////////////////////////////////////
// Returns the serial number of the connected RTCU.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      SerialNumber       - The SerialNumber number of the unit.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonIllegalTarget
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetSerialNumber(
                                   HRMONCON       hCon,
                                   unsigned long* SerialNumber
                                  );

/////////////////////////////////////////////////////////////////////////////
// Fetches RTCU type, firmware version and serial number from the RTCU.
// This function does not require authentication.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      targetID           - The ID number of the unit.
//      firmwareVer        - Firmware version scaled by 100.
//      SerialNumber       - The SerialNumber number of the unit.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetTargetProfile(
                                    HRMONCON       hCon,
                                    int*           targetID,
                                    int*           firmwareVer,
                                    unsigned long* SerialNumber
                                   );

/////////////////////////////////////////////////////////////////////////////
// Fetches information of a device.
// This function does not require authentication.
//
// Input:
//   hCon                  - Handle for connection.
//
// Output:
//   targetID              - The ID number of the device.
//   VerRuntime            - Runtime version of the device.
//   VerSystem             - System version of the device.
//   SerialNumber          - The SerialNumber number of the device.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetDeviceInfo(
                                 HRMONCON       hCon,
                                 int*           targetID,
                                 int*           VerRuntime,
                                 int*           VerSystem,
                                 unsigned long* SerialNumber
                                );

/////////////////////////////////////////////////////////////////////////////
// Checks if the connected RTCU is programmable (a MAX unit) or not
// programmable (a MICRO unit)
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      Programmable       - 1 if unit is programmable (a MAX unit),
//                           0 (zero) if not programmable (a MICRO unit)
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonIllegalTarget
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonIsUnitProgrammable(
                                      HRMONCON       hCon,
                                      unsigned char* Programmable
                                     );

/////////////////////////////////////////////////////////////////////////////
// Returns the Firmware version of the connected RTCU.
// Note that this function must be used to determine if RTCU unit is in
// monitor mode.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      ver                - Firmware version scaled by 100.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonVer(HRMONCON hCon, int* ver);

/////////////////////////////////////////////////////////////////////////////
// This function will reset the connected RTCU unit. If the RTCU unit is
// remotely connected (modem or gateway) the reset will be delayed until
// the connection is lost (by calling rmonDisconnect() etc). However, if the
// connection is thru a direct cable connection, the RTCU unit executes the
// reset command immediately. The reset has the same effect as cycling power
// to the RTCU unit, the VPL program starts executing from the start again.
// Note: This function does the same as the VPL function boardReset().
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReset(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Stops the currently executing VPL program in the RTCU.
// The RTCU unit can be started again with the reset command or by cycling
// power.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonHalt(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Sets new password for access to RTCU.
// This is the password that is to be used in rmonAuthenticate().
// An empty string will disable password protection.
//
// Input:
//      hCon               - Handle for connection.
//      password           - Zero terminated ASCII string containing the new
//                           password.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetPassword(HRMONCON hCon, const char password[21]);

/////////////////////////////////////////////////////////////////////////////
// Checks if Debug messages has been enabled or disabled in unit.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      enabled            - 1 if Debug messages is enabled,
//                           0 (zero) if Debug messages is disabled.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetDebugEnable(HRMONCON hCon, int* enabled);

/////////////////////////////////////////////////////////////////////////////
// Enable or disable Debug messages from unit.
//
// Input:
//      hCon               - Handle for connection.
//      enabled            - 1 to enable Debug messages,
//                           0 (zero) to disable Debug messages.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetDebugEnable(HRMONCON hCon, int enable);

/////////////////////////////////////////////////////////////////////////////
// Receive any incoming Debug messages from RTCU.
// Please notice that rmonReceiveDebugMsg blocks and will not return before
// a debug message has been received.
//
// Input:
//      hCon               - Handle for connection.
//      maxsize            - Maximum number of characters to receive,
//
// Output:
//      msg                - Buffer with received debug message.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReceiveDebugMsg(HRMONCON hCon, char* msg, int maxsize);

/////////////////////////////////////////////////////////////////////////////
// Reads the real time clock on the RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      rtc                - Pointer to the structure where the RTC are
//                           stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetRTC(
                          HRMONCON            hCon,
                          struct rmonRTCTime* rtc
                         );

/////////////////////////////////////////////////////////////////////////////
// Sets the real time clock on the RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//      rtc                - Pointer to the structure where the new RTC are
//                           stored.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetRTC(
                          HRMONCON            hCon,
                          struct rmonRTCTime* rtc
                         );

/////////////////////////////////////////////////////////////////////////////
// Clears the fault log in the RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFaultLogClear(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Fetches fault log in the RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      Fault              - Pointer to the structure where the fault log is
//                           stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFaultLogRead(HRMONCON hCon, struct rmonFault* Fault);

/////////////////////////////////////////////////////////////////////////////
// Fetches fault log in the RTCU unit, with enhanced information.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      Fault              - Pointer to the structure where the fault log is
//                           stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFaultLogReadX(HRMONCON hCon, struct rmonFaultX* Fault);

/////////////////////////////////////////////////////////////////////////////
// Get the text messages for a fault.
//
// Input:
//      Fault              - The fault code.
//      bufsize            - Number of characters to read. If bufsize exceeds
//                           the number of characters in the fault text only
//                           the number of characters present will be put in
//                           the output buffer.
//
// Output:
//      FaultText          - The buffer where the fault string is stored.
//
// Returns:
//      rmonOK
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFaultGetText(
                                unsigned char fault,
                                char* FaultText,
                                int bufsize
                               );

/////////////////////////////////////////////////////////////////////////////
// Upgrades the RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//      string             - Zero terminated ASCII string containing the
//                           upgrade key.
//
// Output:
//      res                - The type of upgrade performed.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError,
//      rmonIllegalTarget
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSoftwareUpgrade(HRMONCON hCon, char string[35], int* res);

/////////////////////////////////////////////////////////////////////////////
// Enables a Flex Option in unit.
//
// Input:
//      hCon               - Handle for connection.
//      string             - Zero terminated ASCII string containing the
//                           flex option key.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFlexOption(HRMONCON hCon, char string[11]);

/////////////////////////////////////////////////////////////////////////////
// Applies an option key to a RTCU device.
//
// Input:
//   hCon                  - Handle for connection.
//   string                - Zero terminated ASCII string containing the
//                           option key.
//
// Output:
//   None.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonOptionApply(HRMONCON hCon, char* string);

/////////////////////////////////////////////////////////////////////////////
// This function read the from the memory I/O system in the RTCU. It is
// possible to indicate what type of data is stored at each location read;
// this is to help the function minimizing communication traffic.
//
// Input:
//      hCon               - Handle for connection.
//      location           - This is the start location to read from, 0 based.
//      count              - Number of memory locations to read.
//      type               - The type of data to read from each location.
//                           1=BOOL, 2=SINT, 3=INT, 4=DINT.
//
// Output:
//      data               - Data read from the RTCU will be stored in this
//                           buffer (must be count number long, and of the
//                           same type at type specifies. (BOOL and SINT is
//                           1 byte, INT is 2 bytes and DINT is 4 bytes)
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReadIOMemory(
                                HRMONCON hCon,
                                int      location,
                                int      count,
                                int      type,
                                void*    data
                               );

/////////////////////////////////////////////////////////////////////////////
// This function writes to the memory I/O system in the RTCU. It is possible
// to indicate what type of data is to be stored at each location; this is
// to help the function minimizing communication traffic. Please note that
// if the location(s) written to, is also used as VAR_OUTPUT variables by
// the VPL program in the RTCU unit, the RTCU and this function will both
// write to the same location, in which case the writing done by this
// function, will be overwritten by the RTCU unit itself (all I/O configured
// variables will be updated in each scan of the VPL program in the RTCU
// unit).
//
// Input:
//      hCon               - Handle for connection.
//      location           - This is the start location to write to, 0 based.
//      count              - Number of memory locations to write.
//      type               - The type of data to write to each location.
//                           1=BOOL, 2=SINT, 3=INT, 4=DINT.
//      data               - Data written to the RTCU is taken from this
//                           buffer (must be count number long, and of the
//                           same type at type specifies. (BOOL and SINT is
//                           1 byte, INT is 2 bytes and DINT is 4 bytes)
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonWriteIOMemory(
                                 HRMONCON hCon,
                                 int      location,
                                 int      count,
                                 int      type,
                                 void*    data
                                );

/////////////////////////////////////////////////////////////////////////////
// This function is used to read the state of the physical in- and output
// signals in the RTCU unit. The iotype indicates which input/output you
// are reading from, and ioindex indicates which input- or output number
// you are reading.
//
// Input:
//      hCon               - Handle for connection.
//      iotype             - Select the type of I/O system you want to read from,
//      ioindex            - Valid index of IO to get value from. Starts with index 0
//
// Output:
//      value              - Input or output value.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetIOState(
                              HRMONCON hCon,
                              int      iotype,
                              int      ioindex,
                              int*     value
                             );

/////////////////////////////////////////////////////////////////////////////
// This function is used to set the status of the physical output signals in
// the RTCU unit. The iotype indicated which Output you are writing to, and
// ioindex indicates which output number you are writing to. Please note
// that if the output written to, is also used as a VAR_OUTPUT variable by
// the VPL program in the RTCU unit, the RTCU and this function will both
// write to the same output, in which case the writing done by this function
// will be overwritten by the RTCU unit itself. (all I/O configured variables
// will be updated in each scan of the VPL program in the RTCU unit)
//
// Input:
//      hCon       - Handle for connection.
//      iotype     - Select the type of I/O system you want to write to.
//      ioindex    - Valid index of IO to set value for. Starts with index 0.
//      value      - Value to set.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetIOState(
                              HRMONCON hCon,
                              int      iotype,
                              int      ioindex,
                              int      value
                             );

/////////////////////////////////////////////////////////////////////////////
// This function is used to read the number of in- and output signals in the
// RTCU unit.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      data               - Structure containing the number of in- and
//                           output signals.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetIOCount(HRMONCON hCon, struct rmonIOCount *data);

/////////////////////////////////////////////////////////////////////////////
// Get the IMEI number from the GSM modem.
//
// Input:
//      hCon               - Handle for connection.
//      bufsize            - Number of characters to read. If bufsize exceeds
//                           the number of characters in the IMEI only the
//                           number of characters present will be put in the
//                           output buffer.
//
// Output:
//      IMEInumber         - The buffer where the IMEI string is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetIMEI(HRMONCON hCon, char *IMEInumber, int bufsize);

/////////////////////////////////////////////////////////////////////////////
// Get the IMSI number from the SIM card.
//
// Input:
//      hCon               - Handle for connection.
//      bufsize            - Number of characters to read. If bufsize exceeds
//                           the number of characters in the IMSI only the
//                           number of characters present will be put in the
//                           output buffer.
//
// Output:
//      IMSInumber         - The buffer where the IMSI string is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetIMSI(HRMONCON hCon, char *IMSInumber, int bufsize);

/////////////////////////////////////////////////////////////////////////////
// Get the ICCID number from the SIM card.
//
// Input:
//      hCon               - Handle for connection.
//      bufsize            - Number of characters to read. If bufsize exceeds
//                           the number of characters in the ICCID only the
//                           number of characters present will be put in the
//                           output buffer.
//
// Output:
//      ICCIDnumber        - The buffer where the ICCID string is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetICCID(HRMONCON hCon, char *ICCIDnumber, int bufsize);

/////////////////////////////////////////////////////////////////////////////
// Send "fake" SMS messages to the RTCU unit. The RTCU will receive SMS
// messages with the gsmIncomingSMS() / gsmIncomingPDU().
// The phonenumber variable in the gsmIncomingSMS() / gsmIncomingPDU() will
// indicate "9999" as the originator of the message.
//
// Input:
//      hCon               - Handle for connection.
//      smstype            - Type of SMS message to send. RMONSMS_TEXT = SMS,
//                           RMONSMS_BINARY = PDU.
//      messageLength      - Number of bytes to send. Not used when smstype
//                           is RMONSMS_TEXT.
//      message            - Pointer to buffer containing data to send.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSendSMS(
                           HRMONCON    hCon,           
                           int         smstype,
                           int         messageLength,
                           const char* message
                          );

/////////////////////////////////////////////////////////////////////////////
// Receive the SMS message the connected RTCU sends to phonenumber "9999"
// using the either gsmSendSMS() or gsmSendPDU().
// Please note that this function is blocking, it will first return when a
// message is received.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      smstype            - Type of SMS message received.
//                           RMONSMS_TEXT = SMS, RMONSMS_BINARY = PDU.
//      dataLength         - Number of bytes in PDU. Not used when smstype
//                           is RMONSMS_TEXT.
//      data               - Pointer to buffer where received data is stored.
//
// Returns:
//      rmonOK, rmonIllegalHandle, rmonTargetError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReceiveSMS(
                              HRMONCON hCon,
                              int*     smstype,
                              int*     dataLength,
                              char*    data
                             );

/////////////////////////////////////////////////////////////////////////////
// Enable or disable reception of the "fake" SMS messages from the RTCU unit.
// Note, if disabling, the rmonReceiveSMS() will still block, waiting for a
// message to arrive.
//
// Input:
//      hCon               - Handle for connection.
//      enable             - 0 (zero) = disable, 1 = enable.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReceiveSMSEnable(HRMONCON hCon, int enable);

/////////////////////////////////////////////////////////////////////////////
// Gets the GSM signal strength
// Note: This function does the same as the VPL function gsmSignalLevel().
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      signal             - The GSM signal strength or 0 (zero) if not connected.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetGSMSignalLevel(HRMONCON hCon, int* signal);

/////////////////////////////////////////////////////////////////////////////
// Get the version of the GSM module.
//
// Input:
//      hCon               - Handle for connection.
//      bufsize            - Number of characters to read. If bufsize exceeds
//                           the number of characters in the GSM version, only
//                           the number of characters present will be put in
//                           the output buffer.
//
// Output:
//      version            - The buffer where the version string is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetGSMVersion(HRMONCON hCon, char *version, int bufsize);

/////////////////////////////////////////////////////////////////////////////
// Gets the list of allowed phone numbers that can make incoming data calls
// to the RTCU. Phone numbers must be separated with the "," character.
// Note: This function does the same as the VPL function
// gsmGetListOfCallers().
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      numbers            - Zero terminated ASCII string containing the list
//                           of phone numbers.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetAllowedCallerList(HRMONCON hCon, char numbers[81]);

/////////////////////////////////////////////////////////////////////////////
// Sets the list of allowed phone numbers that can make incoming data calls
// to the RTCU. Phone numbers must be separated with the "," character.
// Note: This function does the same as the VPL function
// gsmSetListOfCallers().
//
// Input:
//      hCon               - Handle for connection.
//      numbers            - Zero terminated ASCII string containing the list
//                           of phone numbers.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetAllowedCallerList(HRMONCON hCon, const char numbers[81]);

/////////////////////////////////////////////////////////////////////////////
// Fetches the GSM SIM PIN code from the RTCU.
// An empty string denotes that the PIN code has been disabled.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      pin                - Zero terminated ASCII string containing the SIM
//                           PIN code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetGSMPIN(HRMONCON hCon, char pin[5]);

/////////////////////////////////////////////////////////////////////////////
// Sets the SIM PIN code to use for the SIM card in the RTCU unit.
// This does NOT change the PIN code on the SIM card, it simply tells the
// RTCU unit which PIN code to use when powering up the GSM module!
// An empty string will disable use of PIN code (SIM PIN code must be
// disabled on the SIM card, use a normal mobile telephone for doing this)
// Specifying a wrong GSM pin code will cause a RTCU fault.
// If the RTCU is restarted more than 3 times with the wrong SIM pin code,
// the SIM card will be locked, and it must be unlocked in a normal mobile
// phone, using the GSM operator supplied PUK code!
// Note: This function does the same as the VPL function gsmSetPin().
//
// Input:
//      hCon               - Handle for connection.
//      pin                - Zero terminated ASCII string containing the SIM
//                           PIN code
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetGSMPIN(HRMONCON hCon, const char pin[5]);

/////////////////////////////////////////////////////////////////////////////
// Fetches the TCP/IP settings the unit uses to connect over GPRS.
// All the general TCP/IP parameters use a binary packed IP address (a.b.c.d)
// using this format:
//   a     bit 24..31
//   b     bit 16..23
//   c     bit 8..15
//   d     bit 0..7
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      Settings           - Pointer to the structure where the GPRS
//                           settings are stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetGPRSSettings(
                                   HRMONCON                 hCon,
                                   struct rmonGPRSSettings* Settings
                                  );

/////////////////////////////////////////////////////////////////////////////
// Sets the TCP/IP settings the unit uses to connect over GPRS.
// All the general TCP/IP parameters use a binary packed IP address (a.b.c.d)
// using this format:
//   a     bit 24..31
//   b     bit 16..23
//   c     bit 8..15
//   d     bit 0..7
//
// Input:
//      hCon               - Handle for connection.
//      Settings           - The structure containing the new GPRS settings.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetGPRSSettings(
                                   HRMONCON                hCon,
                                   struct rmonGPRSSettings Settings
                                  );

/////////////////////////////////////////////////////////////////////////////
// Fetches the settings the unit uses to connect to the gateway.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      Settings           - Pointer to the structure where the gateway
//                           settings are stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetGatewaySettings(
                                      HRMONCON               hCon,
                                      struct rmonGWSettings* Settings
                                     );

/////////////////////////////////////////////////////////////////////////////
// Sets the settings the unit uses to connect to a gateway.
//
// Input:
//      hCon               - Handle for connection.
//      Settings           - The structure containing the new gateway settings.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetGatewaySettings(
                                      HRMONCON hCon,
                                      struct rmonGWSettings Settings
                                     );

/////////////////////////////////////////////////////////////////////////////
// Fetches a connection setup from the device.
//
// Input:
//      hCon               - Handle for connection.
//      index              - The index of the connection setup.
//
// Output:
//      Settings           - Pointer to the structure where the setup is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonRCHGetConfig(
                                HRMONCON             hCon,
                                const int            index,
                                struct rmonRCHConfig *Settings
                               );

/////////////////////////////////////////////////////////////////////////////
// Sets a connection setup in the device.
//
// Input:
//      hCon               - Handle for connection.
//      index              - The index of the connection setup.
//      Settings           - The structure with the connection setup.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonRCHSetConfig(
                                HRMONCON             hCon,
                                const int            index,
                                struct rmonRCHConfig *Settings
                               );

/////////////////////////////////////////////////////////////////////////////
// Fetches whether RTCU Communication Hub connection starts automatically
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      enable             - The auto connect state.
//
// Returns:
//      rmonOK, rmonComError, rmonError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonRCHGetAutoConnect(HRMONCON hCon, int* enable);

/////////////////////////////////////////////////////////////////////////////
// Sets whether RTCU Communication Hub connection starts automatically
//
// Input:
//      hCon               - Handle for connection.
//      enable             - The auto connect state.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonError, rmonIllegalHandle
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonRCHSetAutoConnect(HRMONCON hCon, const int enable);

/////////////////////////////////////////////////////////////////////////////
// This function reads a specific entry from FRAM based Persistent memory in 
// the RTCU. When called, you must specify what type of data you are exspecting 
// to read, if the specified type of data is not present, the function returns
// rmonNoData, otherwise the data and length are returned.
// Data read with this function, can be stored from VPL with the functions
// SaveStringF() and SaveDataF()
//
// Input:
//      hCon               - Handle for connection.
//      entry              - 1 based index of entry
//      binary             - 0 = Text entry, 1 = Binary entry
//
// Output:
//      data               - Buffer
//      length             - Length of data read from unit
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData,
//      rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReadPersistentFRAM(
                                      HRMONCON hCon,
                                      int      entry,
                                      char*    data,
                                      int*     length,
                                      int      binary
                                     );

/////////////////////////////////////////////////////////////////////////////
// This function writes either binary data or a string to a specific entry in 
// the FRAM based Persistent memory in the RTCU. When called, you must specify 
// what type of data you are storing. 
// Data stored with this function, can be read from VPL with the functions
// LoadStringF() and LoadDataF()
//
// Input:
//      hCon               - Handle for connection.
//      entry              - 1 based index of entry
//      binary             - 0 = Text entry, 1 = Binary entry
//      length             - Length of data to write to unit
//      data               - Buffer
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonWritePersistentFRAM(
                                       HRMONCON    hCon,
                                       int         entry,
                                       const char* data,
                                       int         length,
                                       int         binary
                                      );

/////////////////////////////////////////////////////////////////////////////
// This function reads a specific entry from FLASH based Persistent memory in 
// the RTCU. When called, you must specify what type of data you are exspecting 
// to read, if the specified type of data is not present, the function returns
// rmonNoData, otherwise the data and length are returned.
// Data read with this function, can be stored from VPL with the functions
// SaveString() and SaveData()
//
// Input:
//      hCon               - Handle for connection.
//      entry              - 1 based index of entry
//      binary             - 0 = Text entry, 1 = Binary entry
//
// Output:
//      data               - Buffer
//      length             - Length of data read from unit
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData,
//      rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReadPersistentFLASH(
                                       HRMONCON hCon,
                                       int      entry,
                                       char*    data,
                                       int*     length,
                                       int      binary
                                      );

/////////////////////////////////////////////////////////////////////////////
// This function writes either binary data or a string to a specific entry in 
// the FLASH based Persistent memory in the RTCU. When called, you must specify 
// what type of data you are storing. 
// Data stored with this function, can be read from VPL with the functions
// LoadString() and LoadData()
//
// Input:
//      hCon               - Handle for connection.
//      entry              - 1 based index of entry
//      binary             - 0 = Text entry, 1 = Binary entry
//      length             - Length of data to write to unit
//      data               - Buffer
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonWritePersistentFLASH(
                                        HRMONCON    hCon,
                                        int         entry,
                                        const char* data,
                                        int         length,
                                        int         binary
                                       );

/////////////////////////////////////////////////////////////////////////////
// This function writes either binary data or a string to a specific entry in 
// the extended FLASH based Persistent memory in the RTCU. When called, you
// must specify what type of data you are storing.
// Data stored with this function, can be read from VPL with the functions
// LoadStringX() and LoadDataX()
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      size               - Number of entries in extended flash
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetXFLASHSize(HRMONCON hCon, int *size);

/////////////////////////////////////////////////////////////////////////////
// This function reads a specific entry from extended FLASH based Persistent
// memory in the RTCU. When called, you must specify what type of data you
// are exspecting to read, if the specified type of data is not present, the
// function returns rmonNoData, otherwise the data and length are returned.
// Data read with this function, can be stored from VPL with the functions
// SaveStringX() and SaveDataX()
//
// Input:
//      hCon               - Handle for connection.
//      entry              - 1 based index of entry
//      binary             - 0 = Text entry, 1 = Binary entry
//
// Output:
//      data               - Buffer
//      length             - Length of data read from unit
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError. rmonError,
//      rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonReadPersistentXFLASH(
                                        HRMONCON hCon,
                                        int      entry,
                                        char*    data,
                                        int*     length,
                                        int      binary
                                       );

/////////////////////////////////////////////////////////////////////////////
// This function writes either binary data or a string to a specific entry in 
// the extended FLASH based Persistent memory in the RTCU. When called, you
// must specify what type of data you are storing.
// Data stored with this function, can be read from VPL with the functions
// LoadStringX() and LoadDataX()
//
// Input:
//      hCon               - Handle for connection.
//      entry              - 1 based index of entry
//      binary             - 0 = Text entry, 1 = Binary entry
//      length             - Length of data to write to unit
//      data               - Buffer
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError. rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonWritePersistentXFLASH(
                                         HRMONCON    hCon,
                                         int         entry,
                                         const char* data,
                                         int         length,
                                         int         binary
                                        );

/////////////////////////////////////////////////////////////////////////////
// Reads a range of entries from Persistent memory with bounds check.
// The buffer must be an array of struct rmonPersistEntry.
// Where the number of entrie in the table must be identical to the number
// of entries to read
// The type of each entry can be one of RMON_PERSIST_TEXT and RMON_PERSIST_BINARY
//
// Input:
//      hCon               - Handle for connection.
//      first              - 1 based index of firsat entry to read
//      last               - 1 based index of last entry to read
//      type               - RMON_TYPE_FRAM, RMON_TYPE_FLASH or RMON_TYPE_XFLASH
//      pfunc              - Pointer to progress callback function.
//      uptr               - Pointer to user argument used when reporting
//                           progress.
//      reserved           - Reserved for future use. Must be set to zero
//
// Output:
//      data               - Buffer where persistent entries are stored
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonError, rmonCancelled,
//      rmonIllegalAccess
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonPersistentRead(
                                  HRMONCON       hCon,
                                  int            first,
                                  int            last,
                                  int            type,
                                  char*          data,
                                  rmoncbprogress pfunc,
                                  void*          uptr,
                                  int            reserved
                                 );

/////////////////////////////////////////////////////////////////////////////
// Writes a range of entries from Persistent memory with bounds check.
// The buffer must be an array of struct rmonPersistEntry.
// Where the number of entrie in the table must be identical to the number
// of entries to write
// The type of each entry can be one of RMON_PERSIST_TEXT and RMON_PERSIST_BINARY
//
// Input:
//      hCon               - Handle for connection.
//      first              - 1 based index of firsat entry to read
//      last               - 1 based index of last entry to read
//      type               - RMON_TYPE_FRAM, RMON_TYPE_FLASH or RMON_TYPE_XFLASH
//      data               - Buffer where persistent entries are stored
//      pfunc              - Pointer to progress callback function.
//      uptr               - Pointer to user argument used when reporting
//                           progress.
//      reserved           - Reserved for future use. Must be set to zero
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonError, rmonCancelled,
//      rmonIllegalAccess
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonPersistentWrite(
                                   HRMONCON       hCon,
                                   int            first,
                                   int            last,
                                   int            type,
                                   const char*    data,
                                   rmoncbprogress pfunc,
                                   void*          uptr,
                                   int            reserved
                                  );

/////////////////////////////////////////////////////////////////////////////
// This function returns information about how many values (up to 8) are
// stored at each record in the datalogger. (is configure via the VPL program
// in the RTCU unit)
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      numberofvalues     - Number of values in record, maximum 8.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogGetValuesPerRecord(HRMONCON hCon, int* numberofvalues);

/////////////////////////////////////////////////////////////////////////////
// Clears data in the RTCU datalogger. The current datastructure in the RTCU
// datalogger is maintained.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNotInit
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogClear(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Moves the current read pointer to the first (oldest) entry in the
// datalogger.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogFirst(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// Moves the current read pointer to the last (newest) entry in the
// datalogger.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogLast(HRMONCON hCon);

/////////////////////////////////////////////////////////////////////////////
// rmonLogGotoLinsec will search for an entry in the datalogger, that
// matches the specified timestamp, and if no match is found, it will select
// the nearest record (if any). It is possible to specify the search
// direction as either forward or backward.
//
// Input:
//      hCon               - Handle for connection.
//      timestamp          - Complete time of record to search for.
//      direction          -   0 (zero) means backwards search,
//                           <>0 (zero) means forward search
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogGotoLinsec(
                                 HRMONCON           hCon,
                                 struct rmonRTCTime timestamp,
                                 unsigned char      direction
                                );

/////////////////////////////////////////////////////////////////////////////
// rmonLogSeek will search for an entry in the datalogger, that matches the
// specified tag, and move n records from there. The n parameter determines
// the direction of the search.
//
// Input:
//      hCon               - Handle for connection.
//      tag                - The tag of record to search for.
//      n                  - The number of entries to move after finding tag.
//                           > zero - Seek forward, = zero - No effect,
//                           < zero - Seek Backwards.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogSeek(HRMONCON hCon, short tag, short n);

/////////////////////////////////////////////////////////////////////////////
// This function reads up to 21 log entries from the datalogger each time it
// is called. When it is called, the current read pointer is either
// incremented or decremented, according to the operation parameter.
//
// Input:
//      hCon               - Handle for connection.
//      operation          - RMONLOGGET_NEXT to increase read pointer.
//                           RMONLOGGET_PREV to decrease read pointer.
//
// Output:
//      values_per_rec     - Number of entries in tdefExtLog logvalues
//                           (array length), maximum 8.
//      entries_in_buffer  - Number of tdefExtLog records in buffer,
//                           maximum 21.
//      buffer             - Buffer where the read entries are stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData,
//      rmonNoMoreData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogReadExt(
                              HRMONCON hCon,
                              int      operation,
                              int*     values_per_rec,
                              int*     entries_in_buffer,
                              char*    buffer
                             );

/////////////////////////////////////////////////////////////////////////////
// This function reads up to 21 log entries from the datalogger each time it
// is called. When it is called, the current read pointer is either
// incremented or decremented, according to the operation parameter.
//
// Input:
//      hCon               - Handle for connection.
//      operation          - RMONLOGGET_NEXT to increase read pointer.
//                           RMONLOGGET_PREV to decrease read pointer.
//      tag                - The tag that is used to filter datalog entries.
//
// Output:
//      values_per_rec     - Number of entries in tdefExtLog logvalues
//                           (array length), maximum 8.
//      entries_in_buffer  - Number of tdefExtLog records in buffer,
//                           maximum 21.
//      buffer             - Buffer where the read entries are stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData,
//      rmonNoMoreData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonLogReadByTag(
                                HRMONCON      hCon,
                                int           operation,
                                unsigned char tag,
                                int*          values_per_rec,
                                int*          entries_in_buffer,
                                char*         buffer
                               );

/////////////////////////////////////////////////////////////////////////////
// Get the application info from the unit.
//
// Input:
//      hCon      - Handle for connection.
//
// Output:
//      Appname   - The name of the application. 15 characters + zero terminator.
//      Appver    - The version of the application. scaled by 100.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetAppInfo(HRMONCON hCon, char *Appname, int *Appver);

/////////////////////////////////////////////////////////////////////////////
// Check for special conditions on a transfer
//
// Input:
//      hCon               - Handle for connection.
//      Type               - The type of transfer to check.
//                           1 = Application transfer, direct.
//                           2 = Application transfer, background.
//                           3 = Firmware transfer, direct.
//                           4 = Firmware transfer, background.
//                           5 = Voice transfer.
//      filename           - The name of the file to transfer.
//                           (Not used for voice transfer)
//
// Output:
//      Report             - Report on special conditions.
//                           0 (zero) = No conditions
//                           1 = Voice will be erased.
//                           2 = Requires firmware 3.00
//                           3 = Voice transfer not allowed.
//
// Returns:
//      rmonOK, rmonComError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonCheckTransfer(
                                 HRMONCON    hCon,
                                 int         type,
                                 const char* filename,
                                 int*        report
                                );

/////////////////////////////////////////////////////////////////////////////
// Determine which type of application file to use.
//
// Input:
//      ProjectFilename    - ASCIIZ string with the name of the project file.
//
// Output:
//      Filename           - Name of application file generated from project.
//
// Returns:
//      rmonOK, rmonFileNotFound, rmonIllegalFile
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonApplicationFilename(
                                       const char* ProjectFilename,
                                       char*       Filename
                                      );

/////////////////////////////////////////////////////////////////////////////
// Transfers a .VSX, a .PSX, or a .RPC file to a RTCU unit.
// Please note that the execution of the VPL program in the RTCU unit must be
// halted with rmonHalt(), before calling this function ! The RTCU will have
// to be reset after the transfer, to start the new application.
//
// Input:
//      hCon        - Handle for connection.
//      Filename    - Zero terminated string with the filename of the application.
//      cbfunc      - Pointer to progress callback function.
//      uptr        - User defined value that is returned in the progress
//                    callback function.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonErrorHalt,
//      rmonFileNotFound, rmonIllegalFile, rmonNotProgrammable, rmonCancelled,
//      rmonTargetError, rmonIllegalTarget, rmonImageTooLarge
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonApplicationUpload(
                                     HRMONCON       hCon,
                                     char*          Filename,
                                     rmoncbprogress cbfunc,
                                     void*          uptr
                                    );

/////////////////////////////////////////////////////////////////////////////
// Starts a background transfer of a .VSX, a .PSX, or a .RPC file to a RTCU unit.
// The upload started with this function supports resume if the upload is
// interrupted. If the upload is interrupted the Report structure will
// contain the information needed to resume the upload using
// rmonApplicationResumeUpload.
// The newly uploaded application is used after the unit has been reset.
// Note: On "older" units (Not X32) the voice memory in the unit is used and
// any voice data must be uploaded again.
// Use the function rmonVoiceMessagesAbove64K to determine if the use of this
// function will overwrite any voice data.
//
// Input:
//      hCon        - Handle for connection.
//      Filename    - Zero terminated string with the application filename
//      cbfunc      - Pointer to progress callback function.
//      uptr        - User defined value that is returned in the progress
//                    callback function.
//
// Output:
//      Report      - Background transfer progress report.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonCancelled, rmonNoBackground, rmonFileNotFound, rmonIllegalFile,
//      rmonNotProgrammable, rmonInterrupted, rmonIllegalTarget,
//      rmonImageTooLarge
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonApplicationStartUpload(
                                          HRMONCON             hCon,
                                          const char*          Filename,
                                          struct rmonBGReport* Report,
                                          rmoncbprogress       cbfunc,
                                          void*                uptr
                                         );

/////////////////////////////////////////////////////////////////////////////
// Resumes a background transfer of a .VSX, a .PSX, or a .RPC file to a RTCU unit.
// The upload started with this function supports resume if the upload is
// interrupted. If the upload is interrupted the Report structure will
// contain the information needed to resume.
// The newly uploaded application is used after the unit has been reset.
//
// Input:
//      hCon        - Handle for connection.
//      Filename    - Zero terminated string with the application filename
//      cbfunc      - Pointer to progress callback function.
//      uptr        - User defined value that is returned in the progress
//                    callback function.
//
// Output:
//      Report      - Background transfer progress report.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonCancelled, rmonNoBackground, rmonFileNotFound, rmonIllegalFile,
//      rmonNotProgrammable, rmonInterrupted, rmonIllegalTarget,
//      rmonImageTooLarge
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonApplicationResumeUpload(
                                           HRMONCON             hCon,
                                           const char*          Filename,
                                           struct rmonBGReport* Report,
                                           rmoncbprogress       cbfunc,
                                           void*                uptr
                                          );

/////////////////////////////////////////////////////////////////////////////
// Transfers a .BIN firmware file to a RTCU unit.
// Please note that the execution of the VPL program in the RTCU unit will be
// halted during the transfer, and that the RTCU unit will be reset when the
// transfer is completed.
//
// Input:
//      hCon               - Handle for connection.
//      FirmwareFilename   - Zero terminated string with the filename.
//      pfunc              - Pointer to progress callback function.
//      uptr               - Pointer to user argument used when reporting
//                           progress.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonIllegalTarget,
//      rmonNotGateway, rmonNoMonitorMode, rmonErrorReset, rmonCancelled,
//      rmonFileNotFound, rmonIllegalFile, rmonOldFormat, rmonTargetError,
//      rmonDowngrade
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFirmwareUpload(
                                  HRMONCON       hCon,
                                  const char*    FirmwareFilename,
                                  rmoncbprogress cbfunc,
                                  void*          uptr
                                 );

/////////////////////////////////////////////////////////////////////////////
// Starts a background transfer of a .BIN firmware file to a RTCU unit.
// The upload started with this function supports resume if the upload is
// interrupted. If the upload is interrupted the Report structure will
// contain the information needed to resume the upload using
// rmonFirmwareResumeUpload.
// The newly uploaded firmware is used after the unit has been reset.
// Note: On "older" units (Not X32) the voice memory in the unit is used and
// any voice data must be uploaded again.
// Use the function rmonVoiceMessagesAbove64K to determine if the use of this
// function will overwrite any voice data.
//
// Input:
//      hCon               - Handle for connection.
//      Filename           - Zero terminated string with the firmware filename.
//      pfunc              - Pointer to progress callback function.
//      uptr               - Pointer to user argument used when reporting
//                           progress.
//
// Output:
//      Report             - Background transfer progress report.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonCancelled, rmonNoBackground, rmonFileNotFound, rmonOldFormat,
//      rmonIllegalFile, rmonIllegalTarget, rmonInterrupted,
//      rmonDowngrade
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFirmwareStartUpload(
                                       HRMONCON             hCon,
                                       const char*          Filename,
                                       struct rmonBGReport* Report,
                                       rmoncbprogress       cbfunc,
                                       void*                uptr
                                      );

/////////////////////////////////////////////////////////////////////////////
// Resumes a background transfer of a .BIN firmware file to a RTCU unit.
// The upload started with this function supports resume if the upload is
// interrupted. If the upload is interrupted the Report structure will
// contain the information needed to resume.
// The newly uploaded firmware is used after the unit has been reset.
//
// Input:
//      hCon               - Handle for connection.
//      Filename           - Zero terminated string with the firmware filename.
//      pfunc              - Pointer to progress callback function.
//      uptr               - Pointer to user argument used when reporting
//                           progress.
//
// Output:
//      Report             - Background transfer progress report.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonCancelled, rmonNoBackground, rmonFileNotFound, rmonOldFormat,
//      rmonIllegalFile, rmonIllegalTarget, rmonInterrupted
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFirmwareResumeUpload(
                                        HRMONCON             hCon,
                                        const char*          Filename,
                                        struct rmonBGReport* Report,
                                        rmoncbprogress       cbfunc,
                                        void*                uptr
                                       );

/////////////////////////////////////////////////////////////////////////////
// Determine if voice messages are stored above 64k.
// This function is used to determine if voice messages will be overwritten
// by the rmonApplicationStartUpload and rmonFirmwareStartUpload functions.
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      above              - 1 if Voice messages above 64k,
//                           0 (zero) if no Voice messages above 64k.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonVoiceMessagesAbove64K(HRMONCON hCon, char* above);

/////////////////////////////////////////////////////////////////////////////
// Determine how many voice messages is included in a project.
//
// Input:
//      ProjectFilename    - ASCIIZ string with the name of the project file.
//
// Output:
//      NumFiles           - Number of voice file in .PRJ file.
//
// Returns:
//      rmonOK, rmonFileNotFound, rmonIllegalFile
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonNumOfVoiceMessages(const char* ProjectFilename, int* NumFiles);

/////////////////////////////////////////////////////////////////////////////
// Transfer voice messages to the unit.
//
// Input:
//      hCon               - Handle for connection.
//      ProjectFilename    - ASCIIZ string with the name of the project file.
//      cbfunc             - Pointer to progress callback function.
//      uptr               - User defined value that is returned in the
//                           progress callback function.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonCancelled, rmonFileNotFound, rmonIllegalFile, rmonMemoryConfig,
//      rmonImageTooLarge
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonVoiceUpload(
                               HRMONCON       hCon,
                               const char*    ProjectFilename,
                               rmoncbprogress cbfunc,
                               void*          uptr
                              );

/////////////////////////////////////////////////////////////////////////////
// Transfer voice messages to the unit.
//
// Input:
//      hCon               - Handle for connection.
//      VoiceCount         - The number of voice files in array.
//      VoiceFilename      - Array of ASCIIZ strings with the name of the voice files.
//      cbfunc             - Pointer to progress callback function.
//      uptr               - User defined value that is returned in the
//                           progress callback function.
//
// Output:
//      None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError,
//      rmonCancelled, rmonFileNotFound, rmonIllegalFile, rmonImageTooLarge
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonVoiceUploadFiles(
                                    HRMONCON       hCon,
                                    int            VoiceCount,
                                    char*          VoiceFilename[],
                                    rmoncbprogress cbfunc,
                                    void*          uptr
                                   );

/////////////////////////////////////////////////////////////////////////////
// Query if media is present
//
// Input:
//      hCon               - Handle for connection.
//      media              - The media to query. (0 = SD-CARD)
//
// Output:
//      state              -  =0 (zero) Media is not present.
//                           <>0 (zero) Media is present.
//      fserr              - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaPresent(HRMONCON hCon, int media, int* state, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Query if media is write-protected
//
// Input:
//      hCon       - Handle for connection.
//      media      - The media to query. (0 = SD-CARD)
//
// Output:
//      state      -  =0 (zero) Media is not write-protected,
//                   <>0 (zero) Media is write-protected.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaWriteprotected(HRMONCON hCon, int media, int* state, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Open media.
//
// Input:
//      hCon       - Handle for connection.
//      media      - The media to open. (0 = SD-CARD)
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaOpen(HRMONCON hCon, int media, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Close media.
//
// Input:
//      hCon       - Handle for connection.
//      media      - The media to close. (0 = SD-CARD)
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaClose(HRMONCON hCon, int media, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Quick format the media.
//
// Input:
//      hCon       - Handle for connection.
//      media      - The media to format. (0 = SD-CARD)
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaQuickformat(HRMONCON hCon, int media, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Quick format the media.
//
// Input:
//      hCon       - Handle for connection.
//      media      - The media to format. (0 = SD-CARD)
//      flags      - The option flags.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaQuickformatX(
                                     HRMONCON hCon,
                                     int      media,
                                     int      flags,
                                     int      *fserr
                                    );

/////////////////////////////////////////////////////////////////////////////
// Eject the media.
//
// Input:
//      hCon       - Handle for connection.
//      media      - The media to eject. (0 = SD-CARD)
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaEject(HRMONCON hCon, int media, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Get information on the medias in the unit.
//
// Input:
//   hCon                  - Handle for connection.
//
// Output:
//   media                 - Structure containing info for all 8 supported medias.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaInformation(
                                    HRMONCON             hCon,
                                    struct rmonMediaInfo media[8]
                                   );

/////////////////////////////////////////////////////////////////////////////
// Get the size of a media.
//
// Input:
//   hCon                  - Handle for connection.
//   media                 - The index of the media.
//
// Output:
//   SizeLo                - Total size of the media, lower 32 bit.
//   SizeHi                - Total size of the media, upper 32 bit.
//   FreeLo                - Free size of the media, lower 32 bit.
//   FreeHi                - Free size of the media, upper 32 bit.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonMediaSize(
                             HRMONCON       hCon,
                             int            media,
                             unsigned long* SizeLo,
                             unsigned long* SizeHi,
                             unsigned long* FreeLo,
                             unsigned long* FreeHi
                            );

/////////////////////////////////////////////////////////////////////////////
// Enable/Disable filesystem status LED.
//
// Input:
//      hCon       - Handle for connection.
//      enable     -  =0 (zero) Disable status LED.
//                   <>0 (zero) Enable status LED.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFSStatusLED(HRMONCON hCon, int enable, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Create a directory.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the new directory.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDirCreate(HRMONCON hCon, const char name[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Change working directory.
//
// Input:
//      hCon       - Handle for connection.
//      path       - Name of the new working directory.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDirChange(HRMONCON hCon, const char path[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Get current working directory.
//
// Input:
//      hCon       - Handle for connection.
//
// Output:
//      path       - Name of the new working directory.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDirCurrent(HRMONCON hCon, char path[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Get directory item from working directory.
//
// Input:
//      hCon       - Handle for connection.
//      index      - The index of the directory item to retrieve.
//
// Output:
//      name       - Name of the directory item.
//      time       - The creation time. (Day parameter is un-used).
//      length     - The size in bytes.
//      fserr      - Filesystem error code.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDirCatalog(
                              HRMONCON            hCon,
                              short               index,
                              char                name[15],
                              struct rmonRTCTime* time,
                              long*               length,
                              int*                fserr
                             );

/////////////////////////////////////////////////////////////////////////////
// Get list of directory items from working directory.
//
// Input:
//   hCon          - Handle for connection.
//   wild          - The index of the directory item to retrieve.
//   pfunc         - Pointer to callback function which is called for each
//                   directory item.
//   uptr          - Pointer to user argument used in callback function.
//
// Output:
//   fserr         - Filesystem error code.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonError, rmonCancelled
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDirCatalogX(
                               HRMONCON       hCon,
                               char*          wild,
                               rmoncbdirentry pfunc,
                               void*          uptr,
                               int*           fserr
                              );

/////////////////////////////////////////////////////////////////////////////
// Delete directory.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the directory to delete.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonDirDelete(HRMONCON hCon, const char name[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Create new file.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the file to delete.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileCreate(HRMONCON hCon, const char name[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Open existing file.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the file to open.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileOpen(HRMONCON hCon, const char name[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Query if file exist.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the file to query.
//
// Output:
//      state      -  =0 (zero) File does not exist.
//                   <>0 (zero) File does exist.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileExists(
                              HRMONCON   hCon,
                              const char name[61],
                              char*      state,
                              int*       fserr
                             );

/////////////////////////////////////////////////////////////////////////////
// Rename file.
//
// Input:
//      hCon       - Handle for connection.
//      name_old   - Name of the file to rename.
//      name_new   - The new name.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileRename(
                              HRMONCON   hCon,
                              const char name_old[61],
                              const char name_new[13],
                              int*       fserr
                             );

/////////////////////////////////////////////////////////////////////////////
// Delete file.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the file to delete.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileDelete(HRMONCON hCon, const char name[61], int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Get the status of a file.
//
// Input:
//      hCon       - Handle for connection.
//
// Output:
//      status     - Status of the file. (Similar to fsFileStatus VPL function).
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileStatus(HRMONCON hCon, int* status, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Get the creation time and size of a file.
//
// Input:
//      hCon       - Handle for connection.
//      name       - Name of the file.
//
// Output:
//      time       - The creation time. (Day parameter is un-used).
//      length     - The size in bytes.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileGetInfo(
                               HRMONCON            hCon,
                               const char          name[61],
                               struct rmonRTCTime* time,
                               long*               length,
                               int*                fserr
                              );

/////////////////////////////////////////////////////////////////////////////
// Move the file pointer.
//
// Input:
//      hCon       - Handle for connection.
//      offset     - The new position relative to the Start of file.
//                   (<0 (zero) sets the file pointer to End of File).
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileSeek(HRMONCON hCon, long offset, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Retrieve the file pointer.
//
// Input:
//      hCon       - Handle for connection.
//
// Output:
//      position   - The file pointer position.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFilePosition(HRMONCON hCon, long* position, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Read a block of data from file.
//
// Input:
//      hCon       - Handle for connection.
//      elemcnt    - The number of bytes to read from file.
//      pfunc      - Pointer to function where progress is reported.
//      uptr       - Pointer to user argument used when reporting progress.
//
// Output:
//      buffer     - The buffer where the data read from the file is stored.
//      elemread   - The number of bytes read from file.
//      fserr      - Filesystem error code.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileRead(
                            HRMONCON       hCon,
                            int            elemcnt,
                            char*          buffer,
                            int*           elemread,
                            int*           fserr,
                            rmoncbprogress pfunc,
                            void*          uptr
                           );

/////////////////////////////////////////////////////////////////////////////
// Read a string from file.
//
// Input:
//      hCon       - Handle for connection.
//
// Output:
//      buffer     - The buffer where the data read from the file is stored.
//      elemread   - The number of bytes read from file.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileReadString(
                                  HRMONCON hCon,
                                  char     str[241],
                                  int*     elemread,
                                  int*     fserr
                                 );

/////////////////////////////////////////////////////////////////////////////
// Write a block of data to file.
//
// Input:
//      hCon       - Handle for connection.
//      elemcnt    - The number of bytes to write to file
//      buffer     - The buffer where the data to write is stored.
//      pfunc      - Pointer to function where progress is reported.
//      uptr       - Pointer to user argument used when reporting progress.
//
// Output:
//      elemwr     - The number of bytes written to file.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileWrite(
                             HRMONCON       hCon,
                             int            elemcnt,
                             char*          buffer,
                             int*           elemwr,
                             int*           fserr,
                             rmoncbprogress pfunc,
                             void*          uptr
                            );

/////////////////////////////////////////////////////////////////////////////
// Write a string to file.
//
// Input:
//      hCon       - Handle for connection.
//      str        - The string to write to file.
//
// Output:
//      elemwr     - The number of bytes written to file.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileWriteString(
                                   HRMONCON   hCon,
                                   const char str[241],
                                   int*       elemwr,
                                   int*       fserr
                                  );

/////////////////////////////////////////////////////////////////////////////
// Write a string to file. <CR><LF> are appended.
//
// Input:
//      hCon       - Handle for connection.
//      str        - The string to write to file.
//
// Output:
//      elemwr     - The number of bytes written to file.
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileWriteStringNL(
                                     HRMONCON   hCon,
                                     const char str[241],
                                     int*       elemwr,
                                     int*       fserr
                                    );

/////////////////////////////////////////////////////////////////////////////
// Close file.
//
// Input:
//      hCon       - Handle for connection.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileClose(HRMONCON hCon, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Flush cached write operations to media
//
// Input:
//      hCon       - Handle for connection.
//
// Output:
//      fserr      - Filesystem error code
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonFileFlush(HRMONCON hCon, int* fserr);

/////////////////////////////////////////////////////////////////////////////
// Read statistics from the RTCU unit.
// Please note that the size parameter of the rmonUnitStatistics structure
// must contain the size of the structure. (sizeof(struct rmonUnitStatistics))
//
// Input:
//      hCon               - Handle for connection.
//
// Output:
//      data               - Pointer to the block of data received.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonStatisticsRead(HRMONCON hCon, struct rmonUnitStatistics* data);

/////////////////////////////////////////////////////////////////////////////
// Read the execution state from RTCU unit
//
// Input:
//   hCon                  - Handle for connection.
//   pfunc                 - Pointer to function where unit state is reported.
//   uptr                  - User argument for callback function.
//
// Output:
//   None.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetUnitState(HRMONCON hCon, rmoncbunitstate pfunc, void* uptr);

/////////////////////////////////////////////////////////////////////////////
// Fetches the configuration for a LAN network interface.
//
// Input:
//   hCon                   - Handle for connection.
//   iface                  - The LAN network interface.
//
// Output:
//   Settings               - Pointer to the structure where the configuration
//                            is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetLANSettings(
                                  HRMONCON            hCon,
                                  int                 iface,
                                  struct rmonNetwork* Settings
                                 );

/////////////////////////////////////////////////////////////////////////////
// Writes the configuration for a LAN network interface.
//
// Input:
//   hCon                   - Handle for connection.
//   iface                  - The LAN network interface.
//   Settings               - Pointer to the configuration.
//
// Output:
//   None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetLANSettings(
                                  HRMONCON            hCon,
                                  int                 iface,
                                  struct rmonNetwork* Settings
                                 );

/////////////////////////////////////////////////////////////////////////////
// Fetches the configuration for the WiFi network interface.
//
// Input:
//   hCon                   - Handle for connection.
//
// Output:
//   Settings               - Pointer to the structure where the configuration
//                            is stored.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetWLANSettings(
                                   HRMONCON                 hCon,
                                   int                      index,
                                   struct rmonWLANSettings* Settings
                                  );

/////////////////////////////////////////////////////////////////////////////
// Writes the configuration for the WiFi network interface.
//
// Input:
//   hCon                   - Handle for connection.
//   Settings               - Pointer to the configuration.
//
// Output:
//   None.
//
// Returns:
//      rmonOK, rmonComError, rmonIllegalHandle, rmonTargetError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSetWLANSettings(
                                   HRMONCON                 hCon,
                                   int                      index,
                                   struct rmonWLANSettings* Settings
                                  );

/////////////////////////////////////////////////////////////////////////////
// Imports a security item to the device
//
// Input:
//   hCon                   - Handle for connection.
//   filename_cert          - The filename of the certificate to import.
//   filename_key           - The filename of the key to import.
//   name                   - The name of the item.
//   replace                - The item will be replaced if present.
//
// Output:
//   None.
//
// Returns:
//   rmonOK                 - Success
//   rmonIllegalHandle      - Illegal connection handle
//   rmonFileNotFound       - Cannot find the file
//   rmonIllegalFile        - File is not a valid certificate
//   rmonError              - Error transferring data
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSecurityImport(
                                  HRMONCON    hCon,
                                  const char* name,
                                  const char* filename_cert,
                                  const char* filename_key,
                                  const int   replace
                                 );

/////////////////////////////////////////////////////////////////////////////
// Removes a security item from the device
//
// Input:
//   hCon                   - Handle for connection.
//   name                   - The name of the certificate.
//
// Output:
//   None.
//
// Returns:
//   rmonOK                 - Success
//   rmonIllegalHandle      - Illegal connection handle
//   rmonComError           - Communication error
//   rmonError              - Error transferring data
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSecurityRemove(
                                  HRMONCON    hCon,
                                  const char* name
                                 );

/////////////////////////////////////////////////////////////////////////////
// Retrieves the information about a security item from the device
//
// Input:
//   hCon                   - Handle for connection.
//   name                   - The name of the certificate.
//   pfunc                  - Pointer to a callback function which is called
//                            for each certificate.
//   uptr                   - Pointer to user argument used in callback function.
//
// Output:
//   None.
//
// Returns:
//   rmonOK                 - Success
//   rmonIllegalHandle      - Illegal connection handle
//   rmonComError           - Communication error
//   rmonError              - Error transferring data
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonSecurityInfo(
                                HRMONCON            hCon,
                                const char*         name,
                                rmoncbcertificate   pfunc,
                                void*               uptr
                               );

/////////////////////////////////////////////////////////////////////////////
// Read Generic Object(s) from the device.
// Wildcards (? and _) can be used to read a range of objects
//
// Input:
//   hCon                  - Handle for connection.
//   table                 - The Object table. 1 = System, 2 = User
//   name                  - The name of the Object entry.
//   output                - The function called for each object read.
//   uptr                  - A user pointer which is included when calling output.
//
// Output:
//   None.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonObjectRead(
                              HRMONCON       hCon,
                              const int      table,
                              const char*    name,
                              rmoncbobject   output,
                              void*          uptr
                             );

/////////////////////////////////////////////////////////////////////////////
// Read Generic Object(s) from the device.
// Wildcards (* and ?) can be used to read a range of objects
//
// Input:
//   hCon                  - Handle for connection.
//   type                  - The type of Object. 1 = System, 2 = User
//   name                  - The name of the Object entry.
//   output                - The function called for each object read.
//   uptr                  - A user pointer which is included when calling output.
//
// Output:
//   None.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonObjectReadX(
                               HRMONCON       hCon,
                               const int      type,
                               const char*    name,
                               rmoncbobjectx  output,
                               void*          uptr
                              );

/////////////////////////////////////////////////////////////////////////////
// Write Generic Object(s) to the device
//
// Input:
//   hCon                  - Handle for connection.
//   table                 - The Object table. 1 = System, 2 = User
//   flags                 - Flags to control the write behaviour.
//   count                 - The number of objects to write.
//   data                  - Pointer to an array of objects.
//
// Output:
//   index                 - The index of the object which failed.
//                           (one based, zero if error is not from objects)
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonObjectWrite(
                               HRMONCON               hCon,
                               const int              table,
                               const short            flags,
                               const int              count,
                               struct rmonObjectInfo* data,
                               int*                   index
                              );

/////////////////////////////////////////////////////////////////////////////
// Write Generic Object(s) to the device
//
// Input:
//   hCon                  - Handle for connection.
//   type                  - The type of Object. 1 = System, 2 = User
//   count                 - The number of objects to write.
//   data                  - Pointer to an array of objects.
//
// Output:
//   index                 - The index of the object which failed.
//                           (one based, zero if error is not from objects)
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonError, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonObjectWriteX(
                                HRMONCON                hCon,
                                const int               type,
                                const short             flags,
                                const int               count,
                                struct rmonObjectInfoX* data,
                                int*                    index
                               );

/////////////////////////////////////////////////////////////////////////////
// Erase Generic Object from the device.
//
// Input:
//   hCon                  - Handle for connection.
//   table                 - The Object table. 1 = System, 2 = User
//   name                  - The name of the Object entry.
//
// Output:
//   None.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonError
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonObjectErase(
                               HRMONCON       hCon,
                               const int      table,
                               const char*    name
                              );

/////////////////////////////////////////////////////////////////////////////
// Get device power information.
//
// Input:
//   hCon                  - Handle for connection.
//
// Output:
//   data                  - Pointer to the data structure.
//
// Returns:
//   rmonOK, rmonComError, rmonIllegalHandle, rmonNoData
//
/////////////////////////////////////////////////////////////////////////////
rmonRet RMONCC rmonGetPowerInformation(
                                       HRMONCON                 hCon,
                                       struct rmon_power_info_t *data
                                      );


#ifdef __cplusplus
  }
#endif


#endif  /*_RTCUCSP_H_INCLUDED*/
/****************************************************************************
**
**    EOF 
**
****************************************************************************/
