// ConnectGwPageAdv.cpp : implementation file
//

#include "stdafx.h"
#include "rtcuprog.h"
#include "ConnectGwPageAdv.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConnectGwPageAdv property page

IMPLEMENT_DYNCREATE(CConnectGwPageAdv, CPropertyPage)

CConnectGwPageAdv::CConnectGwPageAdv() : CPropertyPage(CConnectGwPageAdv::IDD)
{
	// Var
	CString str, strList;
	int     i, pos;

	//{{AFX_DATA_INIT(CConnectGwPageAdv)
	m_gwmaxsend		= AfxGetApp()->GetProfileInt("REMOTE GPRS Options", "GW Maxsend", 3);
	m_gwresptimeout = AfxGetApp()->GetProfileInt("REMOTE GPRS Options", "GW Resptimeout", 30);
	m_cryptkeyenable = AfxGetApp()->GetProfileInt("GWConnect Options", "CRYPTKEY Enable", 0);
	//}}AFX_DATA_INIT

	// Get crypt key
	strList = AfxGetApp()->GetProfileString("GWConnect Options", "CRYPTKEY", "00000000-00000000-00000000-00000000");

	// Iterate cryptkey bytes
	pos = 0;
	for (i = 0; i < 16; i++)
	{
		// Insert byte
		m_crypt_key[i] = hatoi(strList.Mid(pos, 2));

		// Calculate next position
		pos += 2;
		if ((i + 1) % 4 == 0) pos++;
	}
}

CConnectGwPageAdv::~CConnectGwPageAdv()
{
}

void CConnectGwPageAdv::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConnectGwPageAdv)
	DDX_Control(pDX, IDC_SPINRESPTIMEOUT, m_gwspinresptimeout);
	DDX_Control(pDX, IDC_SPINMAXSEND, m_gwspinmaxsend);
	DDX_Control(pDX, IDC_CRYPT_KEY, c_cryptkey);
	DDX_Text(pDX, IDC_GWMAXSEND, m_gwmaxsend);
	DDX_Text(pDX, IDC_GWRESPTIMEOUT, m_gwresptimeout);
	DDX_Check(pDX, IDC_CRYPT_KEY_ENABLE, m_cryptkeyenable);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConnectGwPageAdv, CPropertyPage)
	//{{AFX_MSG_MAP(CConnectGwPageAdv)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_SETDEFAULT, OnSetdefault)
	ON_BN_CLICKED(IDC_CRYPT_KEY_ENABLE, OnCheckCryptKey)
	//}}AFX_MSG_MAP
	ON_REGISTERED_MESSAGE(UWM_CONNECT_PAGE, OnConnectPage)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CConnectGwPageAdv::OnInitDialog() 
{
	// Default
	CPropertyPage::OnInitDialog();

	// Set crypt key
	c_cryptkey.m_bpr = 8;
	c_cryptkey.SetOptions(false, true, false, false);
	c_cryptkey.SetData(m_crypt_key, 16);

	// Disable cryptkey?
	if (!m_cryptkeyenable) {
		c_cryptkey.EnableWindow(FALSE);
	}

	// Completed
	return TRUE;
}

void CConnectGwPageAdv::OnDestroy() 
{
	// Var
	CString str, strList;

	// Get changes
	UpdateData(true);

	// Get changed crypt key
	c_cryptkey.GetData(m_crypt_key, 16);

	// Iterate cryptkey bytes
	for (int i = 0; i < 16; i++)
	{
		// Insert byte
		str.Format("%02X", m_crypt_key[i]);
		strList += str;
		// Divider?
		if ((i + 1) % 4 == 0 && (i + 1) != 16) {
			// Insert divider
			strList += _T("-");
		}
	}

	// Save settings
	AfxGetApp()->WriteProfileString("GWConnect Options", "CRYPTKEY", strList);
	AfxGetApp()->WriteProfileInt("GWConnect Options", "CRYPTKEY Enable", m_cryptkeyenable);
	AfxGetApp()->WriteProfileInt("REMOTE GPRS Options", "GW Maxsend", m_gwmaxsend);
	AfxGetApp()->WriteProfileInt("REMOTE GPRS Options", "GW Resptimeout", m_gwresptimeout);

	// Default
	CPropertyPage::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// Button
/////////////////////////////////////////////////////////////////////////////

void CConnectGwPageAdv::OnSetdefault() 
{
	// Verify with user
	if (AfxMessageBox("Are you sure you want to set default values ?", MB_ICONWARNING|MB_YESNO|MB_DEFBUTTON2)==IDYES) {
		// Set default values
		m_gwmaxsend     = 3;
		m_gwresptimeout = 30;
		memset(m_crypt_key, 0, 16);

		// Update page
		UpdateData(false);
		c_cryptkey.SetData(m_crypt_key, 16);
	}
}

void CConnectGwPageAdv::OnCheckCryptKey()
{
  // Update check state
  UpdateData(true);

  // Is cryptkey enablet?
  if (m_cryptkeyenable) {
    // Enable HexEdit
    c_cryptkey.EnableWindow(TRUE);
  } else {
    // Disable HexEdit
    c_cryptkey.EnableWindow(FALSE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// Update
/////////////////////////////////////////////////////////////////////////////

LRESULT CConnectGwPageAdv::OnConnectPage(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	// Update settings
	UpdateData(true);
	if (m_cryptkeyenable) {
		c_cryptkey.GetData(m_crypt_key, 16);
	} else {
		memset(m_crypt_key, 0, 16);
	}

	// Disable cryptkey?
	if (!m_cryptkeyenable) {
		c_cryptkey.EnableWindow(FALSE);
	}

	// Completed
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Utility
/////////////////////////////////////////////////////////////////////////////

char CConnectGwPageAdv::hatoi(const CString& ha) const
{
	// Var
	char res = 0, i;

	// Iterate nibles
	for (i = 0; i < 2; i++) {
		// Move last nible
		res <<= 4;
		// Set nible
		if (ha.GetAt(i) >= '0' && ha.GetAt(i) <= '9')      res += (ha.GetAt(i) - '0');
		else if (ha.GetAt(i) >= 'a' && ha.GetAt(i) <= 'f') res += (ha.GetAt(i) - 'a' + 10);
		else if (ha.GetAt(i) >= 'A' && ha.GetAt(i) <= 'F') res += (ha.GetAt(i) - 'A' + 10);
	}

	// Completed
	return res;
}
