// ConnectGwPageCfg.cpp : implementation file
//

#include "stdafx.h"
#include "rtcuprog.h"
#include "ConnectGwPageCfg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConnectGwPageCfg property page

IMPLEMENT_DYNCREATE(CConnectGwPageCfg, CPropertyPage)

CConnectGwPageCfg::CConnectGwPageCfg() : CPropertyPage(CConnectGwPageCfg::IDD)
{
	// Var
	TCHAR szPath[MAX_PATH];

	// Get path to documents
	if (SUCCEEDED(SHGetFolderPath(NULL, CSIDL_COMMON_APPDATA, NULL, 0, szPath)))
	{
		// Build path
		m_server = szPath;
		if (m_server.Right(1) != _T("\\")) m_server += _T("\\");
		m_server += _T("Logic IO\\RTCU Programming Tool\\Logic IO RCH CA.pem");
	}

	// Default values
	m_gwip		= AfxGetApp()->GetProfileString(_T("GWConnect Options"), _T("GW IP"),   _T("domain.com"));
	m_gwport	= AfxGetApp()->GetProfileInt(   _T("GWConnect Options"), _T("GW Port"), 5001);
	m_gwkey		= AfxGetApp()->GetProfileString(_T("GWConnect Options"), _T("GW Key"),  _T("AABBCCDD"));
	m_enabled	= true;
	m_secure	= FALSE;

	// Read gateway list from register
	load_gwList();
}

CConnectGwPageCfg::~CConnectGwPageCfg()
{
	// Var
	POSITION   pos;
	gateway_t* pItem;

	// Iterate list
	pos = gw_list.GetHeadPosition();
	while (pos)
	{
		pItem = gw_list.GetNext(pos);
		if (pItem) delete pItem;
	}
	gw_list.RemoveAll();
}

void CConnectGwPageCfg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_GATEWAY_SELECT, gw_select);
	DDX_Control(pDX, IDC_SPINPORT, m_ctrgwportspin);
	DDX_Text(pDX, IDC_GWIP, m_gwip);
	DDX_Text(pDX, IDC_GWPORT, m_gwport);
	DDV_MinMaxInt(pDX, m_gwport, 1, 65535);
	DDX_Text(pDX, IDC_GWKEY, m_gwkey);
	DDV_MaxChars(pDX, m_gwkey, 8);
	DDX_Check(pDX, IDC_ENABLE, m_secure);
	DDX_Text(pDX, IDC_EDIT_SERVER, m_server);
}

BEGIN_MESSAGE_MAP(CConnectGwPageCfg, CPropertyPage)
	ON_WM_DESTROY()
	ON_REGISTERED_MESSAGE(UWM_CONNECT_PAGE, &CConnectGwPageCfg::OnConnectPage)
	ON_BN_CLICKED(IDC_CLEARCOMBO,           &CConnectGwPageCfg::OnClearGateway)
	ON_CBN_SELENDOK(IDC_GATEWAY_SELECT,     &CConnectGwPageCfg::OnSelectGateway)
	ON_BN_CLICKED(IDC_BROWSE,               &CConnectGwPageCfg::OnBrowse)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CConnectGwPageCfg::OnInitDialog() 
{
	// Default
	CPropertyPage::OnInitDialog();
 
	// Fill gateway list
	fill_gwList();
	OnSelectGateway();

	// Set controls
	m_ctrgwportspin.SetRange32(1, 65535);
	UpdateData(false);
	if (!m_enabled)
	{
		GetDlgItem(IDC_SPINPORT)->EnableWindow(false);
		GetDlgItem(IDC_GWPORT)->EnableWindow(false);
		GetDlgItem(IDC_GWKEY)->EnableWindow(false);
		GetDlgItem(IDC_GWIP)->EnableWindow(false);
		GetDlgItem(IDC_ENABLE)->EnableWindow(false);
		GetDlgItem(IDC_EDIT_SERVER)->EnableWindow(false);
		GetDlgItem(IDC_BROWSE)->EnableWindow(false);
	}

	// Completed
	return TRUE;
}

void CConnectGwPageCfg::OnDestroy() 
{
	// Save the gateway list
	save_gwList();

	// Default
	CPropertyPage::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// Update
/////////////////////////////////////////////////////////////////////////////

LRESULT CConnectGwPageCfg::OnConnectPage(WPARAM wParam, LPARAM lParam)
{
    // Var
    bool enable;

    // Action
    switch (wParam)
    {
    case 0: // Connection status: Disconnected
        enable = true;
        break;

    case 1: // Connection status: Attempting
        enable = false;
        break;

    case 2: // Connection status: Connecting
        enable = false;
        break;

    case 3: // Connection status: Gateway not found
        enable = true;
        break;

    case 4: // Connection status: Unit not found
        enable = true;
        break;

    case 5: // Connection status: Connected
        enable = false;
        break;

    default: // Update parameters
        UpdateData(true);
        if (m_gwip.IsEmpty()) m_gwip = _T("domain.com");
        OnInsertGateway(m_gwip, m_gwport, m_gwkey, m_secure, m_server);
        return 0;
    }

    // Change of state?
    if (enable != m_enabled)
    {
        m_enabled = enable;
        if (enable)
        {
            if (m_ctrgwportspin.GetSafeHwnd())
            {
                // Set controls
                GetDlgItem(IDC_SPINPORT)->EnableWindow(true);
                GetDlgItem(IDC_GWPORT)->EnableWindow(true);
                GetDlgItem(IDC_GWKEY)->EnableWindow(true);
                GetDlgItem(IDC_GWIP)->EnableWindow(true);
                GetDlgItem(IDC_ENABLE)->EnableWindow(true);
                GetDlgItem(IDC_EDIT_SERVER)->EnableWindow(true);
                GetDlgItem(IDC_BROWSE)->EnableWindow(true);
                GetDlgItem(IDC_CLEARCOMBO)->EnableWindow(true);
                GetDlgItem(IDC_GATEWAY_SELECT)->EnableWindow(true);
            }
        }
        else
        {
            if (m_ctrgwportspin.GetSafeHwnd())
            {
                // Set controls
                GetDlgItem(IDC_SPINPORT)->EnableWindow(false);
                GetDlgItem(IDC_GWPORT)->EnableWindow(false);
                GetDlgItem(IDC_GWKEY)->EnableWindow(false);
                GetDlgItem(IDC_GWIP)->EnableWindow(false);
                GetDlgItem(IDC_ENABLE)->EnableWindow(false);
                GetDlgItem(IDC_EDIT_SERVER)->EnableWindow(false);
                GetDlgItem(IDC_BROWSE)->EnableWindow(false);
                GetDlgItem(IDC_CLEARCOMBO)->EnableWindow(false);
                GetDlgItem(IDC_GATEWAY_SELECT)->EnableWindow(false);
            }
        }
    }
    return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Gateway list
/////////////////////////////////////////////////////////////////////////////

void CConnectGwPageCfg::load_gwList(void)
{
    // Var
    gateway_t* pItem, work;
    CString   StrList, str;
    int       pos, begin, end;

    // Load list of hub servers
    StrList = AfxGetApp()->GetProfileString(_T("Connect\\RCH"), _T("servers"), _T("<EMPTY>"));

    // Parse list
    if (StrList != _T("<EMPTY>"))
    {
        // Var
        CString temp;

        // Iterate
        pos = StrList.Find(_T("||"), 0);
        while (pos > -1)
        {
            // Create entry
            pItem           = new gateway_t;
            if (pItem == nullptr) break;

            // Get entry
            str             = StrList.Left(pos);
            StrList.Delete(0, pos + 2);

            // Parse
            pItem->host     = FindParam(str, _T("host"));
            temp            = FindParam(str, _T("port"));
            pItem->port     = atoi(temp);
            pItem->key      = FindParam(str, _T("key"));
            temp            = FindParam(str, _T("secure"));
            pItem->secure   = atoi(temp);
            pItem->server   = FindParam(str, _T("server"));

            // Insert
            gw_list.AddTail(pItem);

            // Next
            pos = StrList.Find(_T("||"));
        }
    }
}

void CConnectGwPageCfg::save_gwList(void)
{
    // Var
    POSITION  pos;
    gateway_t* pItem;
    CString   StrList, Temp, Text;

    // Iterate Gateway's
    pos = gw_list.GetHeadPosition();
    while (pos)
    {
        pItem = gw_list.GetNext(pos);
        if (pItem)
        {
            // Build entry
            Text     = _T("host=");
            Text    += pItem->host;
            Temp.Format(_T("|port=%d|key="), pItem->port);
            Text    += Temp;
            Text    += pItem->key;
            Temp.Format(_T("|secure=%d|server="), pItem->secure);
            Text    += Temp;
            Text    += pItem->server;
            Text    += _T("||");

            // Insert entry
            StrList += Text;
        }
    }

    // Save list in the registry
    AfxGetApp()->WriteProfileString(_T("Connect\\RCH"), _T("servers"), StrList);
}

void CConnectGwPageCfg::fill_gwList(void)
{
    // Var
    POSITION  pos;
    gateway_t* pItem;
    CString   text;

    // Iterate Gateway's
    pos = gw_list.GetHeadPosition();
    while (pos)
    {
        pItem = gw_list.GetNext(pos);
        if (pItem)
        {
            text.Format(_T("%s:%d"), pItem->host, pItem->port);
            gw_select.AddString(text);
        }
    }
    gw_select.SetCurSel(0);
}

CString CConnectGwPageCfg::FindParam(const CString& str, const CString& param)
{
    // Var
    int pos, div, len;

    // Find
    len = param.GetLength();
    pos = str.Find(param);
    div = str.Find(_T("|"), pos);

    // Validate
    if (pos < 0) return CString();
    if (str.GetAt(pos + len) != _T('=')) return CString();

    if (div < 0) return str.Mid(pos + len + 1);
    return str.Mid(pos + len + 1, div - (pos + len + 1));
}

CConnectGwPageCfg::gateway_t* CConnectGwPageCfg::find_gwList(gateway_t* entry)
{
    // Var
    POSITION  pos;
    gateway_t* pItem = nullptr;

    // Iterate
    pos = gw_list.GetHeadPosition();
    while (pos)
    {
        pItem = gw_list.GetNext(pos);
        if (pItem)
        {
            // Compare
            if (pItem->host == entry->host && pItem->key == entry->key &&
                pItem->port == pItem->port && pItem->secure == entry->secure)
                return pItem;
        }
    }

    // Completed
    return nullptr;
}

void CConnectGwPageCfg::OnSelectGateway()
{
    // Var
    POSITION  pos;
    gateway_t* pItem;
    int       index;

    // Validate
    if (gw_list.IsEmpty()) return;

    // Get selected gateway
    index           = gw_select.GetCurSel();
    pos             = gw_list.FindIndex(index);
    pItem           = gw_list.GetAt(pos);

    // Update dialog
    if (pItem)
    {
        m_gwip      = pItem->host;
        m_gwport    = pItem->port;
        m_gwkey     = pItem->key;
        m_secure    = pItem->secure;
        m_server    = pItem->server;
        UpdateData(FALSE);
    }
}

void CConnectGwPageCfg::OnBrowse()
{
    // Var
    CString     path, title(_T("Select certificate"));

    // Update
    UpdateData(TRUE);

    // Get selected path
    if (m_server.IsEmpty())
    {
        path = AfxGetApp()->GetProfileString(_T("Connect\\RCH"), _T("Certificate"), _T(""));
    }
    else
    {
        path = m_server;
    }

    // Select file
    CFileDialog dlg(
                    TRUE,
                    _T("pem"),
                    path,
                    OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST,
                    _T("Certificate file (*.crt,*.cert,*.cer,*.pem)|*.crt;*.cert;*.cer;*.pem||")
                   );

    // Set file buffer
    dlg.m_ofn.lpstrTitle = title;

    // Get destenation
    if (dlg.DoModal() == IDOK)
    {
        // Update
        m_server = dlg.GetPathName();
        UpdateData(FALSE);
    }
}

void CConnectGwPageCfg::OnInsertGateway(const CString& host, const int port, const CString& key, const int secure, const CString& server)
{
    // Var
    gateway_t* pItem = NULL;
    CString   text;

    // Build text
    text.Format(_T("%s:%d"), host, port);

    // Find entrty
    if (m_ctrgwportspin.GetSafeHwnd())
    {
        // Var
        POSITION pos;
        int      index;

        // Find entrty
        index = gw_select.FindStringExact(-1, text);
        if (index > -1)
        {
            // Get entry data
            pos = gw_list.FindIndex(index);
            pItem = gw_list.GetAt(pos);

            // Remove old entry
            gw_select.DeleteString(index);
            gw_list.RemoveAt(pos);
        }
    }
    else
    {
        // Var
        POSITION  pos, last;
        gateway_t* temp;

        // Iterate gateway list
        pos = gw_list.GetHeadPosition();
        while (pos)
        {
            last = pos;
            temp = gw_list.GetNext(pos);
            if (temp && temp->host == host && temp->port == port)
            {
                // Get entry data
                pItem = temp;

                // Remove old entry
                gw_list.RemoveAt(last);
                break;
            }
        }
    }

    // Create new entry?
    if (!pItem)
    {
        pItem = new gateway_t;
        if (pItem)
        {
            pItem->host = host;
            pItem->port = port;
        }
    }
    if (pItem)
    {
        pItem->key = key;
        pItem->secure = secure;
        pItem->server = server;
    }

    // Insert entry
    if (pItem)
    {
        gw_list.AddHead(pItem);
        if (m_ctrgwportspin.GetSafeHwnd())
        {
            gw_select.InsertString(0, text);
            gw_select.SetCurSel(0);
        }
    }
}

void CConnectGwPageCfg::OnClearGateway()
{
    // Var
    POSITION  pos;
    gateway_t* pItem;

    // Verify with user
    if (AfxMessageBox(_T("Do you really want to clear the list?"), MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDNO)
        return;

    // Empty contents
    gw_select.ResetContent();

    pos = gw_list.GetHeadPosition();
    while (pos)
    {
        pItem = gw_list.GetNext(pos);
        if (pItem) delete pItem;
    }
    gw_list.RemoveAll();
}
