// DlgFilesysReport.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "DlgFilesysReport.h"
#include "afxdialogex.h"


// CDlgFilesysReport dialog

IMPLEMENT_DYNAMIC(CDlgFilesysReport, CDialog)

CDlgFilesysReport::CDlgFilesysReport(CWnd* pParent /*=NULL*/)
   : CDialog(CDlgFilesysReport::IDD, pParent)
{
   m_ReportCancel = false;
}

CDlgFilesysReport::~CDlgFilesysReport()
{
}

void CDlgFilesysReport::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDCANCEL, c_cancel);
   DDX_Control(pDX, IDC_PROGRESS, c_progress);
   DDX_Control(pDX, IDC_LIST, c_list);
}

BEGIN_MESSAGE_MAP(CDlgFilesysReport, CDialog)
   ON_WM_SIZE()
   ON_WM_GETMINMAXINFO()
   ON_REGISTERED_MESSAGE( UWM_UPDATE_WINDOW, &CDlgFilesysReport::OnUpdateWindow )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Create / Destroy
/////////////////////////////////////////////////////////////////////////////

BOOL CDlgFilesysReport::OnInitDialog()
{
   // Default
   CDialog::OnInitDialog();

   // Setup progress
   c_progress.SetRange(0, 100);

   // Completed
   return TRUE;
}

void CDlgFilesysReport::PostNcDestroy()
{
   CDialog::PostNcDestroy();
   delete this;
}

/////////////////////////////////////////////////////////////////////////////
// Fail safe
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesysReport::OnOK()
{
}

/////////////////////////////////////////////////////////////////////////////
// Resize
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesysReport::OnSize(UINT nType, int cx, int cy)
{
   // Var
   RECT rcList, rcProg, rcDone;

   // Default
   CDialog::OnSize(nType, cx, cy);

   // Calculate resize
   rcList.left   = 11;
   rcList.top    = 11;
   rcList.right  = cx - 21;
   rcList.bottom = cy - 81;
   rcProg.left   = 11;
   rcProg.top    = cy - 61;
   rcProg.right  = cx - 21;
   rcProg.bottom = 23;
   rcDone.left   = (cx - 75) / 2;
   rcDone.top    = cy - 34;
   rcDone.right  = 75;
   rcDone.bottom = 23;

   // Resize controls
   if (c_list.GetSafeHwnd())     c_list.MoveWindow(     rcList.left, rcList.top, rcList.right, rcList.bottom);
   if (c_progress.GetSafeHwnd()) c_progress.MoveWindow( rcProg.left, rcProg.top, rcProg.right, rcProg.bottom);
   if (c_cancel.GetSafeHwnd())   c_cancel.MoveWindow(   rcDone.left, rcDone.top, rcDone.right, rcDone.bottom);
}

void CDlgFilesysReport::OnGetMinMaxInfo(MINMAXINFO* lpMMI)
{
   // Set minimum size
   lpMMI->ptMinTrackSize.x = 225;
   lpMMI->ptMinTrackSize.y = 200;
}

/////////////////////////////////////////////////////////////////////////////
// Cancel
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesysReport::OnCancel()
{
   m_ReportCancel = true;
}

/////////////////////////////////////////////////////////////////////////////
// Update contents
/////////////////////////////////////////////////////////////////////////////

LRESULT CDlgFilesysReport::OnUpdateWindow( WPARAM wParam, LPARAM lParam )
{
   // Function
   switch (wParam)
   {
      // Reset view
      case 0:
         SetWindowText(m_Text);
         c_list.ResetContent();
         c_progress.SetPos(0);
         c_cancel.SetWindowText(_T("Cancel"));
         break;

      // Add entry to list
      case 1:
         c_list.AddString(m_Text);
         c_list.SetCurSel( c_list.GetCount()-1 );
         c_progress.SetPos(0);
         break;

      // Set progress
      case 2:
         if (lParam > -1 && lParam < 101) c_progress.SetPos(lParam);
         break;

      // Report completed
      case 3:
         c_list.AddString(m_Text);
         c_list.SetCurSel( c_list.GetCount()-1 );
         c_cancel.SetWindowText(_T("Done"));
         break;

      // Reset view
      case 10:
         SetWindowText((LPCTSTR)lParam);
         c_list.ResetContent();
         c_progress.SetPos(0);
         c_cancel.SetWindowText(_T("Cancel"));
         m_ReportCancel = false;
         break;

      // Add entry to list
      case 11:
         c_list.AddString((LPCTSTR)lParam);
         c_list.SetCurSel(c_list.GetCount()-1);
         c_progress.SetPos(0);
         break;

      // Report completed
      case 13:
         c_list.AddString((LPCTSTR)lParam);
         c_list.SetCurSel(c_list.GetCount()-1);
         c_cancel.SetWindowText(_T("Done"));
         break;

      // Report cancelled
      case 14:
         return m_ReportCancel;

      // Report completed
      case 15:
         c_cancel.EnableWindow(lParam ? TRUE : FALSE);
         break;

      // Show error message
      case 20:
         switch (lParam)
         {
            case 1:
               MessageBox(_T("Filename must follow the old 8.3 format!\neg. \"datalog.dat\""), _T("Illegal file name"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 2:
               MessageBox(_T("Error communicating with device"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 3:
               MessageBox(_T("Media is write-protected!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 4:
               MessageBox(_T("Media is not present!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 5:
               MessageBox(_T("Could not import complete file\nNo more free space on media!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 6:
               MessageBox(_T("Could not export complete file\nNo more free space on drive!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 7:
               MessageBox(_T("Unknown file error!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 8:
               MessageBox(_T("Failed to open media!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 10:
               MessageBox(_T("Error while communicating with device!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 11:
               MessageBox(_T("The media is not present!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 12:
               MessageBox(_T("The media is write-protected!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 13:
               MessageBox(_T("The media cannot be ejected!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 14:
               MessageBox(_T("Error on the media!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 15:
               MessageBox(_T("The media is busy!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 19:
               MessageBox(_T("Unknown error!"), m_Text, MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 20:
               MessageBox(_T("Could not find the file!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 21:
               MessageBox(_T("The file is in use!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 22:
               MessageBox(_T("A file with this name already exists!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 23:
               MessageBox(_T("Could not open the file!"), _T("Error"), MB_ICONERROR|MB_OK);
               m_ReportCancel = true;
               break;

            case 24:
               MessageBox(_T("Timeout waiting for quickformat to complete.\nOperation continue in the device."), m_Text, MB_ICONINFORMATION|MB_OK);
               m_ReportCancel = true;
               break;
         }
         break;

      // Show question message
      case 21:
         switch (lParam)
         {
            case 1:
               m_ReportReply  = MessageBox(_T("A file with this name already exists!\nDo you wish to overwrite it?"), _T("Overwrite - ") + m_Text, MB_ICONQUESTION|MB_YESNO);
               m_ReportCancel = true;
               break;
         }
         break;

      // Show message
      case 22:
         switch (lParam)
         {
            case 1:
               MessageBox(_T("Media is formatted!"), _T("Quick format media"), MB_ICONASTERISK|MB_OK);
               m_ReportCancel = true;
               break;
         }
         break;
   }
   return 0;
}


void FilereportReset(CDialog *pView, CString Text)
{
   pView->SendMessage(UWM_UPDATE_WINDOW, 10, (LPARAM)(LPCTSTR)Text);
}

void FilereportAdd(CDialog *pView, CString Text)
{
   pView->SendMessage(UWM_UPDATE_WINDOW, 11, (LPARAM)(LPCTSTR)Text);
}

void FilereportProgress(CDialog *pView, int val)
{
   pView->SendMessage(UWM_UPDATE_WINDOW, 2, val);
}

void FilereportCompleted(CDialog *pView, CString Text)
{
   pView->SendMessage(UWM_UPDATE_WINDOW, 13, (LPARAM)(LPCTSTR)Text);
}

bool FilereportCancelled(CDialog *pView)
{
   return pView->SendMessage(UWM_UPDATE_WINDOW, 14, 0) ? true : false;
}

void FilereportDisable(CDialog *pView, bool disabled)
{
   pView->SendMessage(UWM_UPDATE_WINDOW, 15, disabled ? 0 : 1);
}
