// DlgFilesysShow.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "DlgFilesysShow.h"
#include "afxdialogex.h"


// CDlgFilesysShow dialog

IMPLEMENT_DYNAMIC(CDlgFilesysShow, CDialog)

CDlgFilesysShow::CDlgFilesysShow(CWnd* pParent /*=NULL*/)
   : CDialog(CDlgFilesysShow::IDD, pParent)
{
   m_view   = AfxGetApp()->GetProfileInt(_T("Filesystem"), _T("ShowFileMethod"), 1);
   m_Source = 0;
}

CDlgFilesysShow::~CDlgFilesysShow()
{
}

void CDlgFilesysShow::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_EDIT_DATA_TXT, c_data_text);
   DDX_Control(pDX, IDC_EDIT_DATA, m_data);
   DDX_Radio(pDX, IDC_RADIO_HEX, m_view);
}

BEGIN_MESSAGE_MAP(CDlgFilesysShow, CDialog)
   ON_STN_CLICKED(IDC_RADIO_HEX, &CDlgFilesysShow::OnRadioHex)
   ON_STN_CLICKED(IDC_RADIO_TEXT, &CDlgFilesysShow::OnRadioText)
   ON_WM_SIZE()
   ON_WM_GETMINMAXINFO()
END_MESSAGE_MAP()

// CDlgFilesysShow message handlers

BOOL CDlgFilesysShow::OnInitDialog()
{
   // Var
   unsigned char   rxBuf[512], *dataBuf;
   long            Len;
   short           rxLen;
   CString         text;
   LPTSTR          memStr;
   WINDOWPLACEMENT wp;

   // Default
   CDialog::OnInitDialog();

   // Position sheet
   if (ReadWindowPlacement(&wp))
   {
      wp.showCmd=false;
      SetWindowPlacement(&wp);
   }

   // Test file
   if (m_File.IsEmpty()) return FALSE;

   GetDlgItem(IDC_EDIT_FILENAME)->SetWindowText(m_File);

   if (m_view == 0)
   {
      m_data.ShowWindow(SW_SHOW);
      c_data_text.ShowWindow(SW_HIDE);
   }
   else
   {
      m_data.ShowWindow(SW_HIDE);
      c_data_text.ShowWindow(SW_SHOW);
   }

   // Set font
   LOGFONT logfont;
   memset(&logfont, 0, sizeof(logfont));
   logfont.lfHeight = -11;
   logfont.lfWeight = FW_NORMAL;
   strcpy_s(logfont.lfFaceName, 32, "Courier new");
   m_font.CreateFontIndirect(&logfont);
   c_data_text.SetFont(&m_font);

   // RACP source
   try
   {
      // Open source file
      CFile srcFile(m_SourceFile, CFile::modeRead);

      // Data buffer
      dataBuf = new unsigned char[srcFile.GetLength()];
      Len     = 0;

      // Iterate file
      do
      {
         rxLen = srcFile.Read(rxBuf, sizeof(rxBuf));
         memcpy(&dataBuf[Len], rxBuf, rxLen);
         Len  += rxLen;
      }
      while(rxLen == sizeof(rxBuf));

      // Insert into text window
      if (Len > 0)
      {
         memStr      = text.GetBuffer(Len+1);
         memcpy(memStr, dataBuf, Len);
         memStr[Len] = 0;
         text.ReleaseBuffer();
      }
      c_data_text.SetWindowText(text);

      // Insert into control
      m_data.m_bpr=16;
      m_data.SetOptions(true, true, true, true);
      m_data.SetReadonly(true);
      m_data.SetData(dataBuf, Len);
      delete [] dataBuf;
   }
   catch (CFileException* e)
   {
      // Cleanup
      e->Delete();
   }

   // Completed
   return TRUE;
}

void CDlgFilesysShow::OnCancel()
{
   // Save position
   WINDOWPLACEMENT wp;
   wp.length = sizeof wp;
   if (GetWindowPlacement(&wp))
   {
      wp.flags = 0;
      if (IsZoomed())
         wp.flags |= WPF_RESTORETOMAXIMIZED;
      // and write it to the .INI file
      WriteWindowPlacement(&wp);
   }

   // Default
   CDialog::OnCancel();
}

void CDlgFilesysShow::OnRadioHex()
{
   m_data.ShowWindow(SW_SHOW);
   c_data_text.ShowWindow(SW_HIDE);
   AfxGetApp()->WriteProfileInt(_T("Filesystem"), _T("ShowFileMethod"), 0);
}

void CDlgFilesysShow::OnRadioText()
{
   m_data.ShowWindow(SW_HIDE);
   c_data_text.ShowWindow(SW_SHOW);
   AfxGetApp()->WriteProfileInt(_T("Filesystem"), _T("ShowFileMethod"), 1);
}

/////////////////////////////////////////////////////////////////////////////
// Resize
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesysShow::OnSize(UINT nType, int cx, int cy)
{
   // Default
   CDialog::OnSize(nType, cx, cy);

   // Ensure controls are created
   if (!m_data.GetSafeHwnd()) return;

   // Set control positions
   GetDlgItem(IDC_EDIT_FILENAME)->MoveWindow(11, 11, cx - 21, 23);
   m_data.MoveWindow(     11, 42, cx - 21, cy - 81);
   c_data_text.MoveWindow(11, 42, cx - 21, cy - 81);
   GetDlgItem(IDC_RADIO_HEX)->MoveWindow( 11,  cy - 32, 80, 16);
   GetDlgItem(IDC_RADIO_TEXT)->MoveWindow(104, cy - 32, 71, 16);
   GetDlgItem(IDCANCEL)->MoveWindow(cx - 86, cy - 34, 75, 23);
}

void CDlgFilesysShow::OnGetMinMaxInfo(MINMAXINFO* lpMMI)
{
   // Set minimum size
   lpMMI->ptMinTrackSize.x = 345;
   lpMMI->ptMinTrackSize.y = 310;
}

/////////////////////////////////////////////////////////////////////////////
// Utility
/////////////////////////////////////////////////////////////////////////////

BOOL CDlgFilesysShow::ReadWindowPlacement(LPWINDOWPLACEMENT pwp)
{
   CString strBuffer = AfxGetApp()->GetProfileString(_T("Filesystem"), _T("ShowFilePos"));
   if (strBuffer.IsEmpty()) return FALSE;

   WINDOWPLACEMENT wp;
   int nRead = _stscanf_s(
                          strBuffer,
                          _T("%u,%u,%d,%d,%d,%d,%d,%d,%d,%d"),
                          &wp.flags,
                          &wp.showCmd,
                          &wp.ptMinPosition.x,
                          &wp.ptMinPosition.y,
                          &wp.ptMaxPosition.x,
                          &wp.ptMaxPosition.y,
                          &wp.rcNormalPosition.left,
                          &wp.rcNormalPosition.top,
                          &wp.rcNormalPosition.right,
                          &wp.rcNormalPosition.bottom
                         );

   if (nRead != 10)
   return FALSE;

   wp.length = sizeof wp;
   *pwp = wp;
   return TRUE;
}

void CDlgFilesysShow::WriteWindowPlacement(LPWINDOWPLACEMENT pwp)
{
   TCHAR szBuffer[sizeof("-32767")*8 + sizeof("65535")*2];

   wsprintf(
            szBuffer,
            _T("%u,%u,%d,%d,%d,%d,%d,%d,%d,%d"),
            pwp->flags,
            pwp->showCmd,
            pwp->ptMinPosition.x,
            pwp->ptMinPosition.y,
            pwp->ptMaxPosition.x,
            pwp->ptMaxPosition.y,
            pwp->rcNormalPosition.left,
            pwp->rcNormalPosition.top,
            pwp->rcNormalPosition.right,
            pwp->rcNormalPosition.bottom
           );
   AfxGetApp()->WriteProfileString(_T("Filesystem"), _T("ShowFilePos"), szBuffer);
}
