// DlgFilesystem.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "DlgFilesystem.h"
#include "DlgFilesysInput.h"
#include "DlgFilesysReport.h"
#include "DlgFilesysShow.h"
#include "afxdialogex.h"


/////////////////////////////////////////////////////////////////////////////
// Defines
/////////////////////////////////////////////////////////////////////////////
#define FS_INVALIDDRIVE       101
#define FS_NOTFORMATTED       102
#define FS_NOTFOUND           105
#define FS_DUPLICATED         106
#define FS_NOMOREENTRY        107
#define FS_NOTOPEN            108
#define FS_EOF                109
#define FS_LOCKED             112
#define FS_NOTEMPTY           114
#define FS_CARDREMOVED        116
#define FS_CARDERROR          117
#define FS_MAINTENENCE        122
#define FS_WRITEPROTECT       123
#define FS_FILECLOSED         138

#define FSF_DIR_CREATE        1
#define FSF_DIR_DELETE        2
#define FSF_FILE_DELETE       3
#define FSF_FILE_EXPORT       4
#define FSF_FILE_IMPORT       5
#define FSF_FILE_RENAME       6
#define FSF_FILE_SHOW         7
#define FSF_MEDIA_EJECT       8
#define FSF_MEDIA_FORMAT      9

static TCHAR BASED_CODE szSection[] = _T("Filesystem");

// CDlgFilesystem dialog

IMPLEMENT_DYNAMIC(CDlgFilesystem, CDialog)

CDlgFilesystem::CDlgFilesystem(CWnd* pParent /*=NULL*/)
   : CDialog(CDlgFilesystem::IDD, pParent)
   , m_wildcard(_T(""))
{
   m_refreshing   = false;
   m_fileproc     = false;
   m_closing      = false;

   // Media drop list
   for (int i = 0; i < MEDIA; i++)
   {
      m_media[i].opened         = false;
      m_media[i].present        = false;
      m_media[i].formatted      = false;
      m_media[i].type           = 0;
      m_media[i].rootdir.Tree   = NULL;
      m_media[i].rootdir.Name   = _T("Not Present");
   }
   m_media[0].title = _T("A: SD-CARD");
   m_media[1].title = _T("B: Internal");
   m_media[2].title = _T("P: Project");
   m_media[3].title = _T("U: USB drive");
}

CDlgFilesystem::~CDlgFilesystem()
{
}

void CDlgFilesystem::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   DDX_Control(pDX, IDC_COMBO_MEDIA, c_drop);
   DDX_Control(pDX, IDC_BUTTON_REFRESH, c_refresh);
   DDX_Control(pDX, IDC_LIST_FILES, c_list);
   DDX_Control(pDX, IDC_TREE_DIR, c_tree);
   DDX_Control(pDX, IDC_WILDCARD, c_wild);
   DDX_Control(pDX, IDC_RECURSIVE, c_recursive);
   DDX_CBString(pDX, IDC_WILDCARD, m_wildcard);
}


BEGIN_MESSAGE_MAP(CDlgFilesystem, CDialog)
   ON_WM_DESTROY()
   ON_WM_CONTEXTMENU()
   ON_BN_CLICKED(IDC_BUTTON_REFRESH, &CDlgFilesystem::OnClickRefresh)
   ON_REGISTERED_MESSAGE(UWM_UPDATE_WINDOW, &CDlgFilesystem::OnUpdateWindow)
   ON_CBN_SELENDOK(IDC_COMBO_MEDIA, &CDlgFilesystem::OnSelectMedia)
   ON_NOTIFY(TVN_SELCHANGED, IDC_TREE_DIR, &CDlgFilesystem::OnSelectDir)
   ON_NOTIFY(TVN_ITEMEXPANDING, IDC_TREE_DIR, &CDlgFilesystem::OnExpandDir)
   ON_NOTIFY(TVN_KEYDOWN, IDC_TREE_DIR, &CDlgFilesystem::OnDeleteDir)
   ON_COMMAND(IDM_DIR_CREATE, &CDlgFilesystem::OnDirCreate)
   ON_COMMAND(IDM_DIR_DELETE, &CDlgFilesystem::OnDirDelete)
   ON_COMMAND(IDM_MEDIA_EJECT, &CDlgFilesystem::OnMediaEject)
   ON_COMMAND(IDM_MEDIA_FORMAT, &CDlgFilesystem::OnMediaFormat)
   ON_COMMAND(IDM_FILE_DELETE, &CDlgFilesystem::OnFileDelete)
   ON_COMMAND(IDM_FILE_EXPORT, &CDlgFilesystem::OnFileExport)
   ON_COMMAND(IDM_FILE_IMPORT, &CDlgFilesystem::OnFileImport)
   ON_COMMAND(IDM_FILE_RENAME, &CDlgFilesystem::OnFileRename)
   ON_COMMAND(IDM_FILE_SELECTALL, &CDlgFilesystem::OnFileSelectall)
   ON_NOTIFY(NM_DBLCLK, IDC_LIST_FILES, &CDlgFilesystem::OnFileShow)
   ON_NOTIFY(LVN_KEYDOWN, IDC_LIST_FILES, &CDlgFilesystem::OnFileRemove)
   ON_WM_TIMER()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Create / Destroy
/////////////////////////////////////////////////////////////////////////////

BOOL CDlgFilesystem::OnInitDialog()
{
   // Var
   CWinApp*        pApp;
   CString         str, strList;
   int             size;

   // Get application
   pApp = AfxGetApp();

   // Default
   CDialog::OnInitDialog();

   // Initialize media list
   for (int i = 0; i < MEDIA; i++) c_drop.AddString(m_media[i].title);
   c_drop.SetCurSel(0);

   // Initialize file report
   c_list.SetExtendedStyle(c_list.GetExtendedStyle()|LVS_EX_FULLROWSELECT);
   size = pApp->GetProfileInt(szSection, _T("ColNameSize"), 90);
   c_list.InsertColumn(0, _T("Name"), LVCFMT_LEFT, size);
   size = pApp->GetProfileInt(szSection, _T("ColSizeSize"), 50);
   c_list.InsertColumn(1, _T("Size"), LVCFMT_RIGHT, size);
   size = pApp->GetProfileInt(szSection, _T("ColTimeSize"), 100);
   c_list.InsertColumn(2, _T("Created"), LVCFMT_LEFT, size);

   // Save recursive fetch
   c_recursive.SetCheck(pApp->GetProfileInt(szSection, _T("Recursive"), 1));

   // Initialize wild cards
   c_wild.LimitText(12);
   c_wild.AddString(_T("*.*"));
   strList = AfxGetApp()->GetProfileString(szSection, _T("wildcards"));
   if (!strList.IsEmpty())
   {
      // Find how many entries exists in the list, format is "AA||BB||CC||DD||"
      int i = strList.Replace(_T("||"), _T("||")); // This gives a count for the number of "||" sequences
      while (i)
      {
         str = strList.Left(strList.Find(_T("||"), 0)); // Extract the string
         strList.Delete(0, strList.Find(_T("||"), 0));  // Kill the string
         strList.Delete(0, 2);                          // Kill the "||"
         if (!str.IsEmpty()) c_wild.AddString(str);
         i--;
      }
   }
   c_wild.SetCurSel(0);

   // Initialize report window
   CDlgFilesysReport* pDlg = new CDlgFilesysReport();
   pDlg->Create(IDD_FILESYS_REPORT, this);
   m_ReportWnd = pDlg;

   // Initialize
   m_root       = &m_media[0].rootdir;
   m_root->Tree = c_tree.InsertItem(m_root->Name);
   m_work       = m_root;

   // Start refresh timer
   SetTimer(123, 3000, NULL);

   // Completed
   return TRUE;
}

void CDlgFilesystem::OnDestroy()
{
   // Var
   CString         str, strList;

   // Save file report settings
   AfxGetApp()->WriteProfileInt(szSection, _T("ColNameSize"), c_list.GetColumnWidth(0));
   AfxGetApp()->WriteProfileInt(szSection, _T("ColSizeSize"), c_list.GetColumnWidth(1));
   AfxGetApp()->WriteProfileInt(szSection, _T("ColTimeSize"), c_list.GetColumnWidth(2));

   // Save recursive fetch
   AfxGetApp()->WriteProfileInt(szSection, _T("Recursive"), c_recursive.GetCheck());

   // Save wild cards
   for (int i = 1; i < c_wild.GetCount(); i++)
   {
      c_wild.GetLBText(i, str);
      if (!str.IsEmpty())
      {
         strList += str;
         strList += "||";
      }
   }
   AfxGetApp()->WriteProfileString(szSection, _T("wildcards"), strList);

   // Default
   CDialog::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// Failsafe
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesystem::OnOK()
{
}

void CDlgFilesystem::OnCancel()
{
   // Stop refreshing
   m_closing = true;

   // Media drop list
   for (int i = 0; i < MEDIA; i++)
   {
      // Close filesystem
      if (m_media[i].opened)
      {
         // Var
         int fserr;

         // Close filesystem
         rmonMediaClose(hCon, i, &fserr);
         m_media[i].opened = false;
      }
   }

   // Hide dialog
   ShowWindow(SW_HIDE);
}

/////////////////////////////////////////////////////////////////////////////
// Connection status
/////////////////////////////////////////////////////////////////////////////

LRESULT CDlgFilesystem::OnUpdateWindow( WPARAM wParam, LPARAM lParam )
{
   // Connection changed
   if (wParam==10)
   {
      // Disconnected
      if (lParam == 0)
      {
         // Unit is disconnected
         m_cur_media = -1;

         // Close medias
         for (int i = 0; i < MEDIA; i++) m_media[i].opened  = false;
      }
      // Connected
      else
      {
         // Var
         DWORD         ThreadId;

         // Start thread
         CreateThread(NULL, 0, thConnection, this, 0, &ThreadId);
      }

      // Update controls
      wParam = 0;
   }

   // Update controls
   if (wParam==0)
   {
      // Get selected media
      int media = GetSelectedMedia();

      // Refresh button
      if (!gUnitInfo.Connected || m_refreshing || m_fileproc)
      {
         // Disable refresh button
         c_refresh.EnableWindow(FALSE);
      }
      else
      {
         // Enable refresh button
         c_refresh.EnableWindow(TRUE);
      }

      // Refresh dropdown list
      c_drop.EnableWindow(m_refreshing ? FALSE : TRUE);
      c_recursive.EnableWindow(m_refreshing ? FALSE : TRUE);

      // Refresh done button
      GetDlgItem(IDCANCEL)->EnableWindow(m_fileproc ? FALSE : TRUE);
   }

   // Update file report?
   else if (wParam==1)
   {
      UpdateFileReport();
   }

   // Show file?
   else if (wParam==2)
   {
      // Var
      CDlgFilesysShow    dlg;
      tdefWorkInfo*      pInfo = (tdefWorkInfo*)lParam;

      // Build file path
      dlg.m_File       = pInfo->srcFiles.GetHead();
      dlg.m_Source     = 1;
      dlg.m_SourceFile = pInfo->dstPath;

      // Show file data
      dlg.DoModal();

      // Remove temporary file
      try
      {
         CFile::Remove(pInfo->dstPath);
      }
      catch (CFileException* e)
      {
         // Cleanup
         e->Delete();
      }

      // Cleanup
      delete pInfo;
   }

   // Completed
   return 0;
}

unsigned long __stdcall CDlgFilesystem::thConnection(LPVOID lpvParam)
{
   // Var
   CTreeCtrl&           rList = ((CDlgFilesystem*)lpvParam)->c_tree;
   CComboBox&           rDrop = ((CDlgFilesystem*)lpvParam)->c_drop;
   tdefFolderInfo*&     rRoot = ((CDlgFilesystem*)lpvParam)->m_root;
   tdefMediaInfo*       pMedia;
   int                  mediaid = ((CDlgFilesystem*)lpvParam)->GetSelectedMedia();
   int                  i, change;

   // Remove "Not Available"
   for (i = 0; i < MEDIA; i++)
   {
      if (((CDlgFilesystem*)lpvParam)->m_media[i].type == 0)
      {
         ((CDlgFilesystem*)lpvParam)->m_media[i].rootdir.Name = _T("Not Present");
         if (rRoot == &((CDlgFilesystem*)lpvParam)->m_media[i].rootdir)
         {
            rList.DeleteAllItems();
            rRoot->Tree = rList.InsertItem(rRoot->Name);
            rList.SetItemData(rRoot->Tree, (DWORD)rRoot);
            rList.SelectItem(rRoot->Tree);
         }
      }
   }

   // Update
   ((CDlgFilesystem*)lpvParam)->UpdateMedia();

   // Update to "No Card"
   for (i = 0; i < MEDIA; i++)
   {
      pMedia = &((CDlgFilesystem*)lpvParam)->m_media[i];
      change = 0;
      switch (pMedia->type)
      {
         case 0: // The media is not available on unit
            if (pMedia->rootdir.Name.Find(_T("\\")) == -1)
            {
               pMedia->rootdir.Name = _T("Not Available");
               change = 1;
            }
            // Empty current data
            ((CDlgFilesystem*)lpvParam)->ClearDirectory(&pMedia->rootdir);
            break;

         case 1: // Removeable media
            if (pMedia->rootdir.Name.Find(_T("\\")) == -1)
            {
               int state, fserr;
               if (rmonMediaPresent(hCon, i, &state, &fserr)) state = 0;
               pMedia->rootdir.Name = (fserr || !state) ? _T("No Card") : _T("Card");
               change = 1;
            }
            break;

         case 2: // Internal media
            if (pMedia->rootdir.Name.Find(_T("\\")) == -1)
            {
               int state, fserr;
               if (rmonMediaPresent(hCon, i, &state, &fserr)) state = 0;
               pMedia->rootdir.Name = (fserr || !state) ? _T("Not Present") : _T("Present");
               change = 1;
            }
            break;

         case 3: // Project drive
            if (pMedia->rootdir.Name.Find(_T("\\")) == -1)
            {
               pMedia->rootdir.Name = _T("Present");
               change = 1;
            }
            break;

         case 4: // USB media
            if (pMedia->rootdir.Name.Find(_T("\\")) == -1)
            {
               int state, fserr;
               if (rmonMediaPresent(hCon, i, &state, &fserr)) state = 0;
               pMedia->rootdir.Name = fserr==124 ? _T("Unknown filesystem") : ((fserr || !state) ? _T("Not Present") : _T("Present"));
               change = 1;
            }
            break;
      }
      if (change)
      {
         if (rRoot == &pMedia->rootdir)
         {
            rList.DeleteAllItems();
            rRoot->Tree = rList.InsertItem(rRoot->Name);
            rList.SetItemData(rRoot->Tree, (DWORD)rRoot);
            rList.SelectItem(rRoot->Tree);
         }
      }
   }

   // Switch to selected media
   if (((CDlgFilesystem*)lpvParam)->m_media[mediaid].type > 0)
   {
      CString drv;
      int     fserr;
      drv = ((CDlgFilesystem*)lpvParam)->GetMediaDrive(mediaid);
      i   = rmonDirChange(hCon, drv, &fserr);
      if (!i && !fserr) ((CDlgFilesystem*)lpvParam)->m_cur_media = mediaid;
   }

   // Completed
   return 0;
}

void CDlgFilesystem::OnTimer(UINT_PTR nIDEvent)
{
  if (m_work)
  {
    if (m_work->FileList.GetCount() > c_list.GetItemCount())
    {
      // Var
      CString       Temp;
      int           Index;
      POSITION      pos;
      tdefFileInfo* Item;

      // Iterate files
      pos = m_work->FileList.FindIndex(c_list.GetItemCount());
      while(pos)
      {
        // Get file
        Item = m_work->FileList.GetNext(pos);
        if (Item)
        {
          // Build size string
          Temp.Format("%d",Item->Size);

          // Insert into report
          Index = c_list.InsertItem(0, Item->Name);
          c_list.SetItemText(Index, 1, Temp);
          c_list.SetItemText(Index, 2, Item->Time);
        }
      }
    }
  }

  // Default
  CDialog::OnTimer(nIDEvent);
}

/////////////////////////////////////////////////////////////////////////////
// Refresh
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesystem::OnClickRefresh()
{
   // Get selected media
   int media = GetSelectedMedia();

   // Lock refresh
   m_refreshing = true;
   m_closing    = false;
   OnUpdateWindow(0, 0);

   // Update wildcard
   c_wild.GetWindowText(m_wildcard);
   if (!m_wildcard.IsEmpty())
   {
      // Var
      CString temp = m_wildcard;
      int     start, valid = 1;
      char    ch;

      // Validate
      start = temp.Find(_T('.'));
      if ((start == -1 && temp.GetLength() > 8) || start > 8 || (start > -1 && temp.GetLength() - start > 4))
      {
         // Show eror message
         temp  = _T("The selected wildcard [");
         temp += m_wildcard;
         temp += _T("]\nis not valid.\nMust be 8.3 format.");
         MessageBox(temp, _T("Fetch"), MB_OK|MB_ICONWARNING);
         c_wild.SetFocus();

         // Unlock refresh
         m_refreshing = false;
         OnUpdateWindow(0, 0);
         return;
      }
      for (ch = 33; ch < 127; ch++)
      {
         // Exceptions: ", /, :, <, >, '\', |
         if (ch==34 || ch==47 || ch==58 || ch==60 || ch==62 || ch==92 || ch==124) continue;
         temp.Remove(ch);
      }
      if (!temp.IsEmpty())
      {
         // Show eror message
         temp  = _T("The selected wildcard [");
         temp += m_wildcard;
         temp += _T("]\nis not valid.\nIt contains illegal character(s).");
         MessageBox(temp, _T("Fetch"), MB_OK|MB_ICONWARNING);
         c_wild.SetFocus();

         // Unlock refresh
         m_refreshing = false;
         OnUpdateWindow(0, 0);
         return;
      }
   }
   if (m_wildcard != _T("*.*"))
   {
      c_wild.DeleteString(c_wild.FindStringExact(-1, m_wildcard));
      c_wild.InsertString(1, m_wildcard);
      c_wild.SetCurSel(1);
      if (c_wild.GetCount() > 6) c_wild.DeleteString(6);
   }
   else
   {
      m_wildcard = _T("*");
   }

   // Clear current data
   ClearDirectory(m_work);
   c_list.DeleteAllItems();
   if (m_work == m_root)
   {
      // Empty Tree
      c_tree.DeleteAllItems();
   }

   // Is media available?
   if (m_media[media].type == 0)
   {
      // Set media status
      m_media[media].present = false;

      // Empty current data
      ClearDirectory(m_root);
      c_tree.DeleteAllItems();

      // Build root item
      m_root->Name = _T("Not Available");
      m_root->Tree = c_tree.InsertItem(m_root->Name);
      c_tree.SetItemData(m_root->Tree, (DWORD)m_root );

      // Set working directory
      m_work      = m_root;
      c_tree.SelectItem(m_root->Tree);

      // Unlock refresh
      m_refreshing = false;
      OnUpdateWindow(0, 0);
      return;
   }

   // Is RTCU IDE connected to unit?
   if (!gUnitInfo.Connected || gUnitInfo.TargetID<100 ||
       (gUnitInfo.TargetID<200 && (gUnitInfo.FirmwareVer<107 ||
        (gUnitInfo.FirmwareVer>8000 && gUnitInfo.FirmwareVer<8107))))
   {
      // Var
      CString tooltip;

      // Set media status
      m_media[media].present = false;

      // Empty current data
      ClearDirectory(m_root);
      c_tree.DeleteAllItems();

      // Build root item
      m_root->Name = media == 0 ? _T("No Card") : _T("Not Present");
      m_root->Tree = c_tree.InsertItem(m_root->Name);
      c_tree.SetItemData(m_root->Tree, (DWORD)m_root );

      // Set working directory
      m_work      = m_root;
      c_tree.SelectItem(m_root->Tree);

      // Unlock refresh
      m_refreshing = false;
      OnUpdateWindow(0, 0);
   }
   else
   {
      // Var
      DWORD         ThreadId;
      tdefWorkInfo* pInfo = NULL;

      // Build refresh info struct
      pInfo            = new tdefWorkInfo;
      pInfo->dstPath   = GetSelectedPath();
      pInfo->Type      = 0;
      pInfo->wndParent = this;
      pInfo->wndReport = NULL;
      pInfo->dirInfo   = m_work;
      if (pInfo->dstPath.Right(1) != _T('\\')) pInfo->dstPath += _T('\\');

      // Start thread
      CreateThread(NULL, 0, thRefresh, pInfo, 0, &ThreadId);
   }
}

unsigned long __stdcall CDlgFilesystem::thRefresh(LPVOID lpvParam)
{
   // Var
   int             fserr, state, load;
   rmonRet         rc;
   tdefMediaInfo*  pMedia;
   tdefWorkInfo*   pInfo = (tdefWorkInfo*)lpvParam;
   CTreeCtrl&      rList = ((CDlgFilesystem*)pInfo->wndParent)->c_tree;
   int             media = ((CDlgFilesystem*)pInfo->wndParent)->GetSelectedMedia();
   bool&           rRefreshing   = ((CDlgFilesystem*)pInfo->wndParent)->m_refreshing;

   // Get media info
   if (media < 0 || media > MEDIA) media = 0;
   pMedia = &((CDlgFilesystem*)pInfo->wndParent)->m_media[media];

   // Open filesystem?
   if (!pMedia->opened)
   {
      // Open filesystem
      rc = rmonMediaOpen(hCon, media, &fserr);
      if (rc || fserr)
      {
         // Set media status
         pMedia->present = false;

         // Switch to ROOT item
         pInfo->dirInfo  = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

         // Empty current data
         ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
         rList.DeleteAllItems();

         // Build root item
         pInfo->dirInfo->Name = media == 0 ? _T("No Card") : _T("Not Present");
         pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
         rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);
         ((CDlgFilesystem*)pInfo->wndParent)->m_work = pInfo->dirInfo;

         // Select refresh directory
         rList.SelectItem(pInfo->dirInfo->Tree);
         ((CDlgFilesystem*)pInfo->wndParent)->m_work = pInfo->dirInfo;

         // Error
         ((CDlgFilesystem*)pInfo->wndParent)->MessageBox(_T("Failed to open media!"), _T("Refresh"), MB_OK|MB_ICONWARNING);

         // Unlock refresh
         rRefreshing = false;
         pInfo->wndParent->PostMessage(UWM_UPDATE_WINDOW, 0, 0);

         // Completed
         return 0;
      }

      // Filesystem is opened
      pMedia->opened = true;

      // Change media
      CString drv;
      drv = ((CDlgFilesystem*)pInfo->wndParent)->GetMediaDrive(media);
      do
      {
         rc = rmonDirChange(hCon, drv, &fserr);
         if (!rc && !fserr) ((CDlgFilesystem*)pInfo->wndParent)->m_cur_media = media;
         if (fserr == 122) Sleep(100);
      }
      while (!rc && fserr == 122);
      if (fserr == 102)
      {
         // Set media status
         pMedia->present   = true;
         pMedia->formatted = false;

         // Switch to ROOT item
         pInfo->dirInfo    = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

         // Empty current data
         ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
         rList.DeleteAllItems();

         // Build root item
         pInfo->dirInfo->Name = _T("Not Formatted");
         pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
         rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);

         // Select refresh directory
         rList.SelectItem(pInfo->dirInfo->Tree);

         // Unlock refresh
         rRefreshing = false;
         pInfo->wndParent->PostMessage(UWM_UPDATE_WINDOW, 0, 0);

         // Completed
         return 0;
      }
      else if (fserr == 124)
      {
         // Set media status
         pMedia->present   = true;
         pMedia->formatted = false;

         // Switch to ROOT item
         pInfo->dirInfo    = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

         // Empty current data
         ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
         rList.DeleteAllItems();

         // Build root item
         pInfo->dirInfo->Name = _T("Unknown filesystem");
         pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
         rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);

         // Select refresh directory
         rList.SelectItem(pInfo->dirInfo->Tree);

         // Unlock refresh
         rRefreshing = false;
         pInfo->wndParent->PostMessage(UWM_UPDATE_WINDOW, 0, 0);

         // Completed
         return 0;
      }
   }

   // Is media present?
   rc = rmonMediaPresent(hCon, media, &state, &fserr);
   if (rc || fserr || !state)
   {
      // Set media status
      pMedia->present = false;

      // Switch to ROOT item
      pInfo->dirInfo = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

      // Empty current data
      ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
      rList.DeleteAllItems();

      // Build root item
      if (fserr==124) {
         pInfo->dirInfo->Name = _T("Unknown filesystem");
      } else {
         pInfo->dirInfo->Name = media == 0 ? _T("No Card") : _T("Not Present");
      }
      pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
      rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);
      ((CDlgFilesystem*)pInfo->wndParent)->m_work = pInfo->dirInfo;

      // Error
      if (rc || (fserr && fserr!=124)) ((CDlgFilesystem*)pInfo->wndParent)->MessageBox(_T("Communication error!"), _T("Refresh"), MB_OK|MB_ICONWARNING);

      // Select refresh directory
      rList.SelectItem(pInfo->dirInfo->Tree);
   }
   else
   {
      // Check selected media
      if (((CDlgFilesystem*)pInfo->wndParent)->m_cur_media != media)
      {
         // Var
         CString drv;

         // Change media
         drv = ((CDlgFilesystem*)pInfo->wndParent)->GetMediaDrive(media);
         do
         {
            rc = rmonDirChange(hCon, drv, &fserr);
            if (!rc && !fserr) ((CDlgFilesystem*)pInfo->wndParent)->m_cur_media = media;
            if (fserr == 122) Sleep(100);
         }
         while (!rc && fserr == 122);
      }

      // Set media status
      pMedia->present   = true;
      pMedia->formatted = true;

      if (pInfo->dirInfo == ((CDlgFilesystem*)pInfo->wndParent)->m_root)
      {
         // Build root item
         pInfo->dirInfo->Name = _T("\\");
         pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
         rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);
      }

      // Select refresh directory
      rList.SelectItem(pInfo->dirInfo->Tree);

      // Load refresh directory contents
      load = ((CDlgFilesystem*)pInfo->wndParent)->LoadDirectory(pInfo->dstPath, pInfo->dirInfo);
      if (pInfo->dirInfo == ((CDlgFilesystem*)pInfo->wndParent)->m_root)
      {
         if (load && pInfo->dirInfo->FolderList.IsEmpty() && pInfo->dirInfo->FileList.IsEmpty())
         {
            // Set media status
            pMedia->present = false;

            // Switch to ROOT item
            pInfo->dirInfo = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

            // Empty current data
            ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
            rList.DeleteAllItems();

            // Build root item
            pInfo->dirInfo->Name = _T("No Card");
            pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
            rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);
            ((CDlgFilesystem*)pInfo->wndParent)->m_work = pInfo->dirInfo;

            // Select refresh directory
            rList.SelectItem(pInfo->dirInfo->Tree);
         }
      }
   }

   // Is dialog closed?
   if (((CDlgFilesystem*)pInfo->wndParent)->m_closing)
   {
      // Set media status
      pMedia->present = false;

      // Switch to ROOT item
      pInfo->dirInfo = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

      // Empty current data
      ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
      rList.DeleteAllItems();

      // Build root item
      pInfo->dirInfo->Name = media == 0 ? _T("No Card") : _T("Not Present");
      pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
      rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);
      ((CDlgFilesystem*)pInfo->wndParent)->m_work = pInfo->dirInfo;

      // Select refresh directory
      rList.SelectItem(pInfo->dirInfo->Tree);
   }

   // Set CWD to media root
   rmonDirChange(hCon, _T("/"), &fserr);

   // Unlock refresh
   rRefreshing = false;
   pInfo->wndParent->PostMessage(UWM_UPDATE_WINDOW, 0, 0);

   // Completed
   return 0;
}

void CDlgFilesystem::ClearDirectory(tdefFolderInfo* dirItem)
{
	// Var
	POSITION        pos;
	tdefFolderInfo* Item;
	tdefFileInfo*   FileItem;

	// Validate input
	if (!dirItem) return;

	// Iterate files
	pos = dirItem->FileList.GetHeadPosition();
	while (pos)
	{
		// Get item
		FileItem = dirItem->FileList.GetNext(pos);
		if (FileItem)
		{
			// Remove item
			delete FileItem;
		}
	}
	dirItem->FileList.RemoveAll();

	// Iterate directories
	pos = dirItem->FolderList.GetHeadPosition();
	while (pos)
	{
		// Get item
		Item = dirItem->FolderList.GetNext(pos);
		if (Item)
		{
			// Clear directory
			ClearDirectory(Item);
			// Remove item from tree
			c_tree.DeleteItem(Item->Tree);
			// Remove item
			delete Item;
		}
	}
	dirItem->FolderList.RemoveAll();
}

int CDlgFilesystem::LoadDirectory(const CString& path, tdefFolderInfo* dirItem)
{
   // Var
   char            path_cur[61], name[15];
   CString         Temp;
   int             fserr;
   long            length;
   rmonRet         rc;
   rmonRTCTime     time;
   short           index = 0;
   tdefFolderInfo* Item;
   tdefFileInfo*   FileItem;
   struct cbarg_t  cbarg;

   // Closing?
   if (m_closing) return 1;

   // Change working directory
   rc = rmonDirChange(hCon, path, &fserr);
   if (fserr == 105 && GetSelectedMedia() == 2)
   {
      // The project drive requires the full drive to be present
      rc = rmonDirChange(hCon, _T("P:") + path, &fserr);
   }
   if (fserr == 102)
   {
      // Set media status
      int media                = GetSelectedMedia();
      m_media[media].present   = true;
      m_media[media].formatted = false;

      // Empty current data
      ClearDirectory(m_root);
      c_tree.DeleteAllItems();

      // Build root item
      m_root->Name = _T("Not Formatted");
      m_root->Tree = c_tree.InsertItem(m_root->Name);
      c_tree.SetItemData(m_root->Tree, (DWORD)m_root);

      // Select refresh directory
      c_tree.SelectItem(m_root->Tree);
      m_work = m_root;

      // Completed
      return 0;
   }
   if (rc || fserr || m_closing) return 1;

   // Validate the CWD is changed
   rc = rmonDirCurrent(hCon, path_cur, &fserr);
   if (rc || fserr || m_closing) return 1;
   Temp = &path_cur[2];
   Temp.Replace(_T('/'), _T('\\'));
   if (path.Right(1) == _T('\\') && Temp.Right(1) != _T('\\')) Temp += _T("\\");
   if (path.Right(1) != _T('\\') && Temp.Right(1) == _T('\\')) Temp  = Temp.Left(Temp.GetLength() - 1);
   if (path != Temp) return 1;

   cbarg.dirItem = dirItem;
   cbarg.view    = this;

   // Iterate directory items
   rc = rmonDirCatalogX(hCon, (LPTSTR)(LPCTSTR)m_wildcard, cbcatalog, &cbarg, &fserr);
   if ((rc && rc != rmonError) || fserr || m_closing) return 1;
   if (rc)
   {
      do
      {
         // Read directory entry
         rc = rmonDirCatalog(hCon, index, name, &time, &length, &fserr);
         if (!rc && !fserr && !m_closing)
         {
            // Set entry name
            Temp = name;
            // Is this a folder?
            if( name[0] == '[' )
            {
               // Build directory info struct
               Item       = new tdefFolderInfo;
               Item->Name = Temp.Mid(1, Temp.GetLength()-2);
               Item->Tree = c_tree.InsertItem(Item->Name, dirItem->Tree, TVI_SORT);
               c_tree.SetItemData(Item->Tree, (DWORD)Item);

               // Insert directory into list
               dirItem->FolderList.AddTail(Item);
            }
            else
            {
               // Build file info struct
               FileItem       = new tdefFileInfo;
               FileItem->Name = Temp;
               FileItem->Size = length;
               FileItem->Time.Format("%02d-%02d-%04d %02d:%02d",time.date,time.month,time.year,time.hour,time.minute);

               // Insert file into list
               dirItem->FileList.AddTail(FileItem);
            }
         }
         // Next
         index++;

         // Closing?
         if (m_closing) return 1;
      }
      while (!rc && !fserr);
   }

   // Expand root
   if (path == _T("\\")) c_tree.Expand(dirItem->Tree, TVE_EXPAND);
   if (c_tree.ItemHasChildren(dirItem->Tree)) c_tree.RedrawWindow();

   // Iterate sub directories
   if (c_recursive.GetCheck())
   {
      if (!dirItem->FolderList.IsEmpty())
      {
         // Var
         POSITION pos;

         // Update path
         Temp = path;
         if (Temp.Right(1) != '\\') Temp += _T("\\");

         // Iterate directories
         pos = dirItem->FolderList.GetHeadPosition();
         while(pos)
         {
            // Get item
            Item = dirItem->FolderList.GetNext(pos);
            if (Item)
            {
               // Load root contents
               LoadDirectory(Temp + Item->Name, Item);
               if (m_closing) return 1;
            }
         }
      }
   }

   // Completed
   return 0;
}

int __stdcall CDlgFilesystem::cbcatalog(void* uptr, const char* filename, unsigned char type, long timestamp, unsigned long size)
{
  // Var
  struct cbarg_t* pDoc = (struct cbarg_t*)uptr;

  // Insert folder
  if (type == 1) {
    // Var
    tdefFolderInfo* Item;

    // Build directory info struct
    Item       = new tdefFolderInfo;
    Item->Name = filename;
    Item->Tree = pDoc->view->c_tree.InsertItem(Item->Name, pDoc->dirItem->Tree, TVI_SORT);
    pDoc->view->c_tree.SetItemData(Item->Tree, (DWORD)Item);

    // Insert directory into list
    pDoc->dirItem->FolderList.AddTail(Item);
  }
  // Insert file
  else {
    // Var
    tdefFileInfo* Item;
    CTime         time(timestamp); // timestamp is LINSEC, add 10 years when 
    CTimeSpan     offset(315532800);

    time += offset;

    // Build file info struct
    Item       = new tdefFileInfo;
    Item->Name = filename;
    Item->Size = size;
    Item->Time = time.FormatGmt("%d-%m-%Y %H:%M");

    // Insert file into list
    pDoc->dirItem->FileList.AddTail(Item);
  }

  // Reply: Continue
  return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Menu
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesystem::OnContextMenu(CWnd* pWnd, CPoint point)
{
   // Directory tree
   if (pWnd == &c_tree)
   {
      // Var
      bool      on_item = false;
      CMenu     menu, *pPopup;
      CPoint    view;
      HTREEITEM hItem;
      UINT      flags;

      // Get selected media
      int media = GetSelectedMedia();

      // Get item
      view  = point;
      c_tree.ScreenToClient(&view);
      hItem = c_tree.HitTest( view, &flags );

      // Identify item
      if (hItem != NULL && ((flags & TVHT_ONITEM ) > 0) ) on_item = true;

      // Select item
      if (on_item) c_tree.SelectItem(hItem);

      // Load menu
      VERIFY(menu.LoadMenu(IDR_POPUP_FILESYSTEM));
      pPopup = menu.GetSubMenu(1);// Get Directory submenu
      ASSERT(pPopup != NULL);

      // Disable illegal menu items
      if (!gUnitInfo.Connected || m_media[media].type == 0 || m_media[media].type == 3 || !m_media[media].present || !m_media[media].formatted || m_fileproc || !on_item || m_refreshing)
         pPopup->EnableMenuItem( IDM_DIR_CREATE, MF_BYCOMMAND|MF_GRAYED );
      if (!gUnitInfo.Connected || m_media[media].type == 0 || !m_media[media].present || !m_media[media].formatted || m_fileproc || !on_item || m_refreshing || hItem == m_root->Tree)
         pPopup->EnableMenuItem( IDM_DIR_DELETE, MF_BYCOMMAND|MF_GRAYED );
      if (!gUnitInfo.Connected || !m_media[media].present || m_fileproc || m_refreshing || (m_media[media].type != 1 && m_media[media].type != 4))
         pPopup->EnableMenuItem( IDM_MEDIA_EJECT, MF_BYCOMMAND|MF_GRAYED );
      if (!gUnitInfo.Connected || m_media[media].type == 0 || m_media[media].type == 3 || m_media[media].type == 4 || !m_media[media].present || m_fileproc || m_refreshing)
         pPopup->EnableMenuItem( IDM_MEDIA_FORMAT, MF_BYCOMMAND|MF_GRAYED );

      // Show menu
      pPopup->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, this);
   }
   // File report
   else if (pWnd == &c_list)
   {
      // Var
      bool  on_item = false;
      CMenu menu, *pPopup;

      // Get selected media
      int media = GetSelectedMedia();

      // Itentify item
      if (c_list.GetFirstSelectedItemPosition()) on_item = true;

      // Load menu
      VERIFY(menu.LoadMenu(IDR_POPUP_FILESYSTEM));
      pPopup = menu.GetSubMenu(0);// Get File submenu
      ASSERT(pPopup != NULL);

      // Disable illegal menu items
      if (!gUnitInfo.Connected || m_media[media].type == 0 || m_media[media].type == 3 || !m_media[media].present || !m_media[media].formatted || m_fileproc || m_refreshing)
         pPopup->EnableMenuItem( IDM_FILE_IMPORT, MF_BYCOMMAND|MF_GRAYED );
      if (!gUnitInfo.Connected || m_media[media].type == 0 || !m_media[media].present || !m_media[media].formatted || m_fileproc || m_refreshing || !on_item)
         pPopup->EnableMenuItem( IDM_FILE_EXPORT, MF_BYCOMMAND|MF_GRAYED );
      if (!gUnitInfo.Connected || m_media[media].type == 0 || m_media[media].type == 3 || !m_media[media].present || !m_media[media].formatted || m_fileproc || m_refreshing || !on_item)
         pPopup->EnableMenuItem( IDM_FILE_DELETE, MF_BYCOMMAND|MF_GRAYED );
      if (!gUnitInfo.Connected || m_media[media].type == 0 || m_media[media].type == 3 || !m_media[media].present || !m_media[media].formatted || m_fileproc || m_refreshing || !on_item)
         pPopup->EnableMenuItem( IDM_FILE_RENAME, MF_BYCOMMAND|MF_GRAYED );
      if (c_list.GetItemCount() == 0)
         pPopup->EnableMenuItem( IDM_FILE_SELECTALL, MF_BYCOMMAND|MF_GRAYED );

      // Show menu
      pPopup->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, this);
   }
}

/////////////////////////////////////////////////////////////////////////////
// Media
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesystem::UpdateMedia(void)
{
   // Var
   int     i, sel, pos;
   rmonRet rc;

   // Get selected media
   sel = GetSelectedMedia();

   // Monitor mode
   if (gUnitInfo.monitor)
   {
      // No medias!
      for (i = 0; i < MEDIA; i++) m_media[i].type = 0;
   }

   // Load media information
   else
   {
      // Var
      struct rmonMediaInfo media[8];

      // Perform 3 tries
      for (i=0; i<3; i++)
      {
         rc = rmonMediaInformation(hCon, media);
         if (!rc) break;
         Sleep(3000);
      }

      // Update media information
      for (i = 0; i < MEDIA; i++) m_media[i].type = media[i].Type;
   }

   // Remove media
   c_drop.ResetContent();

   // Add media
   for (i = 0; i < MEDIA; i++)
   {
      if (m_media[i].type) c_drop.AddString(m_media[i].title);
   }

   // Select media
   pos = c_drop.FindStringExact(-1, m_media[sel].title);
   if (pos == CB_ERR) pos = 0;
   c_drop.SetCurSel(pos);
   OnSelectMedia();
}

int CDlgFilesystem::GetSelectedMedia(void)
{
   // Var
   CString drv;
   int     i;

   // Get title of selected media
   i = c_drop.GetCount();
   if (i<1) return 0;
   i = c_drop.GetCurSel();
   if (i > -1) c_drop.GetLBText(i, drv);

   // Iterate medias
   for (i=0; i<MEDIA; i++)
   {
      if (!drv.Compare(m_media[i].title)) return i;
   }

   // Completed
   return 0;
}

CString CDlgFilesystem::GetMediaDrive(int media)
{
   // Validate
   if (media < 0 || media >= MEDIA) return CString();

   // Completed
   return m_media[media].title.Left(2);
}

void CDlgFilesystem::OnSelectMedia()
{
   // Var
   int media;

   // Get selected media
   media = GetSelectedMedia();

   // Change media?
   if (m_root != &m_media[media].rootdir)
   {
      // Switch to selected media
      if (m_media[media].type > 0)
      {
         CString drv;
         int     fserr, rc;
         drv = GetMediaDrive(media);
         rc  = rmonDirChange(hCon, drv, &fserr);
         if (!rc && !fserr) m_cur_media = media;
      }

      // Empty Directory tree
      c_tree.DeleteAllItems();

      // Build root item
      m_root       = &m_media[media].rootdir;
      m_root->Tree = c_tree.InsertItem(m_root->Name);
      c_tree.SetItemData(m_root->Tree, (DWORD)m_root);
      ShowDirectory(m_root);

      // Select directory
      c_tree.SelectItem(m_root->Tree);

      // Change created
      if (media == 2)
      {
         // Var
         LVCOLUMN col;
         CString  text;

         // Setup
         col.pszText    = text.GetBuffer(20);
         col.cchTextMax = 20;
         col.mask       = LVCF_TEXT;

         // Update column text
         strcpy_s(col.pszText, 20, _T("Modified"));
         c_list.SetColumn(2, &col);

         // Cleanup
         text.ReleaseBuffer();
      }
      else
      {
         // Var
         LVCOLUMN col;
         CString  text;

         // Setup
         col.pszText    = text.GetBuffer(20);
         col.cchTextMax = 20;
         col.mask       = LVCF_TEXT;

         // Update column text
         strcpy_s(col.pszText, 20, _T("Created"));
         c_list.SetColumn(2, &col);

         // Cleanup
         text.ReleaseBuffer();
      }

      // Set working directory
      m_work = m_root;
      c_tree.SelectItem(m_root->Tree);
      UpdateFileReport();
   }
}

/////////////////////////////////////////////////////////////////////////////
// Directories
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesystem::OnExpandDir(NMHDR *pNMHDR, LRESULT *pResult)
{
   // Header
   NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

   // Root?
   if( pNMTreeView->itemNew.hItem == m_root->Tree )
   {
      if( pNMTreeView->action == TVE_COLLAPSE )
      {
         *pResult = TRUE;
         return;
      }
   }
   *pResult = 0;
}

void CDlgFilesystem::OnSelectDir(NMHDR *pNMHDR, LRESULT *pResult)
{
   // Header
   NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

   // Get folder
   tdefFolderInfo* temp = (tdefFolderInfo*)c_tree.GetItemData(pNMTreeView->itemNew.hItem);
   if (temp)
   {
      // Select new working directory
      m_work = temp;
      // Update file report
      UpdateFileReport();
   }
   *pResult = 0;
}

void CDlgFilesystem::OnDeleteDir(NMHDR *pNMHDR, LRESULT *pResult)
{
   // Header
   TV_KEYDOWN* pTVKeyDown = (TV_KEYDOWN*)pNMHDR;

   // Only for VK_DELETE
   if( pTVKeyDown->wVKey == VK_DELETE )
   {
      // Delete directory
      if (!m_fileproc) OnDirDelete();
   }

   // Completed
   *pResult = 0;
}

void CDlgFilesystem::OnDirCreate()
{
   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Select new folder name
   CDlgFilesysInput dlg;
   dlg.m_type = 1;
   if (dlg.DoModal() == IDOK)
   {
      // Var
      CString path;
      rmonRet rc;
      int     fserr;
      tdefFolderInfo* Item;

      // Filesystem uses only uppercase
      dlg.m_NewName.MakeUpper();

      // Build directory path
      path = GetSelectedPath();
      if (path.Right(1) != _T('\\')) path += _T('\\');
      path += dlg.m_NewName;

      // Open filesystem?
      if (!m_media[media].opened)
      {
         // Open filesystem
         rc = rmonMediaOpen(hCon, media, &fserr);
         if (rc || fserr)
         {
            MessageBox( _T("Failed to open media!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
            return;
         }

         // Filesystem is opened
         m_media[media].opened = true;

         // Change media
         CString drv;
         drv = GetMediaDrive(media);
         rc  = rmonDirChange(hCon, drv, &fserr);
         if (!rc && !fserr) m_cur_media = media;
      }

      // Check selected media
      if (m_cur_media != media)
      {
         // Var
         CString drv;

         // Change media
         drv = GetMediaDrive(media);
         do
         {
            rc = rmonDirChange(hCon, drv, &fserr);
            if (!rc && !fserr) m_cur_media = media;
            if (fserr == 122) Sleep(100);
         }
         while (!rc && fserr == 122);
         if (rc || fserr)
         {
            MessageBox( _T("The media is not present!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
            return;
         }
      }

      // Create directory
      rc = rmonDirCreate(hCon, path, &fserr);
      if (!rc && !fserr)
      {
         // Build directory info struct
         Item = new tdefFolderInfo;
         Item->Name = dlg.m_NewName;
         Item->Tree = c_tree.InsertItem(Item->Name, m_work->Tree, TVI_SORT);
         c_tree.SetItemData(Item->Tree, (DWORD)Item);

         // Insert directory into list
         m_work->FolderList.AddTail(Item);

         // Update directory tree
         if (m_work == m_root)
         {
            c_tree.Expand(m_root->Tree, TVE_EXPAND);
         }
         else
         {
            c_tree.RedrawWindow();
         }
      }
      else
      {
         if (rc)                          MessageBox( _T("Error while communicating with device!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_INVALIDDRIVE) MessageBox( _T("Media is not opened!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_DUPLICATED)   MessageBox( _T("A Directory with this name allready exist!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_NOMOREENTRY)  MessageBox( _T("There are no more room on the media!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_CARDREMOVED)  MessageBox( _T("The media is not present!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_WRITEPROTECT) MessageBox( _T("The media is write-protected!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_NOTFORMATTED) MessageBox( _T("*** media is not formatted ***"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_CARDERROR)    MessageBox( _T("*** error on media ***"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else if (fserr==FS_MAINTENENCE)  MessageBox( _T("*** media is busy ***"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
         else                             MessageBox( _T("Unknown error!"), _T("Create Directory"), MB_OK|MB_ICONWARNING );
      }
   }
}

void CDlgFilesystem::OnDirDelete()
{
   // Var
   CString Text;

   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Delete Directory"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Check selected media
   if (m_cur_media != media)
   {
      // Var
      CString drv;
      int     rc, fserr;

      // Change media
      drv = GetMediaDrive(media);
      do
      {
         rc = rmonDirChange(hCon, drv, &fserr);
         if (!rc && !fserr) m_cur_media = media;
         if (fserr == 122) Sleep(100);
      }
      while (!rc && fserr == 122);
      if (rc || fserr)
      {
         MessageBox( _T("The media is not present!"), _T("Delete Directory"), MB_OK|MB_ICONWARNING );
         return;
      }
   }

   // Is root selected?
   if (m_work == m_root)
   {
      // Message
      MessageBox(_T("The root directory cannot be deleted!"), _T("Delete Directory"), MB_OK|MB_ICONERROR );
      return;
   }

   // Wrong folder selected?
   if (m_work->FileList.GetCount() > 0 || m_work->FolderList.GetCount() > 0)
   {
      Text  = _T("Are you sure you wish to delete\nthe directory: ") + c_tree.GetItemText(m_work->Tree);
      Text += _T("\n\nFiles and/or sub-directories\nwill also be deleted");
   }
   else
   {
      Text = _T("Are you sure you wish to delete\nthe directory: ") + c_tree.GetItemText(m_work->Tree);
   }
   if (MessageBox(Text, _T("Delete Directory"), MB_YESNO|MB_ICONWARNING|MB_DEFBUTTON2) == IDYES)
   {
      // Var
      tdefWorkInfo* pInfo;
      DWORD         ThreadId;

      // Build copy info struct
      pInfo            = new tdefWorkInfo;
      pInfo->Type      = FSF_DIR_DELETE;
      pInfo->wndParent = this;
      pInfo->wndReport = m_ReportWnd;
      pInfo->dirInfo   = m_work;

      // Start thread
      CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
   }
}

void CDlgFilesystem::OnMediaEject()
{
   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Eject media"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Ask "Are you sure?"
   if (MessageBox(_T("Are you sure you wish to eject the media?"), _T("Eject media"), MB_YESNO|MB_ICONWARNING|MB_DEFBUTTON2) == IDYES)
   {
      // Var
      tdefWorkInfo* pInfo;
      DWORD         ThreadId;

      // Build copy info struct
      pInfo            = new tdefWorkInfo;
      pInfo->Type      = FSF_MEDIA_EJECT;
      pInfo->wndParent = this;
      pInfo->wndReport = m_ReportWnd;

      // Start thread
      CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
   }
}

void CDlgFilesystem::OnMediaFormat()
{
   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Quick format media"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Ask "Are you sure?"
   if (MessageBox(_T("Are you sure you wish to quick format the media?"), _T("Quick format media"), MB_YESNO|MB_ICONWARNING|MB_DEFBUTTON2) == IDYES)
   {
      // Var
      tdefWorkInfo* pInfo;
      DWORD         ThreadId;

      // Build copy info struct
      pInfo            = new tdefWorkInfo;
      pInfo->Type      = FSF_MEDIA_FORMAT;
      pInfo->wndParent = this;
      pInfo->wndReport = m_ReportWnd;

      // Start thread
      CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
   }
}

/////////////////////////////////////////////////////////////////////////////
// File report
/////////////////////////////////////////////////////////////////////////////

void CDlgFilesystem::UpdateFileReport()
{
   // Var
   CString       Temp;
   int           Index;
   POSITION      pos;
   tdefFileInfo* Item;

   // Clear report
   c_list.DeleteAllItems();

   // Validate working directory
   if (!m_work) return;

   // Iterate files
   pos = m_work->FileList.GetHeadPosition();
   while(pos)
   {
      // Get file
      Item = m_work->FileList.GetNext(pos);
      if (Item)
      {
         // Build size string
         Temp.Format("%d",Item->Size);

         // Insert into report
         Index = c_list.InsertItem(0, Item->Name);
         c_list.SetItemText(Index, 1, Temp);
         c_list.SetItemText(Index, 2, Item->Time);
      }
   }
}

void CDlgFilesystem::OnFileDelete()
{
   // Var
   CString       Filename, Path;
   DWORD         ThreadId;
   int           Index;
   POSITION      pos;
   tdefWorkInfo* pInfo = NULL;

   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Delete file"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Get selected position
   pos = c_list.GetFirstSelectedItemPosition();
   if(!pos) return;

   // Delete files?
   if( MessageBox("Are you sure you wish to delete the file(s)?", "Delete file(s)", MB_YESNO|MB_ICONQUESTION) == IDNO ) return;

   // Build copy info struct
   pInfo            = new tdefWorkInfo;
   pInfo->dstPath   = GetSelectedPath();
   pInfo->Type      = FSF_FILE_DELETE;
   pInfo->wndParent = this;
   pInfo->wndReport = m_ReportWnd;
   pInfo->dirInfo   = m_work;
   if( pInfo->dstPath.Right(1) != _T('\\') ) pInfo->dstPath += _T('\\');

   // Iterate selected files
   while (pos)
   {
      // Get index of selected file
      Index  = c_list.GetNextSelectedItem(pos);

      // Insert into file list
      pInfo->srcFiles.AddTail( c_list.GetItemText(Index, 0) );
   }

   // Start thread
   CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
}

void CDlgFilesystem::OnFileExport()
{
   // Var
   CString     path, temp, dstPath;
   CWinApp*    pApp;
   POSITION    pos;
   BOOL        select;

   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Export file"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Get selected position
   pos = c_list.GetFirstSelectedItemPosition();
   if (!pos) return;

   // Get selected export path
   pApp = AfxGetApp();
   path = pApp->GetProfileString(szSection, _T("ExportPath"), _T(""));

   // Select export directory
   CFolderPickerDialog dlg(path);
   select  = (dlg.DoModal() == IDOK);
   dstPath = dlg.GetPathName();
   if (dstPath.Right(1) != _T('\\')) dstPath += _T('\\');
   if (select)
   {
      // Var
      CString       srcPath;
      DWORD         ThreadId;
      int           Index;
      tdefWorkInfo* pInfo;

      // Save selected export path
      AfxGetApp()->WriteProfileString(szSection, _T("ExportPath"), dstPath);

      // Build copy info struct
      pInfo            = new tdefWorkInfo;
      pInfo->dstPath   = dstPath;
      pInfo->Type      = FSF_FILE_EXPORT;
      pInfo->wndParent = this;
      pInfo->wndReport = m_ReportWnd;
      pInfo->dirInfo   = m_work;

      // Get source path
      srcPath          = GetSelectedPath();
      if( srcPath.Right(1) != _T('\\') ) srcPath += _T('\\');

      // Iterate selected files
      while (pos)
      {
         // Get index of selected file
         Index  = c_list.GetNextSelectedItem(pos);

         // Insert into file list
         pInfo->srcFiles.AddTail( srcPath + c_list.GetItemText(Index, 0) );
      }

      // Start thread
      CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
   }
}

void CDlgFilesystem::OnFileImport()
{
   // Var
   CFileDialog dlg( TRUE, NULL, NULL, OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST|OFN_ALLOWMULTISELECT|OFN_HIDEREADONLY, _T("All files (*.*)|*.*||") );
   CString     title(_T("Import files"));
   char        buf[0x8000];

   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Import file"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Set file buffer
   memset(buf, 0, sizeof(buf));
   dlg.m_ofn.lpstrFile  = buf;
   dlg.m_ofn.nMaxFile   = sizeof(buf);
   dlg.m_ofn.lpstrTitle = title;

   // Get destenation
   if (dlg.DoModal() == IDOK)
   {
      // Var
      tdefWorkInfo* pInfo;
      DWORD         ThreadId;
      POSITION      pos;

      // Build copy info struct
      pInfo            = new tdefWorkInfo;
      pInfo->dstPath   = GetSelectedPath();
      pInfo->Type      = FSF_FILE_IMPORT;
      pInfo->wndParent = this;
      pInfo->wndReport = m_ReportWnd;
      pInfo->dirInfo   = m_work;
      if (pInfo->dstPath.Right(1) != _T('\\')) pInfo->dstPath += _T('\\');

      // Iterate selected files
      pos = dlg.GetStartPosition();
      while (pos)
      {
         // Insert into file list
         pInfo->srcFiles.AddTail( dlg.GetNextPathName(pos) );
      }

      // Start thread
      CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
   }
}

void CDlgFilesystem::OnFileRename()
{
   // Var
   CDlgFilesysInput dlg;
   POSITION         pos;

   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Rename file"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Get selected position
   pos = c_list.GetFirstSelectedItemPosition();
   if(!pos) return;

   // Rename file
   dlg.m_text = c_list.GetItemText(c_list.GetNextSelectedItem(pos), 0);
   if (dlg.DoModal() == IDOK)
   {
      // Var
      CString       filePath;
      DWORD         ThreadId;
      tdefWorkInfo* pInfo = NULL;

      // Get file path
      filePath = GetSelectedPath();
      if( filePath.Right(1) != _T('\\') ) filePath += _T('\\');

      // Build copy info struct
      pInfo            = new tdefWorkInfo;
      pInfo->dstPath   = dlg.m_NewName;
      pInfo->dstPath.MakeUpper();
      pInfo->Type      = FSF_FILE_RENAME;
      pInfo->wndParent = this;
      pInfo->wndReport = m_ReportWnd;
      pInfo->dirInfo   = m_work;
      pInfo->srcFiles.AddTail(filePath+dlg.m_text);

      // Start thread
      CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);
   }
}

void CDlgFilesystem::OnFileSelectall()
{
   // Is report empty?
   if (c_list.GetItemCount() > 0)
   {
      // Iterate files
      for (int i=0; i<c_list.GetItemCount(); i++)
      {
         // Select file
         c_list.SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
      }
      // Update view
      c_list.RedrawWindow();
   }
}

void CDlgFilesystem::OnFileShow(NMHDR *pNMHDR, LRESULT *pResult)
{
   // Var
   CString       filePath;
   DWORD         ThreadId;
   POSITION      pos;
   tdefWorkInfo* pInfo = NULL;

   // Report window is busy
   if (m_fileproc) return;

   // Not connected to unit
   if (!gUnitInfo.Connected) return;

   // Get selected media
   int media = GetSelectedMedia();
   if (m_media[media].type == 0)
   {
      MessageBox( _T("The selected media is not available!"), _T("Open file"), MB_OK|MB_ICONWARNING );
      return;
   }

   // Get selected position
   pos = c_list.GetFirstSelectedItemPosition();
   if (!pos) return;

   // Get file path
   filePath = GetSelectedPath();
   if (filePath.Right(1) != _T('\\')) filePath += _T('\\');

   // Build copy info struct
   pInfo            = new tdefWorkInfo;
   pInfo->dstPath   = g_workdir + _T("show.tmp");
   pInfo->Type      = FSF_FILE_SHOW;
   pInfo->wndParent = this;
   pInfo->wndReport = m_ReportWnd;
   pInfo->dirInfo   = m_work;
   pInfo->srcFiles.AddTail(filePath+c_list.GetItemText(c_list.GetNextSelectedItem(pos), 0));

   // Start thread
   CreateThread(NULL, 0, thWork, pInfo, 0, &ThreadId);

   // Completed
   *pResult = 0;
}

void CDlgFilesystem::OnFileRemove(NMHDR *pNMHDR, LRESULT *pResult)
{
   // Header
   LV_KEYDOWN* pLVKeyDow = (LV_KEYDOWN*)pNMHDR;

   // Only for VK_DELETE
   if (pLVKeyDow->wVKey == VK_DELETE)
   {
      // Delete selected files
      if (!m_fileproc && gUnitInfo.Connected) OnFileDelete();
   }

   // Completed
   *pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
// Work
/////////////////////////////////////////////////////////////////////////////

unsigned long __stdcall CDlgFilesystem::thWork(LPVOID lpvParam)
{
  // Var
  tdefWorkInfo*      pInfo = (tdefWorkInfo*)lpvParam;
  CDlgFilesysReport* pView = (CDlgFilesysReport*)pInfo->wndReport;

  // Get selected media
  int media = ((CDlgFilesystem*)pInfo->wndParent)->GetSelectedMedia();

  // Lock file prossecing
  ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = true;
  ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

  // Open filesystem?
  if (!((CDlgFilesystem*)pInfo->wndParent)->m_media[media].opened)
  {
    // Open filesystem
    int fserr;
    int rc = rmonMediaOpen(hCon, media, &fserr);
    if (rc || fserr)
    {
      // Show error messagebox
      pView->m_ReportCancel = false;
      pView->PostMessage(UWM_UPDATE_WINDOW, 20, 8);
      while (!pView->m_ReportCancel) Sleep(100);

      // Unlock report view
      ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
      ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

      // Cleanup
      delete pInfo;

      // Completed
      return 0;
    }

    // Filesystem is opened
    ((CDlgFilesystem*)pInfo->wndParent)->m_media[media].opened = true;

    // Change media
    CString drv;
    drv = ((CDlgFilesystem*)pInfo->wndParent)->GetMediaDrive(media);
    rc  = rmonDirChange(hCon, drv, &fserr);
    if (!rc && !fserr) ((CDlgFilesystem*)pInfo->wndParent)->m_cur_media = media;
  }

  // Setup report dialog
  switch (pInfo->Type) {
    case FSF_FILE_IMPORT:  pView->m_Text = _T("Import file(s)");     break;
    case FSF_FILE_EXPORT:  pView->m_Text = _T("Export file(s)");     break;
    case FSF_FILE_DELETE:  pView->m_Text = _T("Delete file(s)");     break;
    case FSF_FILE_RENAME:  pView->m_Text = _T("Rename file");        break;
    case FSF_FILE_SHOW:    pView->m_Text = _T("Show file");          break;
    case FSF_DIR_DELETE:   pView->m_Text = _T("Delete directory");   break;
    case FSF_MEDIA_FORMAT: pView->m_Text = _T("Quick format media"); break;
    default:               pView->m_Text = _T("Report view");        break;
  }
  pView->m_ReportCancel = false;

  // Show report dialog
  if (pInfo->Type != FSF_MEDIA_EJECT) {
    pView->PostMessage( UWM_UPDATE_WINDOW, 0, 0 );
    pView->ShowWindow(SW_SHOW);
  }

  // Import file(s)
  if (pInfo->Type == FSF_FILE_IMPORT)
  {
    // Var
    POSITION      pos;
    CString       Filename, Filepath, dstPath;
    int           Start, fserr, Len, txLen;
    rmonRet       rc;
    char          state;
    DWORD         curLen, totLen;
    char          buf[12192];
    tdefFileInfo* FileInfo;
    bool          overwrite;

    // Iterate files
    pos = pInfo->srcFiles.GetHeadPosition();
    while (pos)
    {
      // Get file
      Filepath = pInfo->srcFiles.GetNext(pos);

      // Validate source file
      Start    = Filepath.ReverseFind(_T('\\'));
      Filename = Filepath.Mid(Start+1);
      Filename.MakeUpper();
      Start    = Filename.Find(_T('.'));
      if (Start > 8 || Filename.GetLength() - Start > 4)
      {
        // Show error text
        FilereportAdd(pView, _T("File \"") + Filename + _T("\" import error. Name is not in 8.3 format"));
        Sleep(100);

        // Next file
        continue;
      }

      // Build destination filename
      dstPath = pInfo->dstPath + Filename;

      // Overwrite?
      overwrite = false;
      rc        = rmonFileExists(hCon, dstPath, &state, &fserr);
      if (rc || fserr)
      {
        // Show error text
        FilereportCompleted(pView, _T("*** communication error ***"));

        // Completed
        break;
      }
      if (state)
      {
        // Show error text
        pView->m_Text         = Filename;
        pView->PostMessage( UWM_UPDATE_WINDOW, 21, 1 );
        while (!pView->m_ReportCancel) Sleep(100);
        pView->m_ReportCancel = false;
        // Overwrite file?
        if (pView->m_ReportReply == IDNO)
        {
          // Next file
          continue;
        }
        overwrite = true;
      }

			// Open destination file
			rc = rmonFileCreate(hCon, dstPath, &fserr);
			if (!rc && !fserr)
			{
				try
				{
					// Open source file
					CFile srcFile(Filepath, CFile::modeRead);

					// Show import text
          FilereportAdd(pView, _T("Importing ") + Filename);

					// Get file total size
					totLen = srcFile.GetLength();
					curLen = 0;

					// Iterate file
					do
					{
						// Read data from source
						Len   = srcFile.Read(buf, sizeof(buf));
						// Write data to destination
						txLen = 0;
						rc    = rmonFileWrite(hCon, Len, buf, &txLen, &fserr, NULL, NULL);
            if (!rc && fserr == FS_FILECLOSED)
            {
              // When unit is halted, the file is closed.
              rc = rmonFileExists(hCon, dstPath, &state, &fserr);
			        if (!rc && !fserr)
              {
                // Try to open the file again
                rc = rmonFileOpen(hCon, dstPath, &fserr);
                if (!rc && !fserr)
                {
						      // Write data to destination
						      txLen = 0;
						      rc    = rmonFileWrite(hCon, Len, buf, &txLen, &fserr, NULL, NULL);
                }
                else
                {
                  // Report original error
                  rc    = rmonOK;
                  fserr = FS_FILECLOSED;
                }
              }
              else
              {
                // Report original error
                rc    = rmonOK;
                fserr = FS_FILECLOSED;
              }
            }
						if (!rc && !fserr)
						{
							// Calculate percent
							curLen += txLen;
              FilereportProgress(pView, (int)(((float)(curLen)/(float)totLen)*100));
							// Cancel import?
							if (pView->m_ReportCancel) break;
						}
					}
					while( Len == sizeof(buf) && txLen == Len && !rc && !fserr);

					// Cancel import?
					if (pView->m_ReportCancel)
					{
						// Show error text
            FilereportCompleted(pView, _T("*** cancelled ***"));

						// Close destination file
						rmonFileClose(hCon, &fserr);

						// Delete incomplete file
						rmonFileDelete(hCon, dstPath, &fserr);

						// Completed
						break;
					}

					// Errors?
					if (rc || fserr)
					{
						// Show error text
						if (rc)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** communication error ***"));

							// Completed
							break;
						}
						else if (fserr == FS_WRITEPROTECT)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** media is writeprotected ***"));

							// Close destination file
							rmonFileClose(hCon, &fserr);

							// Delete incomplete file
							rmonFileDelete(hCon, dstPath, &fserr);
							fserr = 1; // Do not show completed message

							// Completed
							break;
						}
						else if (fserr == FS_FILECLOSED)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** media is ejected ***"));

							// Close destination file
							rmonFileClose(hCon, &fserr);

							// Delete incomplete file
							rmonFileDelete(hCon, dstPath, &fserr);
							fserr = 1; // Do not show completed message

							// Completed
							break;
						}
						else if (txLen != Len)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** media is full ***"));

							// Close destination file
							rmonFileClose(hCon, &fserr);

							// Delete incomplete file
							rmonFileDelete(hCon, dstPath, &fserr);
							fserr = 1; // Do not show completed message

							// Completed
							break;
						}
						else
						{
							// Show error text
              FilereportAdd(pView, _T("File \"") + Filename + _T("\" import error. Destination file not open"));

							// Close destination file
							rmonFileClose(hCon, &fserr);

							// Delete incomplete file
							rmonFileDelete(hCon, dstPath, &fserr);
							fserr = 1; // Do not show completed message
						}

						// Next file
						continue;
					}
					if (txLen != Len)
					{
						// Show error text
            FilereportCompleted(pView, _T("*** media is full ***"));

						// Close destination file
						rmonFileClose(hCon, &fserr);

						// Delete incomplete file
						rmonFileDelete(hCon, dstPath, &fserr);
						fserr = 1; // Do not show completed message

						// Completed
						break;
					}

					// Build file info struct
               if (!overwrite)
               {
					FileInfo       = new tdefFileInfo;
					FileInfo->Name = Filename;
					FileInfo->Size = srcFile.GetLength();
					FileInfo->Time = CTime::GetCurrentTime().Format("%d-%m-%Y %H:%M");

					// Insert file into directory list
					pInfo->dirInfo->FileList.AddTail(FileInfo);
               }
				}
				catch( CFileException* e )
				{
					// Cleanup
					e->Delete();
				}

				// Close destination file
				rmonFileClose(hCon, &fserr);
			}
			else
			{
				// Show error text
				if (rc)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** communication error ***"));

					// Completed
					break;
				}
				else if (fserr == FS_WRITEPROTECT)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is writeprotected ***"));

					// Completed
					break;
				}
				else if (fserr == FS_NOTFORMATTED)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not formatted ***"));

					// Completed
					break;
				}
				else if (fserr==FS_INVALIDDRIVE || fserr==FS_CARDREMOVED)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not present ***"));

					// Completed
					break;
				}
				else if (fserr==FS_CARDERROR)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** error on media ***"));

					// Completed
					break;
				}
				else if (fserr==FS_CARDERROR)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is busy ***"));

					// Completed
					break;
				}
				else
				{
					// Show error text
          if (pos) {
            FilereportAdd(pView, _T("File \"") + Filename + _T("\" import error. Could not create destination file"));
          } else {
            FilereportCompleted(pView, _T("File \"") + Filename + _T("\" import error. Could not create destination file"));
          }

					// Completed
					continue;
				}

				// Next file
				continue;
			}
    }

    if (!rc && !fserr && !pView->m_ReportCancel)
    {
      // Set import completed text
      FilereportCompleted(pView, _T("*** files imported ***"));
    }

    // Update file report
    pInfo->wndParent->PostMessage( UWM_UPDATE_WINDOW, 1, 0 );

    // Wait for done
    pView->m_ReportCancel = false;
    while (!pView->m_ReportCancel) Sleep(100);

    // Unlock file prossecing
    ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
    ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

    // Cleanup
    pView->ShowWindow(SW_HIDE);
    delete pInfo;

    // Completed
    return 0;
  }

	// Export file(s)
	else if (pInfo->Type == FSF_FILE_EXPORT)
	{
		// Var
		char        buf[12192];
		CFileStatus status;
		CString     Filename, Filepath, dstPath;
		long        curLen, totLen;
		int         Start, fserr, Len;
		POSITION    pos;
		rmonRet     rc;
		bool        error = false;

		// Iterate files
		pos = pInfo->srcFiles.GetHeadPosition();
		while (pos)
		{
			// Get file
			Filepath = pInfo->srcFiles.GetNext(pos);

			// Get source filename
			Start    = Filepath.ReverseFind(_T('\\'));
			Filename = Filepath.Mid(Start+1);

			// Build destination filename
			dstPath  = pInfo->dstPath + Filename;

			// Overwrite?
			try
			{
				if (CFile::GetStatus(dstPath, status))
				{
					// Ask for overwrite
					pView->m_ReportCancel = false;
					pView->m_Text         = Filename;
					pView->PostMessage( UWM_UPDATE_WINDOW, 21, 1 );
					while (!pView->m_ReportCancel) Sleep(100);
					pView->m_ReportCancel = false;

					// Overwrite file?
					if (pView->m_ReportReply == IDNO)
					{
						// Next file
						continue;
					}
				}
			}
			catch( CFileException* e )
			{
				// Cleanup
				e->Delete();
			}

			// Open source file
			rc = rmonFileOpen(hCon, Filepath, &fserr);
			if (!rc && !fserr)
			{
				try
				{
					// Open destination file
					CFile srcFile(dstPath, CFile::modeWrite|CFile::modeCreate);

					// Show export text
          FilereportAdd(pView, _T("Exporting ") + Filename);

					// Get file total size
					rc = rmonFilePosition(hCon, &totLen, &fserr);
					if (rc || fserr)
					{
						// Show error text
						if (rc)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** communication error ***"));

							// Completed
							break;
						}
						else
						{
							// Show error text
              FilereportAdd(pView, _T("File \"") + Filename + _T("\" export error. Source file not open"));
							error = true;

							// Close source file
							rmonFileClose(hCon, &fserr);

							// Close destination file
							srcFile.Close();

							// Remove incomplete file
							CFile::Remove(dstPath);
						}

						// Next file
						continue;
					}
					curLen = 0;

					// Move to start of file
					rc = rmonFileSeek(hCon, 0, &fserr);
					if (rc || fserr)
					{
						// Show error text
						if (rc)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** communication error ***"));

							// Completed
							break;
						}
						else
						{
							// Show error text
              FilereportAdd(pView, _T("File \"") + Filename + _T("\" export error. Source file not open"));
							error = true;

							// Close source file
							rmonFileClose(hCon, &fserr);

							// Close destination file
							srcFile.Close();

							// Remove incomplete file
							CFile::Remove(dstPath);
						}

						// Next file
						continue;
					}

					// Iterate file
					do
					{
						// Read data from source
						rc = rmonFileRead(hCon, sizeof(buf), buf, &Len, &fserr, NULL, NULL);
						if (!rc && !fserr)
						{
							// Write data to destination
							srcFile.Write(buf, Len);

							// Calculate percent
							curLen += Len;
              FilereportProgress(pView, (int)(((float)(curLen)/(float)totLen)*100));

							// Cancel export?
							if (pView->m_ReportCancel) break;
						}
					}
					while( Len == sizeof(buf) && !rc && !fserr);

					// Cancel export?
					if (pView->m_ReportCancel)
					{
						// Show error text
            FilereportCompleted(pView, _T("*** cancelled ***"));

						// Close source file
						rmonFileClose(hCon, &fserr);

						// Close destination file
						srcFile.Close();

						// Remove incomplete file
						CFile::Remove(dstPath);

						// Completed
						break;
					}

					// Errors?
					if (rc || fserr)
					{
						// Show error text
						if (rc)
						{
							// Show error text
              FilereportCompleted(pView, _T("*** communication error ***"));

							// Close destination file
							srcFile.Close();

							// Remove incomplete file
							CFile::Remove(dstPath);

							// Completed
							break;
						}
						else
						{
							// Show error text
              FilereportAdd(pView, _T("File \"") + Filename + _T("\" export error. Source file not open"));
							error = true;

							// Close source file
							rmonFileClose(hCon, &fserr);

							// Close destination file
							srcFile.Close();

							// Remove incomplete file
							CFile::Remove(dstPath);
						}

						// Next file
						continue;
					}
				}
				catch (CFileException* e)
				{
					// Error message
					pView->m_ReportCancel = false;
               if (e->m_cause == CFileException::accessDenied)
               {
						// Show error text
                  FilereportCompleted(pView, _T("*** destination drive is write protected ***"));

						// Close source file
						rmonFileClose(hCon, &fserr);

						// Cleanup
						e->Delete();

						// Completed
						break;
               }
					if (e->m_cause == CFileException::fileNotFound) pView->PostMessage( UWM_UPDATE_WINDOW, 20, 6 );
					{
						// Show error text
            FilereportCompleted(pView, _T("*** destination drive is full ***"));

						// Close source file
						rmonFileClose(hCon, &fserr);

						// Cleanup
						e->Delete();

						// Completed
						break;
					}

					// Show error text
          FilereportAdd(pView, _T("File \"") + Filename + _T("\" export error. Unknown error"));
					error = true;

					// Close source file
					rmonFileClose(hCon, &fserr);

					// Cleanup
					e->Delete();

					// Completed
					continue;
				}

				// Close source file
				rmonFileClose(hCon, &fserr);
			}
			else
			{
				// Show error text
				if (rc)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** communication error ***"));

					// Completed
					break;
				}
				else if (fserr==FS_NOTFORMATTED)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not formatted ***"));

					// Completed
					break;
				}
				else if (fserr==FS_INVALIDDRIVE || fserr==FS_CARDREMOVED)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not present ***"));

					// Completed
					break;
				}
				else if (fserr==FS_CARDERROR)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** error on media ***"));

					// Completed
					break;
				}
				else if (fserr==FS_CARDERROR)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is busy ***"));

					// Completed
					break;
				}
				else
				{
					// Show error text
          FilereportAdd(pView, _T("File \"") + Filename + _T("\" export error. Could not open source file"));
					error = true;

					// Completed
					continue;
				}
			}
		}

		if (!rc && !fserr && !pView->m_ReportCancel)
		{
			// Set export completed text
			if (error) FilereportCompleted(pView, _T("*** files exported, with errors ***"));
			else       FilereportCompleted(pView, _T("*** files exported ***"));
		}

		// Update file report
		pInfo->wndParent->PostMessage( UWM_UPDATE_WINDOW, 1, 0 );

		// Wait for done
		pView->m_ReportCancel = false;
		while (!pView->m_ReportCancel) Sleep(100);

		// Unlock file prossecing
		((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
		((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

		// Cleanup
		pView->ShowWindow(SW_HIDE);
		delete pInfo;

		// Completed
		return 0;
	}

  // Delete file(s)
  else if (pInfo->Type == FSF_FILE_DELETE)
  {
    // Var
    CString  Filename, Filepath;
    int      fserr, res_msg = 0;
    POSITION pos;
    rmonRet  rc;

		// Iterate files
		pos = pInfo->srcFiles.GetHeadPosition();
		while (pos)
		{
			// Get file
			Filename = pInfo->srcFiles.GetNext(pos);

			// Build destination filename
			Filepath = pInfo->dstPath + Filename;

			// Clear message flag
			pView->m_ReportCancel = false;

			// Delete file
			rc = rmonFileDelete(hCon, Filepath, &fserr);
			if (!rc && !fserr)
			{
				// Show delete text
        FilereportAdd(pView, Filename + _T(" deleted"));

				// Remove file from directory list
				((CDlgFilesystem*)pInfo->wndParent)->RemoveFile(pInfo->dirInfo, Filename);

				// Cancel import?
				if (pView->m_ReportCancel)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** cancelled ***"));
					res_msg       = -1; // No msg

					// Close destination file
					rmonFileClose(hCon, &fserr);

					// Completed
					break;
				}
			}
			else
			{
				// Show error text
				if (rc)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** communication error ***"));
					res_msg       = -1; // No msg

					// Completed
					break;
				}
				else if (fserr == FS_WRITEPROTECT)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is writeprotected ***"));
					res_msg       = -1; // No msg

					// Completed
					break;
				}
				else if (fserr == FS_NOTFORMATTED)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not formatted ***"));
					res_msg       = -1; // No msg

					// Completed
					break;
				}
				else if (fserr==FS_CARDERROR)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** error on media ***"));

					// Completed
					break;
				}
				else if (fserr==FS_CARDERROR)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is busy ***"));

					// Completed
					break;
				}
				else if (fserr == FS_NOTFOUND)
				{
					// Show error text
          FilereportAdd(pView, _T("File \"") + Filename + _T("\" delete error. File not found"));
					res_msg       = 1; // Not all files was deleted
				}
				else if (fserr == FS_LOCKED)
				{
					// Show error text
          FilereportAdd(pView, _T("File \"") + Filename + _T("\" delete error. File is in use"));
					res_msg       = 1; // Not all files was deleted
				}
				else
				{
					// Show error text
          FilereportAdd(pView, _T("File \"") + Filename + _T("\" delete error. Unknown error"));
					res_msg       = 1; // Not all files was deleted
				}

				// Cancel import?
				if (pView->m_ReportCancel)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** cancelled ***"));
					res_msg       = -1; // No msg

					// Close destination file
					rmonFileClose(hCon, &fserr);

					// Completed
					break;
				}
			}
		}

    // Completed message
    Sleep(100);
    if (!res_msg)
    {
      // Set import completed text
      FilereportCompleted(pView, _T("*** files deleted ***"));
    }
    else if (res_msg > 0)
    {
      // Set import completed text
      FilereportCompleted(pView, _T("*** not all files was deleted ***"));
    }

    // Update file report
    pInfo->wndParent->PostMessage( UWM_UPDATE_WINDOW, 1, 0 );

    // Wait for done
    pView->m_ReportCancel = false;
    while (!pView->m_ReportCancel) Sleep(100);

    // Unlock file prossecing
    ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
    ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

    // Cleanup
    pView->ShowWindow(SW_HIDE);
    delete pInfo;

    // Completed
    return 0;
  }

  // Rename file
  else if (pInfo->Type == FSF_FILE_RENAME)
  {
    // Var
    CString       Filename, Filepath;
    int           fserr;
    POSITION      pos;
    rmonRet       rc;
    tdefFileInfo* FileItem;

    // Get source file
    Filepath = pInfo->srcFiles.GetHead();
    fserr    = Filepath.ReverseFind(_T('\\'));
    Filename = Filepath.Mid(fserr+1);

    // Show rename text
    FilereportAdd(pView, _T("Renaming \"") + Filename + _T("\""));

    // Rename file
    rc = rmonFileRename(hCon, Filepath, pInfo->dstPath, &fserr);
    if (!rc && !fserr)
    {
      // Iterate files
      pos = pInfo->dirInfo->FileList.GetHeadPosition();
      while (pos)
      {
        // Get item
        FileItem = pInfo->dirInfo->FileList.GetNext(pos);
        if (FileItem)
        {
          // Compare filename
          if (FileItem->Name == Filename)
          {
            // Rename item
            FileItem->Name = pInfo->dstPath;
            break;
          }
        }
      }

      // Update file report
      pInfo->wndParent->PostMessage( UWM_UPDATE_WINDOW, 1, 0 );

      // Set 'completed' text
      FilereportCompleted(pView, _T("*** File renamed ***"));
    }
    else
    {
      // Show error message
      if (rc)                            FilereportCompleted(pView, _T("*** communication error ***"));
      else if (fserr == FS_WRITEPROTECT) FilereportCompleted(pView, _T("*** media is writeprotected ***"));
      else if (fserr == FS_NOTFORMATTED) FilereportCompleted(pView, _T("*** media is not formatted ***"));
      else if (fserr == FS_NOTFOUND)     FilereportCompleted(pView, _T("File \"") + Filename + _T("\" rename error. File not found"));
      else if (fserr == FS_LOCKED)       FilereportCompleted(pView, _T("File \"") + Filename + _T("\" rename error. File is in use"));
      else if (fserr == FS_DUPLICATED)   FilereportCompleted(pView, _T("File \"") + Filename + _T("\" rename error. A file with the name already exists"));
      else if (fserr == FS_INVALIDDRIVE) FilereportCompleted(pView, _T("*** media is not present ***"));
      else if (fserr == FS_CARDREMOVED)  FilereportCompleted(pView, _T("*** media is not present ***"));
      else if (fserr == FS_CARDERROR)    FilereportCompleted(pView, _T("*** error on media ***"));
      else if (fserr == FS_MAINTENENCE)  FilereportCompleted(pView, _T("*** media is busy ***"));
      else                               FilereportCompleted(pView, _T("File \"") + Filename + _T("\" rename error. Unknown error"));
    }

    // Wait for done
    pView->m_ReportCancel = false;
    while (!pView->m_ReportCancel) Sleep(100);

    // Unlock file prossecing
    ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
    ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

    // Cleanup
    pView->ShowWindow(SW_HIDE);
    delete pInfo;

    // Completed
    return 0;
  }

  // Show file
  else if (pInfo->Type == FSF_FILE_SHOW)
  {
    // Var
    char     buf[12192];
    CString  Filename, Filepath;
    int      fserr, Len, resMsg = 0;
    long     curLen, totLen;
    rmonRet  rc;

    // Get source file
    Filepath = pInfo->srcFiles.GetHead();
    fserr    = Filepath.ReverseFind(_T('\\'));
    Filename = Filepath.Mid(fserr+1);

    // Open source file
    rc = rmonFileOpen(hCon, Filepath, &fserr);
    if (!rc && !fserr)
    {
      // Get file total size
      rc = rmonFilePosition(hCon, &totLen, &fserr);
      if (!rc && !fserr)
      {
        // Reset current length
        curLen = 0;

        // Move to start of file
        rc = rmonFileSeek(hCon, 0, &fserr);
        if (!rc && !fserr)
        {
          try
          {
            // Open source file
            CFile srcFile(pInfo->dstPath, CFile::modeWrite|CFile::modeCreate);

            // Show export text
            FilereportAdd(pView, _T("Loading ") + Filename);

            // Iterate file
            do
            {
              // Read data from source
              rc = rmonFileRead(hCon, sizeof(buf), buf, &Len, &fserr, NULL, NULL);
              if (!rc && !fserr)
              {
                // Write data to destination
                srcFile.Write(buf, Len);

                // Calculate percent
                curLen += Len;
                FilereportProgress(pView, (int)(((float)(curLen)/(float)totLen)*100));

                // Cancel import?
                if (pView->m_ReportCancel)
                {
                  // Show error text
                  FilereportCompleted(pView, _T("*** cancelled ***"));
                  resMsg        = 1;

                  // Completed
                  break;
                }
              }
            }
            while( Len == sizeof(buf) && !rc && !fserr);

            // Errors?
            if ((rc || fserr || curLen<totLen) && !pView->m_ReportCancel)
            {
              // Show error text
              if (rc) FilereportCompleted(pView, _T("*** communication error ***"));
              else    FilereportCompleted(pView, _T("File \"") + Filename + _T("\" load error. File has been closed"));
              resMsg = 2;
            }
          }
          catch (CFileException* e)
          {
            // Error message
            if (e->m_cause == CFileException::fileNotFound) FilereportCompleted(pView, _T("*** destination drive is full ***"));
            else                                            FilereportCompleted(pView, _T("File \"") + Filename + _T("\" load error. Unknown error at destination"));
            resMsg = 2;

            // Cleanup
            e->Delete();
          }
        }
        else
        {
          // Show error text
          if (rc) FilereportCompleted(pView, _T("*** communication error ***"));
          else    FilereportCompleted(pView, _T("File \"") + Filename + _T("\" load error. File may be in use"));
          pView->PostMessage( UWM_UPDATE_WINDOW, 3, 0 );
          resMsg = 2;
        }
      }
      else
      {
        // Show error text
        if (rc) FilereportCompleted(pView, _T("*** communication error ***"));
        else    FilereportCompleted(pView, _T("File \"") + Filename + _T("\" load error. File may be in use"));
        resMsg = 2;
      }

      // Close source file
      if (!rc) rmonFileClose(hCon, &fserr);
    }
    else
    {
      // Show error text
      if (rc)                            FilereportCompleted(pView, _T("*** communication error ***"));
      else if (fserr == FS_NOTOPEN)      FilereportCompleted(pView, _T("File \"") + Filename + _T("\" load error. File may be in use or not present"));
      else                               FilereportCompleted(pView, _T("File \"") + Filename + _T("\" load error. Unknown error"));
      resMsg = 2;
    }

    // Wait for user close?
    if (resMsg)
    {
      // Remove temporary file
      try
      {
        CFile::Remove(pInfo->dstPath);
      }
      catch (CFileException* e)
      {
        // Cleanup
        e->Delete();
      }

      // Wait for done
      pView->m_ReportCancel = false;
      while (!pView->m_ReportCancel) Sleep(100);
    }

    // Unlock file prossecing
    ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
    ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

    // Cleanup
    pView->ShowWindow(SW_HIDE);
    if (!resMsg)
    {
      // Show dialog (not if cancelled)
      ((CDlgFilesystem*)pInfo->wndParent)->PostMessage( UWM_UPDATE_WINDOW, 2, (LPARAM)pInfo );
    }
    else
    {
      // Cleanup
      delete pInfo;
    }

    // Completed
    return 0;
  }

  // Delete directory
  else if (pInfo->Type == FSF_DIR_DELETE)
  {
    // Delete selected directory
    int rc = ((CDlgFilesystem*)pInfo->wndParent)->fsfDirDelete(pInfo->dirInfo);

    // Show 'completed' text
    if (rc < 2)       FilereportCompleted(pView, _T("*** delete completed ***"));
    else if (rc == 6) FilereportCompleted(pView, _T("*** delete failed, directory is not empty ***"));

    // Wait for done
    while (!pView->m_ReportCancel) Sleep(100);

    // Unlock report view
    ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
    ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

    // Cleanup
    pView->ShowWindow(SW_HIDE);
    delete pInfo;

    // Completed
    return 0;
	}

	// Eject media
	else if (pInfo->Type == FSF_MEDIA_EJECT)
	{
		// Var
		int fserr;

		// Eject media
		rmonRet rc = rmonMediaEject(hCon, media, &fserr);
		if (rc || fserr)
		{
			// Set error source
			pView->m_Text         = _T("Eject Media");
			pView->m_ReportCancel = false;

			// Show error text
			if (rc)                          pView->PostMessage( UWM_UPDATE_WINDOW, 20, 10 );
			else if (fserr==FS_CARDREMOVED)  pView->PostMessage( UWM_UPDATE_WINDOW, 20, 11 );
			else if (fserr==FS_INVALIDDRIVE) pView->PostMessage( UWM_UPDATE_WINDOW, 20, 13 );
			else                             pView->PostMessage( UWM_UPDATE_WINDOW, 20, 19 );
			while (!pView->m_ReportCancel) Sleep(100);
		}
		else
		{
			// Var
			CTreeCtrl& rList = ((CDlgFilesystem*)pInfo->wndParent)->c_tree;

			// Switch to ROOT item
			pInfo->dirInfo = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

			// Empty current data
			((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
			rList.DeleteAllItems();

			// Build root item
			pInfo->dirInfo->Name = media == 0 ? _T("No Card") : _T("Not Present");;
			pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
			rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);

			// Select refresh directory
			rList.SelectItem(pInfo->dirInfo->Tree);
		}

		// Unlock report view
		((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
		((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

		// Cleanup
		delete pInfo;

		// Completed
		return 0;
	}

  // Quick format media
  else if (pInfo->Type == FSF_MEDIA_FORMAT)
  {
    // Var
    CString drv, tooltip;
    int     fserr;

    // Read media name
    drv = ((CDlgFilesystem*)pInfo->wndParent)->GetMediaDrive(media);
    FilereportAdd(pView, _T("Formatting ") + drv);
    if (media==1)
    {
       FilereportAdd(pView, _T("This can take a few minutes"));
    }
    else
    {
       FilereportAdd(pView, _T("This can take a few minutes, please don't"));
       FilereportAdd(pView, _T("remove the media until the format is finished."));
    }
    FilereportDisable(pView, true);

    // Quick format media
    rmonRet rc = rmonMediaQuickformat(hCon, media, &fserr);
    FilereportDisable(pView, false);
    if (rc || fserr) {
      // Show error text
      if (rc) {
        if (gUnitInfo.TargetID < 200 && (gUnitInfo.FirmwareVer < 284 || (gUnitInfo.FirmwareVer > 8000 && gUnitInfo.FirmwareVer < 8284))) {
                                       FilereportAdd(pView, _T("Timeout waiting for quickformat to complete."));
        } else {
                                       FilereportAdd(pView, _T("*** communication error ***"));
        }
                                       FilereportCompleted(pView, _T("Operation continue in the device."));
      }
      else if (fserr==FS_CARDREMOVED)  FilereportCompleted(pView, _T("*** media is not present ***"));
      else if (fserr==FS_INVALIDDRIVE) FilereportCompleted(pView, _T("*** media is not present ***"));
      else if (fserr==FS_WRITEPROTECT) FilereportCompleted(pView, _T("*** media is writeprotected ***"));
      else if (fserr==FS_CARDERROR)    FilereportCompleted(pView, _T("*** error on media ***"));
      else if (fserr==FS_MAINTENENCE)  FilereportCompleted(pView, _T("*** media is busy ***"));
      else                             FilereportCompleted(pView, _T("Quick format media error. Unknown error"));
    } else {
      // Var
      CTreeCtrl& rList = ((CDlgFilesystem*)pInfo->wndParent)->c_tree;

      // Switch to ROOT item
      pInfo->dirInfo = ((CDlgFilesystem*)pInfo->wndParent)->m_root;

      // Empty current data
      ((CDlgFilesystem*)pInfo->wndParent)->ClearDirectory(pInfo->dirInfo);
      rList.DeleteAllItems();

      // Build root item
      pInfo->dirInfo->Name = _T("\\");
      pInfo->dirInfo->Tree = rList.InsertItem(pInfo->dirInfo->Name);
      rList.SetItemData(pInfo->dirInfo->Tree, (DWORD)pInfo->dirInfo);

      // Select refresh directory
      rList.SelectItem(pInfo->dirInfo->Tree);

      // Show completed messagebox
      FilereportCompleted(pView, _T("*** Media is formatted ***"));
    }

    // Wait for done
    while (!pView->m_ReportCancel) Sleep(100);

    // Unlock report view
    ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
    ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

    // Cleanup
    pView->ShowWindow(SW_HIDE);
    delete pInfo;

    // Completed
    return 0;
  }

  // Wait for user close?
  if (pInfo->Type != FSF_FILE_SHOW)
  {
    // Show 'completed' text
    pView->PostMessage( UWM_UPDATE_WINDOW, 3, 0 );
    pView->m_ReportCancel = false;

    // Wait for done
    while (!pView->m_ReportCancel) Sleep(100);
  }

  // Unlock file prossecing
  ((CDlgFilesystem*)pInfo->wndParent)->m_fileproc = false;
  ((CDlgFilesystem*)pInfo->wndParent)->OnUpdateWindow(0, 0);

  // Cleanup
  if (pInfo->Type == FSF_FILE_SHOW)
  {
    // Cleanup
    pView->ShowWindow(SW_HIDE);

    // Show dialog (not if cancelled)
    if (!pView->m_ReportCancel)
    {
      ((CDlgFilesystem*)pInfo->wndParent)->PostMessage( UWM_UPDATE_WINDOW, 2, (LPARAM)pInfo );
    }
    else
    {
      // Cleanup
      delete pInfo;
    }
  }
  else
  {
    // Cleanup
    pView->ShowWindow(SW_HIDE);
    delete pInfo;
  }

  // Completed
  return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Utility
/////////////////////////////////////////////////////////////////////////////

CString CDlgFilesystem::GetSelectedPath()
{
	// Var
	CString   path, temp;
	HTREEITEM hItem;

	// Get Selected directory
	hItem = c_tree.GetSelectedItem();

	// Iterate path
	while(hItem)
	{
		// Add curent to path
		temp = c_tree.GetItemText(hItem);
		if(hItem != m_root->Tree) temp += _T('\\');
		path = temp + path;

		// Next
		hItem = c_tree.GetParentItem(hItem);
	}

	// Completed
	return path;
}

CString CDlgFilesystem::GetPath(tdefFolderInfo* dirItem)
{
	// Var
	CString   path, temp;
	HTREEITEM hItem;

	// Get directory tree handle
	hItem = dirItem->Tree;

	// Iterate path
	while(hItem)
	{
		// Add curent to path
		temp = c_tree.GetItemText(hItem);
		if(hItem != m_root->Tree) temp += _T('\\');
		path = temp + path;

		// Next
		hItem = c_tree.GetParentItem(hItem);
	}

	// Completed
	return path;
}

void CDlgFilesystem::ShowDirectory(tdefFolderInfo* dirItem)
{
	// Var
	tdefFolderInfo* Item;
	POSITION		pos;

	// Iterate directories
	pos = dirItem->FolderList.GetHeadPosition();
	while (pos)
	{
		Item       = dirItem->FolderList.GetNext(pos);
		Item->Tree = c_tree.InsertItem(Item->Name, dirItem->Tree, TVI_SORT);
		c_tree.SetItemData(Item->Tree, (DWORD)Item);
		ShowDirectory(Item);
	}

	// Expand root
	if (dirItem == m_root) c_tree.Expand(dirItem->Tree, TVE_EXPAND);
	if (c_tree.ItemHasChildren(dirItem->Tree)) c_tree.RedrawWindow();
}

void CDlgFilesystem::RemoveDirectory(tdefFolderInfo* dirItem)
{
	// Var
	HTREEITEM       parent;
	POSITION        pos, posDel;
	tdefFolderInfo* Item, *parItem;

	// Validate input
	if (!dirItem) return;

	// Clear directory
	ClearDirectory(dirItem);
   Item = nullptr;

	// Get parent
	parent = c_tree.GetParentItem(dirItem->Tree);
	if (parent)
	{
		// Get parent info
		parItem = (tdefFolderInfo*)c_tree.GetItemData(parent);
		if (parItem)
		{
			// Iterate directories
			pos = parItem->FolderList.GetHeadPosition();
			while(pos)
			{
				// Set delete position
				posDel = pos;
				// Get item
				Item = parItem->FolderList.GetNext(pos);
				if (Item == dirItem)
				{
					// Remove from list
					parItem->FolderList.RemoveAt(posDel);
					// Completed
					break;
				}
			}
		}
	}

	// Remove item from tree
	if (Item) c_tree.DeleteItem(Item->Tree);

	// Remove item
	delete dirItem;
}

void CDlgFilesystem::RemoveFile(tdefFolderInfo* dirItem, const CString& filename)
{
	// Var
	POSITION      pos, posDel;
	tdefFileInfo* FileItem;

	// Validate input
	if (!dirItem) return;

	// Iterate files
	pos = dirItem->FileList.GetHeadPosition();
	while(pos)
	{
		// Save position
		posDel = pos;
		// Get item
		FileItem = dirItem->FileList.GetNext(pos);
		if (FileItem)
		{
			// Compare filename
			if (FileItem->Name == filename)
			{
				// Remove item
				dirItem->FileList.RemoveAt(posDel);
				delete FileItem;
				// Completed
				return;
			}
		}
	}
}

int CDlgFilesystem::fsfDirDelete(tdefFolderInfo* pDirItem)
{
		// Var
	CString            path;
	int                fserr;
	rmonRet            rc;
	CDlgFilesysReport* pView = (CDlgFilesysReport*)m_ReportWnd;

	// Validate param
	if (!pDirItem)
	{
		// Show error text
    FilereportAdd(pView, _T("No directory selected"));

		// Completed
		return 1;
	}

	// Clear message flag
	pView->m_ReportCancel = false;

	// Does the directory have sub-directories?
	if (pDirItem->FolderList.GetCount() > 0)
	{
		// Var
		POSITION        pos;
		tdefFolderInfo* pItem;
		int             rc;

		// Iterate sub-directories
		pos = pDirItem->FolderList.GetHeadPosition();
		while (pos)
		{
			// Get sub-directory
			pItem = pDirItem->FolderList.GetNext(pos);

			// Delete sub-directory
			rc = fsfDirDelete(pItem);
			if (rc > 1 && rc != 6) return rc;

			// Cancel delete?
			if (pView->m_ReportCancel)
			{
				// Show error text
        FilereportCompleted(pView, _T("*** cancelled ***"));

				// Completed
				return 2;
			}
		}
	}

	// Get path
	path = GetPath(pDirItem);
	if (path.Right(1) != _T('\\')) path += _T('\\');

	// Does the directory have files?
	if (pDirItem->FileList.GetCount() > 0)
	{
		// Var
		POSITION      pos;
		tdefFileInfo* pItem;
		CString       Filepath;

		// Iterate files
		pos = pDirItem->FileList.GetHeadPosition();
		while (pos)
		{
			// Get file
			pItem = pDirItem->FileList.GetNext(pos);

			// Build destination filename
			Filepath = path + pItem->Name;

			// Delete file
			rc = rmonFileDelete(hCon, Filepath, &fserr);
			if (!rc && !fserr)
			{
				// Show delete text
        FilereportAdd(pView, _T("File \"") + pItem->Name + _T("\" deleted"));

				// Cancel delete?
				if (pView->m_ReportCancel)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** cancelled ***"));

					// Completed
					return 2;
				}
			}
			else
			{
				// Show error text
				if (rc)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** communication error ***"));

					// Completed
					return 3;
				}
				else if (fserr == FS_WRITEPROTECT)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is writeprotected ***"));

					// Completed
					return 4;
				}
				else if (fserr == FS_NOTFORMATTED)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not formatted ***"));

					// Completed
					return 7;
				}
				else if (fserr == FS_NOTFOUND)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** media is not present ***"));

					// Completed
					return 5;
				}
				else if (fserr == FS_LOCKED) FilereportAdd(pView, _T("File \"") + pItem->Name + _T("\" delete error, sharing violation"));
				else                         FilereportAdd(pView, _T("File \"") + pItem->Name + _T("\" delete error, unknown error"));

				// Cancel delete?
				if (pView->m_ReportCancel)
				{
					// Show error text
          FilereportCompleted(pView, _T("*** cancelled ***"));

					// Completed
					return 2;
				}
			}
		}
	}

	// Remove trailing '\\'
	path = path.Left( path.GetLength() - 1 );

	// Remove directory
	rc = rmonDirDelete(hCon, path, &fserr);
	if (!rc && !fserr)
	{
		// Switch selected
		if (m_work == pDirItem) c_tree.SelectItem(m_root->Tree);

		// Show completed text
    FilereportAdd(pView, _T("Directory \"") + pDirItem->Name + _T("\" deleted"));

		// Remove directory, sub-directories and files
		RemoveDirectory(pDirItem);
	}
	else
	{
		if (rc)
		{
			// Show error text
      FilereportCompleted(pView, _T("*** communication error ***"));

			// Completed
			return 3;
		}
		else if (fserr==FS_INVALIDDRIVE || fserr==FS_CARDREMOVED)
		{
			// Show error text
      FilereportCompleted(pView, _T("*** media is not present ***"));

			// Completed
			return 5;
		}
		else if (fserr==FS_WRITEPROTECT)
		{
			// Show error text
      FilereportCompleted(pView, _T("*** media is writeprotected ***"));

			// Completed
			return 4;
		}
		else if (fserr==FS_NOTFORMATTED)
		{
			// Show error text
      FilereportCompleted(pView, _T("*** media is not formatted ***"));

			// Completed
			return 7;
		}
		else if (fserr==FS_NOTEMPTY)
		{
			// Show error text
      FilereportCompleted(pView, _T("Directory \"") + pDirItem->Name + _T("\" delete error, not empty"));

			// Completed
			return 6;
		}
		else if (fserr!=FS_NOTFOUND) FilereportAdd(pView, _T("Directory \"") + pDirItem->Name + _T("\" delete error, unknown error"));
	}

	// Completed
	return 0;
}
