#pragma once
#include "afxwin.h"
#include "afxcmn.h"


// CDlgFilesystem dialog

class CDlgFilesystem : public CDialog
{
   DECLARE_DYNAMIC(CDlgFilesystem)

public:
   CDlgFilesystem(CWnd* pParent = NULL);   // standard constructor
   virtual ~CDlgFilesystem();

// Dialog Data
   enum { IDD = IDD_FILESYSTEM };
   CComboBox      c_drop;
   CButton        c_refresh;
   CListCtrl      c_list;
   CTreeCtrl      c_tree;
   CComboBox      c_wild;
   CButton        c_recursive;
   CString        m_wildcard;

protected:
   // Defines
   #define MEDIA       4

   // Data structures
   struct tdefFileInfo
   {
      CString Name;
      CString Time;
      long    Size;
   };
   struct tdefFolderInfo
   {
      CString                                Name;
      HTREEITEM                              Tree;
      CList<tdefFolderInfo*,tdefFolderInfo*> FolderList;
      CList<tdefFileInfo*,tdefFileInfo*>     FileList;
   };
   struct tdefMediaInfo
   {
      bool            opened;
      bool            present;
      bool            formatted;
      unsigned char   type;
      CString         title;
      tdefFolderInfo  rootdir;
   };
   struct tdefWorkInfo
   {
      CDialog*        wndParent;
      CDialog*        wndReport;
      CStringList     srcFiles;
      CString         dstPath;
      int             Type;
      tdefFolderInfo* dirInfo;
   };
   struct cbarg_t
   {
      CDlgFilesystem* view;
      tdefFolderInfo* dirItem;
   };

   // Controls
   CDialog*        m_ReportWnd;
   bool            m_refreshing;
   bool            m_fileproc;
   bool            m_closing;

   // Media drop list
   tdefMediaInfo   m_media[MEDIA];
   int             m_cur_media;

   // Directory tree
   tdefFolderInfo* m_root;

   // File report
   tdefFolderInfo* m_work;

protected:

   // Connection status
   static unsigned long __stdcall thConnection(LPVOID lpvParam);

   // Refresh
   static unsigned long __stdcall thRefresh( LPVOID lpvParam );
   void         ClearDirectory(tdefFolderInfo* dirItem);
   int          LoadDirectory(const CString& path, tdefFolderInfo* dirItem);
   static int __stdcall cbcatalog(void* uptr, const char* filename, unsigned char type, long timestamp, unsigned long size);

   // Media
   void         UpdateMedia(void);
   int          GetSelectedMedia(void);
   CString      GetMediaDrive(int media);

   // File report
   void         UpdateFileReport();

   // Work
   static unsigned long __stdcall thWork(LPVOID lpvParam);

   // Utility
   CString      GetSelectedPath();
   CString      GetPath(tdefFolderInfo* dirItem);
   void         ShowDirectory(tdefFolderInfo* dirItem);
   void         RemoveDirectory(tdefFolderInfo* dirItem);
   void         RemoveFile(tdefFolderInfo* dirItem, const CString& filename);
   int          fsfDirDelete(tdefFolderInfo* pDirItem);

   // Message map functions
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
   virtual BOOL OnInitDialog();
   afx_msg void OnDestroy();
   virtual void OnOK();
   virtual void OnCancel();
   afx_msg void OnClickRefresh();
   afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);
   afx_msg void OnSelectMedia();
   afx_msg void OnExpandDir(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnSelectDir(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnDeleteDir(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnDirCreate();
   afx_msg void OnDirDelete();
   afx_msg void OnMediaEject();
   afx_msg void OnMediaFormat();
   afx_msg void OnFileDelete();
   afx_msg void OnFileExport();
   afx_msg void OnFileImport();
   afx_msg void OnFileRename();
   afx_msg void OnFileSelectall();
   afx_msg void OnFileShow(NMHDR *pNMHDR, LRESULT *pResult);
   afx_msg void OnFileRemove(NMHDR *pNMHDR, LRESULT *pResult);
   LRESULT      OnUpdateWindow( WPARAM wParam, LPARAM lParam );

   DECLARE_MESSAGE_MAP()
public:
   afx_msg void OnTimer(UINT_PTR nIDEvent);
};
