// UnitNetPageCell.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "UnitNetPageCell.h"


// CUnitNetPageCell

IMPLEMENT_DYNAMIC(CUnitNetPageCell, CPropertyPage)

CUnitNetPageCell::CUnitNetPageCell()
	: CPropertyPage(CUnitNetPageCell::IDD)
{
}

CUnitNetPageCell::~CUnitNetPageCell()
{
}

void CUnitNetPageCell::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_INIT, m_ctrinit);
    DDX_Control(pDX, IDC_AUTHENTICATION, m_ctrauthentication);
    DDX_Control(pDX, IDC_APN, m_ctrapn);
    DDX_Control(pDX, IDC_GATEWAYIP, m_ctrgatewayip);
    DDX_Control(pDX, IDC_SUBNET, m_ctrsubnet);
    DDX_Control(pDX, IDC_IP, m_ctrip);
    DDX_Control(pDX, IDC_DNS2, m_ctrdns2);
    DDX_Control(pDX, IDC_DNS1, m_ctrdns1);
    DDX_Text(pDX, IDC_APN, m_apn);
    DDV_MaxChars(pDX, m_apn, 32);
    DDX_Text(pDX, IDC_PASSWORD, m_password);
    DDV_MaxChars(pDX, m_password, 16);
    DDX_Text(pDX, IDC_USERID, m_userid);
    DDV_MaxChars(pDX, m_userid, 32);
    DDX_CBIndex(pDX, IDC_AUTHENTICATION, m_authentication);
    DDX_Text(pDX, IDC_INIT, m_init);
    DDX_Check(pDX, IDC_RADIO_DYNAMIC, m_dynamic);
    DDX_Check(pDX, IDC_CHECK_NAT_GATEWAY, m_nat_gateway);
    DDX_Control(pDX, IDC_LABEL_NAT_GATEWAY, c_label_gateway);
    DDX_Control(pDX, IDC_CHECK_NAT_GATEWAY, c_check_gateway);
}

BEGIN_MESSAGE_MAP(CUnitNetPageCell, CPropertyPage)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CUnitNetPageCell::OnInitDialog()
{
	// Default
	CPropertyPage::OnInitDialog();

    // Update APN history
    if (m_apn_history.IsEmpty()) m_apn_history = m_apn + _T("||");
    if (!m_apn_history.IsEmpty())
    {
        CString str, strList = m_apn_history;
        // Find how many entries exists in the list, format is "AA||BB||CC||DD||"
        int i = strList.Replace("||", "||"); // This gives a count for the number of "||" sequences
        while (i)
        {
            str = strList.Left(strList.Find("||", 0)); // Extract the string
            strList.Delete(0, strList.Find("||", 0)); // Kill the string
            strList.Delete(0, 2); // Kill the "||"
            if (!str.IsEmpty())  m_ctrapn.AddString(str);
            i--;
        }
    }

    // Update view
    m_ctrdns1.SetAddress(m_dns1);
    m_ctrdns2.SetAddress(m_dns2);
    m_ctrip.SetAddress(m_ip);
    m_ctrgatewayip.SetAddress(m_gatewayip);
    m_ctrsubnet.SetAddress(m_subnet);
    m_ctrdns1.EnableWindow(!m_dynamic);
    m_ctrdns2.EnableWindow(!m_dynamic);
    m_ctrip.EnableWindow(!m_dynamic);
    m_ctrgatewayip.EnableWindow(!m_dynamic);
    m_ctrsubnet.EnableWindow(!m_dynamic);
    if (m_ctrapn.SelectString(-1, m_apn) == CB_ERR)
    {
        m_ctrapn.SetCurSel(-1); // 20161212 - MO - Ensure that the APN only contains text found in dropdown
        m_ctrapn.SetCurSel(0);  // 20120113 - MO - Ensure that the APN is newer empty
    }
    UpdateData(false);

    // Set controls
    UpdateControls();

	// Completed
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// Function
/////////////////////////////////////////////////////////////////////////////

bool CUnitNetPageCell::StripWhiteSpace(CString& text)
{
    // Var
    CString  res, span, temp, include;
    int      pos;
    bool     change;

    // Setup
    include = _T("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ.{}|'_[]/\\@?<>=;:-,+*()&%#!$");
    temp = text;
    pos = 0;

    // Strip illegal characters
    do
    {
        // Get next span
        span = temp.SpanIncluding(include);

        // Add span
        if (!span.IsEmpty())
        {
            res += span;
            temp = temp.Mid(span.GetLength());
        }
        // Remove illegal character
        else
        {
            temp = temp.Mid(1);
        }
    } while (!temp.IsEmpty());

    // Set output
    change = text.Compare(res) != 0;
    text = res;

    // Completed
    return change;
}

void CUnitNetPageCell::SetDefault(void)
{
    // Update variables
    m_apn               = _T("internet");
    m_userid            = _T("");
    m_password          = _T("");
    m_ip                = 0;
    m_subnet            = 0;
    m_dns1              = 0;
    m_dns2              = 0;
    m_gatewayip         = 0;
    m_authentication    = 3;
    m_init              = _T("");
    m_dynamic           = TRUE;
    m_nat_gateway       = FALSE;

    // Validate
    if (!m_ctrinit.GetSafeHwnd()) return;

    // Update view
    m_ctrdns1.SetAddress(m_dns1);
    m_ctrdns2.SetAddress(m_dns2);
    m_ctrip.SetAddress(m_ip);
    m_ctrgatewayip.SetAddress(m_gatewayip);
    m_ctrsubnet.SetAddress(m_subnet);
    m_ctrdns1.EnableWindow(!m_dynamic);
    m_ctrdns2.EnableWindow(!m_dynamic);
    m_ctrip.EnableWindow(!m_dynamic);
    m_ctrgatewayip.EnableWindow(!m_dynamic);
    m_ctrsubnet.EnableWindow(!m_dynamic);
    UpdateData(false);
}

BOOL CUnitNetPageCell::ValidateSetup(void)
{
    // Completed
    return TRUE;
}

int CUnitNetPageCell::ReadSetup(void)
{
    // Var
    rmonGPRSSettings  Settings;
    CString           temp;
    int               rc;

    // Get current
    UpdateData(TRUE);

    // Read configuration
    rc = rmonGetGPRSSettings(hCon, &Settings);
    if (rc) return (rmonNoData == rc) ? 1 : 2;

    // Read NAT configuration
    if (!(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201))
    {
        rmonObjectRead(
            hCon,
            RMONOBJ_TABLE_SYSTEM,
            "network.cell.nat*",
            object_input,
            &m_nat_gateway
        );
    }

    // Update APN history
    m_ctrapn.DeleteString(m_ctrapn.FindStringExact(-1, Settings.APN));
    m_ctrapn.InsertString(0, Settings.APN);
    m_ctrapn.SetCurSel(0);
    UpdateHistory();

    // Update variables
    m_dynamic = (Settings.dns_1 == 0 && Settings.dns_2 == 0 && Settings.ip_address == 0 && Settings.gateway == 0 && Settings.subnet_mask == 0) ? TRUE : FALSE;
    m_apn = Settings.APN;
    m_userid = Settings.username;
    m_password = Settings.password;
    m_ip = Settings.ip_address;
    m_subnet = Settings.subnet_mask;
    m_dns1 = Settings.dns_1;
    m_dns2 = Settings.dns_2;
    m_gatewayip = Settings.gateway;
    m_authentication = Settings.authentication;
    m_init = Settings.modem_init;

    // Remove white space characters
    StripWhiteSpace(m_userid);
    StripWhiteSpace(m_password);

    // Warn user
    if (StripWhiteSpace(m_apn))
    {
        MessageBox(_T("Illegal characters have been detected in the APN."), _T("Fetch from device"), MB_ICONWARNING);
    }

    // Update view
    m_ctrdns1.SetAddress(m_dns1);
    m_ctrdns2.SetAddress(m_dns2);
    m_ctrip.SetAddress(m_ip);
    m_ctrgatewayip.SetAddress(m_gatewayip);
    m_ctrsubnet.SetAddress(m_subnet);
    m_ctrdns1.EnableWindow(!m_dynamic);
    m_ctrdns2.EnableWindow(!m_dynamic);
    m_ctrip.EnableWindow(!m_dynamic);
    m_ctrgatewayip.EnableWindow(!m_dynamic);
    m_ctrsubnet.EnableWindow(!m_dynamic);
    UpdateData(false);

    // Completed
    return 0;
}

int CUnitNetPageCell::WriteSetup(void)
{
    // Var
    rmonObjectInfo    info;
    rmonGPRSSettings  Settings;
    int               rc;

    // Update data in variables
    UpdateData(true);

    // Remove white  space characters
    StripWhiteSpace(m_userid);
    StripWhiteSpace(m_password);
    StripWhiteSpace(m_apn);

    // Build settings struct
    m_ctrip.GetAddress(Settings.ip_address);                  // ip-address
    m_ctrsubnet.GetAddress(Settings.subnet_mask);             // subnet mask
    m_ctrgatewayip.GetAddress(Settings.gateway);              // gateway-address
    m_ctrdns1.GetAddress(Settings.dns_1);                     // DNS-address 1
    m_ctrdns2.GetAddress(Settings.dns_2);                     // DNS-address 2
    strcpy_s(Settings.username, 34, (LPCTSTR)m_userid);       // Username.
    strcpy_s(Settings.password, 34, (LPCTSTR)m_password);     // Password.
    strcpy_s(Settings.APN, 34, (LPCTSTR)m_apn);               // APN
    Settings.authentication = m_authentication;               // Authentication type, 0=none, 1=pap, 2=chap, 3=pap/chap (Input to SGAUTH)
    strcpy_s(Settings.modem_init, 254, (LPCTSTR)m_init);      // Modem init string
    if (m_dynamic)
    {
        Settings.dns_1       = 0;
        Settings.dns_2       = 0;
        Settings.ip_address  = 0;
        Settings.gateway     = 0;
        Settings.subnet_mask = 0;
    }

    // Write GPRS configuration
    rc = rmonSetGPRSSettings(hCon, Settings);
    if (rc) return 1;

    // Update APN history
    m_ctrapn.DeleteString(m_ctrapn.FindStringExact(-1, m_apn));
    m_ctrapn.InsertString(0, m_apn);
    m_ctrapn.SetCurSel(0);
    UpdateHistory();

    if (!(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201))
    {
        // Build NAT configuration
        strcpy_s(info.name, 200, "network.cell.nat.gateway");
        info.desc[0] = 0;
        info.type    = RMONOBJ_TYPE_BOOL;
        info.flags   = 0;
        info.max_val = 0;
        info.min_val = 0;
        info.size    = 4;
        info.data    = (unsigned char*)&m_nat_gateway;

        // Write
        rmonObjectWrite(hCon, RMONOBJ_TABLE_SYSTEM, 2, 1, &info, nullptr);
    }

    // Completed
    return 0;
}


/////////////////////////////////////////////////////////////////////////////
// Controls
/////////////////////////////////////////////////////////////////////////////

void CUnitNetPageCell::OnClickedDynamic()
{
    // Update variables
    m_dynamic = !m_dynamic;

    // Update view
    m_ctrdns1.EnableWindow(!m_dynamic);
    m_ctrdns2.EnableWindow(!m_dynamic);
    m_ctrip.EnableWindow(!m_dynamic);
    m_ctrgatewayip.EnableWindow(!m_dynamic);
    m_ctrsubnet.EnableWindow(!m_dynamic);
}


/////////////////////////////////////////////////////////////////////////////
// Utility
/////////////////////////////////////////////////////////////////////////////

void CUnitNetPageCell::UpdateHistory(void)
{
    // Var
    CString str, strList;
    int     i;

    // Iterate APN history
    for (i = 0; i < m_ctrapn.GetCount(); i++)
    {
        m_ctrapn.GetLBText(i, str);
        if (!str.IsEmpty())
        {
            strList += str;
            strList += "||";
        }
    }

    // Update
    m_apn_history = strList;
}

int CUnitNetPageCell::object_input(
                                   void*          uptr,
                                   const char*    name,
                                   int            type,
                                   int            size,
                                   unsigned char* data,
                                   unsigned long  flags,
                                   int            min_val,
                                   int            max_val,
                                   const char*    description
                                  )
{
    // Var
    BOOL* gateway = static_cast<BOOL*>(uptr);

    // Set
    if (!strcmp(&name[17], "gateway")) *gateway = (BOOL) * ((int*)data);

    // Completed
    return 1;
}

void CUnitNetPageCell::UpdateControls(void)
{
    // Validate
    if (!m_ctrinit.GetSafeHwnd()) return;

    // Set controls
    m_ctrinit.EnableWindow(gUnitInfo.TargetID > 100 || (gUnitInfo.TargetID >= 9 && gUnitInfo.FirmwareVer >= 454));
    m_ctrauthentication.EnableWindow(gUnitInfo.TargetID > 100 || (gUnitInfo.TargetID >= 9 && gUnitInfo.FirmwareVer >= 454));
    c_label_gateway.EnableWindow(!(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201));
    c_check_gateway.EnableWindow(!(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201));
}
