// UnitNetPageWlan.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "UnitNetPageWlan.h"
#include "UnitNetWlanSetup.h"


// CUnitNetPageWlan

IMPLEMENT_DYNAMIC(CUnitNetPageWlan, CPropertyPage)

CUnitNetPageWlan::CUnitNetPageWlan()
	: CPropertyPage(CUnitNetPageWlan::IDD)
{
    for (int i = 0; i < WIFI_NETWORKS; i++)
    {
        m_wifi_nodes[i] = nullptr;
    }
}

CUnitNetPageWlan::~CUnitNetPageWlan()
{
    for (int i = 0; i < WIFI_NETWORKS; i++)
    {
        if (m_wifi_nodes[i])
        {
            delete m_wifi_nodes[i];
            m_wifi_nodes[i] = nullptr;
        }
    }
}

void CUnitNetPageWlan::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_SSID, c_list);
    DDX_Control(pDX, IDC_ADD, c_add);
    DDX_Control(pDX, IDC_REMOVE, c_remove);
    DDX_Check(pDX, IDC_CHECK_NAT_GATEWAY, m_nat_gateway);
    DDX_Control(pDX, IDC_CHECK_NAT_GATEWAY, c_check_nat_gateway);
    DDX_Control(pDX, IDC_LABEL_NAT_GATEWAY, c_label_nat_gateway);
}

BEGIN_MESSAGE_MAP(CUnitNetPageWlan, CPropertyPage)
    ON_BN_CLICKED(IDC_ADD,               &CUnitNetPageWlan::OnClickedAdd)
    ON_BN_CLICKED(IDC_REMOVE,            &CUnitNetPageWlan::OnClickedRemove)
    ON_NOTIFY(NM_DBLCLK, IDC_SSID,       &CUnitNetPageWlan::OnListDClick)
    ON_NOTIFY(LVN_ITEMCHANGED, IDC_SSID, &CUnitNetPageWlan::OnListSelection)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CUnitNetPageWlan::OnInitDialog()
{
	// Default
	CPropertyPage::OnInitDialog();

    // Setup table
    c_list.InsertColumn(0, _T("A"), LVCFMT_RIGHT);
    c_list.InsertColumn(1, _T("A"), LVCFMT_LEFT);
    c_list.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
    c_list.SetColumnWidth(1, LVSCW_AUTOSIZE_USEHEADER);
    c_list.SetExtendedStyle(c_list.GetExtendedStyle() | LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);
    c_list.InsertItem(0, _T("1"));
    c_list.InsertItem(1, _T("2"));
    c_list.InsertItem(2, _T("3"));
    c_list.InsertItem(3, _T("4"));
    c_list.InsertItem(4, _T("5"));
    c_list.InsertItem(5, _T("6"));
    c_list.InsertItem(6, _T("7"));
    c_list.InsertItem(7, _T("8"));
    c_list.InsertItem(8, _T("9"));
    c_list.InsertItem(9, _T("10"));
    c_list.SetItemState(0, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);

    // Update data
    UpdateList();

    // Update controls
    UpdateView();

	// Completed
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// Buttons
/////////////////////////////////////////////////////////////////////////////

void CUnitNetPageWlan::OnClickedAdd()
{
    OnListDClick(nullptr, nullptr);
}

void CUnitNetPageWlan::OnClickedRemove()
{
    // Var
    CString      text, title, item;
    int cursel;

    // Get selected
    cursel = GetCurSel();
    if (cursel == LB_ERR) return;
    cursel++; // Convert to index in memory table

    // Validate
    if (!m_wifi_nodes[cursel]) return;

    // Show message
    text.Format(_T("Are you sure you want\nto remove the '%s' SSID?"), m_wifi_nodes[cursel]->ssid);
    title = _T("Remove SSID");
    if (MessageBox(text, title, MB_ICONWARNING | MB_YESNO) == IDNO)
    {
        return;
    }

    // Remove entry
    delete m_wifi_nodes[cursel];
    m_wifi_nodes[cursel] = nullptr;

    // Update
    UpdateList();
    UpdateView();
}

void CUnitNetPageWlan::OnListDClick(NMHDR* pNMHDR, LRESULT* pResult)
{
    // Var
    CUnitNetWlanSetup dlg;
    int               cursel, i;

    // Get selected
    cursel = GetCurSel();
    if (cursel == LB_ERR) return;
    cursel++; // Convert to index in memory table

    // Setup
    for (i = 1; i < WIFI_NETWORKS; i++)
    {
        if (i != cursel && m_wifi_nodes[i]) dlg.m_reserved.AddTail(m_wifi_nodes[i]->ssid);
    }
    if (m_wifi_nodes[cursel])
    {
        dlg.m_ssid      = m_wifi_nodes[cursel]->ssid;
        dlg.m_security  = m_wifi_nodes[cursel]->security.type;
        dlg.m_phrase    = m_wifi_nodes[cursel]->security.phrase;
        dlg.m_flags     = m_wifi_nodes[cursel]->ipv4.flags;
        dlg.m_ip        = m_wifi_nodes[cursel]->ipv4.ip;
        dlg.m_subnet    = m_wifi_nodes[cursel]->ipv4.subnet;
        dlg.m_gateway   = m_wifi_nodes[cursel]->ipv4.gateway;
        dlg.m_dns1      = m_wifi_nodes[cursel]->ipv4.dns1;
        dlg.m_dns2      = m_wifi_nodes[cursel]->ipv4.dns2;
    }

    // Edit setup
    if (dlg.DoModal() == IDOK)
    {
        // Var
        wifi_node_t* pItem;

        // Create SSID
        if (m_wifi_nodes[cursel])
        {
            pItem = m_wifi_nodes[cursel];
        }
        else
        {
            pItem = new wifi_node_t;
            if (pItem == NULL) return;
        }

        // Build SSID
        pItem->ssid             = dlg.m_ssid;
        pItem->security.type    = dlg.m_security;
        pItem->security.phrase  = dlg.m_phrase;
        pItem->ipv4.dns1        = dlg.m_dns1;
        pItem->ipv4.dns2        = dlg.m_dns2;
        pItem->ipv4.flags       = dlg.m_flags;
        pItem->ipv4.gateway     = dlg.m_gateway;
        pItem->ipv4.ip          = dlg.m_ip;
        pItem->ipv4.subnet      = dlg.m_subnet;

        // Insert SSID
        if (!m_wifi_nodes[cursel]) m_wifi_nodes[cursel] = pItem;

        // Update
        UpdateList();
        UpdateView();
    }
}


/////////////////////////////////////////////////////////////////////////////
// Functions
/////////////////////////////////////////////////////////////////////////////

void CUnitNetPageWlan::SetDefault(void)
{
    // Empty
    for (int i = 0; i < WIFI_NETWORKS; i++)
    {
        if (m_wifi_nodes[i])
        {
            delete m_wifi_nodes[i];
            m_wifi_nodes[i] = nullptr;
        }
    }

    // NAT
    m_nat_gateway = FALSE;

    // Update controls
    UpdateList();
    UpdateView();
    if (c_list.GetSafeHwnd())
        c_list.SetItemState(0, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
}

BOOL CUnitNetPageWlan::ValidateSetup(void)
{
    // Get information
    if (c_list.GetSafeHwnd()) UpdateData(TRUE);

    // Completed
    return TRUE;
}

int CUnitNetPageWlan::ReadSetup(void)
{
    // Var
    rmonWLANSettings Settings;
    rmonRet          rc;
    int              i;

    // Get current
    if (c_list.GetSafeHwnd()) UpdateData(TRUE);

    // Iterate WLAN networks
    for (i = 1; i < WIFI_NETWORKS; i++)
    {
        // Read configuration
        rc = rmonGetWLANSettings(hCon, i, &Settings);
        if (rc) return (rmonNoData == rc) ? 1 : 2;

        // Validate
        if (!Settings.ssid[0])
        {
            // Clear entry
            if (m_wifi_nodes[i])
            {
                delete m_wifi_nodes[i];
                m_wifi_nodes[i] = nullptr;
            }
            continue;
        }

        // Create new entry
        if (!m_wifi_nodes[i])
        {
            m_wifi_nodes[i]                 = new wifi_node_t;
        }

        // Update SSID
        m_wifi_nodes[i]->ssid               = Settings.ssid;

        // Update security
        m_wifi_nodes[i]->security.type      = Settings.security;
        m_wifi_nodes[i]->security.phrase    = Settings.sec.wpa.phrase;

        // Update network settings
        m_wifi_nodes[i]->ipv4.ip            = Settings.ipv4.addr;
        m_wifi_nodes[i]->ipv4.subnet        = Settings.ipv4.subnet;
        m_wifi_nodes[i]->ipv4.dns1          = Settings.ipv4.dns_1;
        m_wifi_nodes[i]->ipv4.dns2          = Settings.ipv4.dns_2;
        m_wifi_nodes[i]->ipv4.gateway       = Settings.ipv4.gateway;
        m_wifi_nodes[i]->ipv4.flags         = Settings.ipv4.flags;
    }

    // Read service configuration
    if (!(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201))
    {
        // Read NAT
        rmonObjectRead(
                       hCon,
                       RMONOBJ_TABLE_SYSTEM,
                       "network.wlan.nat*",
                       object_input,
                       &m_nat_gateway
                      );
    }

    // Update controls
    UpdateList();
    UpdateView();

    // Completed
    return 0;
}

int CUnitNetPageWlan::WriteSetup(void)
{
    // Var
    rmonWLANSettings Settings;
    rmonRet          rc;
    int              i;

    // Iterate
    for (i = 1; i < WIFI_NETWORKS; i++)
    {
        // Read configuration
        rmonGetWLANSettings(hCon, i, &Settings);

        // Remove entry
        if (!m_wifi_nodes[i] && Settings.ssid[0])
        {
            // Setup
            memset(&Settings, 0, sizeof(Settings));

            // Write configuration
            rc = rmonSetWLANSettings(hCon, i, &Settings);
            if (rc) return (rmonNoData == rc) ? 1 : 2;
        }
        // Update entry
        else if (m_wifi_nodes[i])
        {
            // Generate net
            strcpy_s((char*)Settings.ssid, 32, m_wifi_nodes[i]->ssid);
            strcpy_s((char*)Settings.sec.wpa.phrase, 64, m_wifi_nodes[i]->security.phrase);
            Settings.security       = m_wifi_nodes[i]->security.type;
            Settings.ipv4.dns_1     = m_wifi_nodes[i]->ipv4.dns1;
            Settings.ipv4.dns_2     = m_wifi_nodes[i]->ipv4.dns2;
            Settings.ipv4.flags     = m_wifi_nodes[i]->ipv4.flags;
            Settings.ipv4.gateway   = m_wifi_nodes[i]->ipv4.gateway;
            Settings.ipv4.addr      = m_wifi_nodes[i]->ipv4.ip;
            Settings.ipv4.subnet    = m_wifi_nodes[i]->ipv4.subnet;

            // Write configuration
            rc = rmonSetWLANSettings(hCon, i, &Settings);
            if (rc) return (rmonNoData == rc) ? 1 : 2;
        }
    }

    // Write service configuration
    if (!(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201))
    {
        // Var
        rmonObjectInfo    info;

        // Build NAT configuration
        strcpy_s(info.name, 200, "network.wlan.nat.gateway");
        info.desc[0]    = 0;
        info.type       = RMONOBJ_TYPE_BOOL;
        info.flags      = 0;
        info.max_val    = 0;
        info.min_val    = 0;
        info.size       = 4;
        info.data       = (unsigned char*)&m_nat_gateway;

        // Write
        rmonObjectWrite(hCon, RMONOBJ_TABLE_SYSTEM, 2, 1, &info, nullptr);
    }

    // Completed
    return 0;
}

void CUnitNetPageWlan::Load(void)
{
    // Var
    wifi_node_t* pItem;
    CString      profile;
    int          i, ssid_type;

    // Iterate
    for (i = 1; i < WIFI_NETWORKS; i++)
    {
        // Generate profile key
        profile.Format(_T("NetworkOptions\\WLAN\\Net-%d"), i);

        // Load SSID type
        ssid_type               = AfxGetApp()->GetProfileInt(   profile, _T("SSID TYPE"),  0);
        if (ssid_type != 1 && ssid_type != 2) continue;

        // Create SSID
        pItem                   = new wifi_node_t;
        if (pItem == NULL) return;

        // Build SSID
        pItem->ssid             = AfxGetApp()->GetProfileString(profile, _T("SSID"),       _T(""));
        pItem->security.type    = AfxGetApp()->GetProfileInt(   profile, _T("SECURITY"),   0);
        pItem->security.phrase  = AfxGetApp()->GetProfileString(profile, _T("PASSPHRASE"), _T(""));
        pItem->ipv4.dns1        = AfxGetApp()->GetProfileInt(   profile, _T("DNS1"),       0);
        pItem->ipv4.dns2        = AfxGetApp()->GetProfileInt(   profile, _T("DNS2"),       0);
        pItem->ipv4.flags       = AfxGetApp()->GetProfileInt(   profile, _T("FLAGS"),      3);
        pItem->ipv4.gateway     = AfxGetApp()->GetProfileInt(   profile, _T("GATEWAY"),    0);
        pItem->ipv4.ip          = AfxGetApp()->GetProfileInt(   profile, _T("ADDRESS"),    0);
        pItem->ipv4.subnet      = AfxGetApp()->GetProfileInt(   profile, _T("SUBNET"),     0);

        // Insert SSID
        m_wifi_nodes[i]         = pItem;
    }

    // Read NAT
    m_nat_gateway = AfxGetApp()->GetProfileInt(_T("NetworkOptions\\WLAN\\NAT"), _T("GATEWAY"), 0);
}

void CUnitNetPageWlan::Save(void)
{
    // Var
    CString profile;
    int     i;

    // Iterate
    for (i = 1; i < WIFI_NETWORKS; i++)
    {
        // Generate profile key
        profile.Format(_T("NetworkOptions\\WLAN\\Net-%d"), i);

        // Save setting
        if (m_wifi_nodes[i])
        {
            AfxGetApp()->WriteProfileInt(   profile, _T("SSID TYPE"),  1);
            AfxGetApp()->WriteProfileString(profile, _T("SSID"),       m_wifi_nodes[i]->ssid);
            AfxGetApp()->WriteProfileInt(   profile, _T("SECURITY"),   m_wifi_nodes[i]->security.type);
            AfxGetApp()->WriteProfileString(profile, _T("PASSPHRASE"), m_wifi_nodes[i]->security.phrase);
            AfxGetApp()->WriteProfileInt(   profile, _T("DNS1"),       m_wifi_nodes[i]->ipv4.dns1);
            AfxGetApp()->WriteProfileInt(   profile, _T("DNS2"),       m_wifi_nodes[i]->ipv4.dns2);
            AfxGetApp()->WriteProfileInt(   profile, _T("FLAGS"),      m_wifi_nodes[i]->ipv4.flags);
            AfxGetApp()->WriteProfileInt(   profile, _T("GATEWAY"),    m_wifi_nodes[i]->ipv4.gateway);
            AfxGetApp()->WriteProfileInt(   profile, _T("ADDRESS"),    m_wifi_nodes[i]->ipv4.ip);
            AfxGetApp()->WriteProfileInt(   profile, _T("SUBNET"),     m_wifi_nodes[i]->ipv4.subnet);
        }
        else
        {
            AfxGetApp()->WriteProfileInt(   profile, _T("SSID TYPE"),  0);
        }
    }

    // Write NAT
    AfxGetApp()->WriteProfileInt(_T("NetworkOptions\\WLAN\\NAT"), _T("GATEWAY"), m_nat_gateway);
}


/////////////////////////////////////////////////////////////////////////////
// Data functions
/////////////////////////////////////////////////////////////////////////////

void CUnitNetPageWlan::UpdateList(void)
{
    // Var
    CString entry;
    int     i;

    // Validate
    if (!c_list.GetSafeHwnd()) return;

    // Iterate
    for (i = 1; i < WIFI_NETWORKS; i++)
    {
        // Get network data
        entry = c_list.GetItemText(i - 1, 1);

        // Remove entry
        if (!m_wifi_nodes[i] && !entry.IsEmpty())
        {
            c_list.SetItemText(i - 1, 1, _T(""));
        }
        // Insert text
        else if (m_wifi_nodes[i] && m_wifi_nodes[i]->ssid.CompareNoCase(entry))
        {
            c_list.SetItemText(i - 1, 1, m_wifi_nodes[i]->ssid);
        }
    }

    // NAT
    UpdateData(FALSE);
}

int CUnitNetPageWlan::object_input(
                                   void*          uptr,
                                   const char*    name,
                                   int            type,
                                   int            size,
                                   unsigned char* data,
                                   unsigned long  flags,
                                   int            min_val,
                                   int            max_val,
                                   const char*    description
                                  )
{
    // Var
    BOOL* gateway = static_cast<BOOL*>(uptr);

    // Set
    if (!strcmp(&name[17], "gateway")) *gateway = (BOOL) * ((int*)data);

    // Completed
    return 1;
}


/////////////////////////////////////////////////////////////////////////////
// Control functions
/////////////////////////////////////////////////////////////////////////////

int CUnitNetPageWlan::GetCurSel(void)
{
    // Var
    POSITION pos;

    // Get selected
    pos = c_list.GetFirstSelectedItemPosition();
    if (pos == nullptr) return LB_ERR;
    return c_list.GetNextSelectedItem(pos);
}

void CUnitNetPageWlan::OnListSelection(NMHDR* pNMHDR, LRESULT* pResult)
{
    LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
    *pResult = 0;

    if ((pNMLV->uChanged & LVIF_STATE) && (pNMLV->uNewState & LVIS_SELECTED))
    {
        UpdateView();
    }
}

void CUnitNetPageWlan::UpdateView(void)
{
    // Var
    BOOL nat_enable;
    int cursel = -1;

    // Validate
    if (!c_list.GetSafeHwnd()) return;

    // Get selected
    cursel = GetCurSel();
    nat_enable = !(gUnitInfo.TargetID < 200 || gUnitInfo.TargetID == 201);

    // Update
 //   c_add.EnableWindow(wifi_nodes.GetCount() < 10 ? TRUE : FALSE);
    c_remove.EnableWindow((cursel < 0 || !m_wifi_nodes[cursel + 1]) ? FALSE : TRUE);
    c_check_nat_gateway.EnableWindow(nat_enable);
    c_label_nat_gateway.EnableWindow(nat_enable);
}
