// UnitNetWlanSetup.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "UnitNetWlanSetup.h"
#include "afxdialogex.h"


// CUnitNetWlanSetup dialog

IMPLEMENT_DYNAMIC(CUnitNetWlanSetup, CDialog)

CUnitNetWlanSetup::CUnitNetWlanSetup(CWnd* pParent /*=nullptr*/)
	: CDialog(IDD_UNIT_NET_WLAN_SETUP, pParent)
	, m_ssid(_T(""))
	, m_security(0)
	, m_phrase(_T(""))
	, m_flags(3)
	, m_ip(0)
	, m_subnet(0)
	, m_gateway(0)
	, m_dns1(0)
	, m_dns2(0)
	, m_sel_ip(0)
	, m_sel_dns(0)
{
}

CUnitNetWlanSetup::~CUnitNetWlanSetup()
{
}

void CUnitNetWlanSetup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SECURITY, c_security);
	DDX_Control(pDX, IDC_GATEWAYIP, c_gateway);
	DDX_Control(pDX, IDC_SUBNET, c_subnet);
	DDX_Control(pDX, IDC_IP, c_address);
	DDX_Control(pDX, IDC_DNS2, c_dns2);
	DDX_Control(pDX, IDC_DNS1, c_dns1);
	DDX_Radio(pDX, IDC_RADIO_DYNAMIC, m_sel_ip);
	DDX_Radio(pDX, IDC_DNS_DYNAMIC, m_sel_dns);
}

BEGIN_MESSAGE_MAP(CUnitNetWlanSetup, CDialog)
	ON_CBN_SELENDOK(IDC_SECURITY,    &CUnitNetWlanSetup::OnSelectSecurity)
	ON_BN_CLICKED(IDC_RADIO_DYNAMIC, &CUnitNetWlanSetup::OnClickedDynamic)
	ON_BN_CLICKED(IDC_RADIO_STATIC,  &CUnitNetWlanSetup::OnClickedStatic)
	ON_BN_CLICKED(IDC_DNS_DYNAMIC,   &CUnitNetWlanSetup::OnClickedDynamicDns)
	ON_BN_CLICKED(IDC_DNS_STATIC,    &CUnitNetWlanSetup::OnClickedStaticDns)
END_MESSAGE_MAP()

// CUnitNetWlanSetup message handlers

BOOL CUnitNetWlanSetup::OnInitDialog()
{
    // Default
    CDialog::OnInitDialog();

    // Initialize controls
    ((CEdit*)GetDlgItem(IDC_PASSPHRASE))->LimitText(62);

    // Fill controls
    c_security.AddString(_T("None"));
    c_security.AddString(_T("WPA/WPA2 - PSK"));

    // Update
    GetDlgItem(IDC_SSID)->SetWindowText(m_ssid);

    // Update security settings
    c_security.SetCurSel(m_security);
    GetDlgItem(IDC_PASSPHRASE)->SetWindowText(m_phrase);

    // Update network settings
    c_dns1.SetAddress(m_dns1);
    c_dns2.SetAddress(m_dns2);
    c_address.SetAddress(m_ip);
    c_gateway.SetAddress(m_gateway);
    c_subnet.SetAddress(m_subnet);
    m_sel_ip = (m_flags & 0x0001) ? 0 : 1;
    m_sel_dns = (m_flags & 0x0002) ? 0 : 1;

    // Update view
    UpdateData(FALSE);
    ShowType(m_security);
    UpdateView();
    GetDlgItem(IDC_SSID)->SetFocus();

    // Completed
    return TRUE;
}

void CUnitNetWlanSetup::OnOK()
{
    // Get configuration
    UpdateData(TRUE);

    // Read settings
    GetDlgItem(IDC_SSID)->GetWindowText(m_ssid);

    // Read security settings
    m_security = c_security.GetCurSel();
    GetDlgItem(IDC_PASSPHRASE)->GetWindowText(m_phrase);

    // Read network settings
    c_dns1.GetAddress(m_dns1);
    c_dns2.GetAddress(m_dns2);
    c_address.GetAddress(m_ip);
    c_subnet.GetAddress(m_subnet);
    c_gateway.GetAddress(m_gateway);
    m_flags = 0;
    if (m_sel_ip == 0)  m_flags |= 1;
    if (m_sel_dns == 0) m_flags |= 2;

    // Validate SSID
    if (m_ssid.IsEmpty())
    {
        // Var
        CString text, title;

        // Generate text
        text  = _T("The SSID cannot be empty");
        title = _T("Edit network");

        // Show error
        MessageBox(text, title, MB_ICONWARNING | MB_OK);

        // Completed
        GetDlgItem(IDC_SSID)->SetFocus();
        return;
    }
    if (m_reserved.Find(m_ssid))
    {
        // Var
        CString text, title;

        // Generate text
        text.Format(_T("A WLAN network with the SSID '%s' already exists."), m_ssid);
        title = _T("Edit network");

        // Show error
        MessageBox(text, title, MB_ICONWARNING | MB_OK);

        // Completed
        GetDlgItem(IDC_SSID)->SetFocus();
        return;
    }

    // Validate security
    if (m_security && m_phrase.GetLength() < 8)
    {
        // Var
        CString text, title;

        // Generate text
        text  = _T("The passphrase must be 8 or more characters long");
        title = _T("Edit network");

        // Show error
        MessageBox(text, title, MB_ICONWARNING | MB_OK);

        // Completed
        GetDlgItem(IDC_PASSPHRASE)->SetFocus();
        return;
    }

    // Completed
    CDialog::OnOK();
}

/////////////////////////////////////////////////////////////////////////////
// Events
/////////////////////////////////////////////////////////////////////////////

void CUnitNetWlanSetup::OnSelectSecurity()
{
    ShowType(c_security.GetCurSel());
}

void CUnitNetWlanSetup::OnClickedDynamic()
{
    UpdateData(TRUE);
    UpdateView();
}

void CUnitNetWlanSetup::OnClickedStatic()
{
    // Update
    UpdateData(TRUE);
    m_sel_dns = 1;
    UpdateData(FALSE);

    // Update controls
    UpdateView();
}

void CUnitNetWlanSetup::OnClickedDynamicDns()
{
    UpdateData(TRUE);
    UpdateView();
}

void CUnitNetWlanSetup::OnClickedStaticDns()
{
    UpdateData(TRUE);
    UpdateView();
}

/////////////////////////////////////////////////////////////////////////////
// Utility
/////////////////////////////////////////////////////////////////////////////

void CUnitNetWlanSetup::ShowType(int type)
{
    // Var
    int show_text_phrase, show_edit_phrase;

    //
    // Show WPA/WPA2 - PSK
    //
    if (type == 1)
    {
        show_text_phrase = SW_SHOW;
        show_edit_phrase = SW_SHOW;
    }
    else
    {
        show_text_phrase = SW_HIDE;
        show_edit_phrase = SW_HIDE;
    }

    // Show/Hide controls
    GetDlgItem(IDC_TEXT_PASSPHRASE)->ShowWindow(show_text_phrase);
    GetDlgItem(IDC_PASSPHRASE)->ShowWindow(show_edit_phrase);
}

void CUnitNetWlanSetup::UpdateView(void)
{
    // Update IP address
    c_gateway.EnableWindow(m_sel_ip ? TRUE : FALSE);
    c_subnet.EnableWindow(m_sel_ip ? TRUE : FALSE);
    c_address.EnableWindow(m_sel_ip ? TRUE : FALSE);

    // Update DNS
    GetDlgItem(IDC_DNS_DYNAMIC)->EnableWindow(m_sel_ip ? FALSE : TRUE);
    c_dns1.EnableWindow(m_sel_dns ? TRUE : FALSE);
    c_dns2.EnableWindow(m_sel_dns ? TRUE : FALSE);
}
