// UnitOptionServer.cpp : implementation file
//

#include "stdafx.h"
#include "RTCUProg.h"
#include "UnitOptionServer.h"
#include "DlgAuthentication.h"
#include "library\libOKSClient.h"


// CUnitOptionServer dialog

IMPLEMENT_DYNAMIC(CUnitOptionServer, CDialogEx)

CUnitOptionServer::CUnitOptionServer(CWnd* pParent /*=NULL*/)
  : CDialogEx(CUnitOptionServer::IDD, pParent)
  , m_user(_T(""))
  , m_pass(_T(""))
  , m_credits(_T(""))
  , m_updating_list(false)
  , m_inupdate(false)
  , m_selected(0)
  , m_serial(0)
{
}

CUnitOptionServer::~CUnitOptionServer()
{
}

void CUnitOptionServer::DoDataExchange(CDataExchange* pDX)
{
  CDialogEx::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_LIST, m_list);
  DDX_Text(pDX, IDC_STATUS, m_credits);
}


BEGIN_MESSAGE_MAP(CUnitOptionServer, CDialogEx)
  ON_WM_DESTROY()
  ON_WM_SHOWWINDOW()
  ON_BN_CLICKED(IDC_AUTHENTICATE, &CUnitOptionServer::OnClickedAuthenticate)
  ON_WM_TIMER()
  ON_BN_CLICKED(IDC_UPDATE, &CUnitOptionServer::OnClickedUpdate)
  ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST, &CUnitOptionServer::OnItemChanged)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Initiation / Destruction
/////////////////////////////////////////////////////////////////////////////

BOOL CUnitOptionServer::OnInitDialog()
{
  // Default initialization
  CDialogEx::OnInitDialog();
  CenterWindow();

  // Initialize option list
  m_list.SetExtendedStyle(m_list.GetExtendedStyle() | LVS_EX_CHECKBOXES);
  m_list.InsertColumn(0, _T(""), LVCFMT_LEFT, 20);
  m_list.InsertColumn(1, _T("  ID  "));
  m_list.InsertColumn(2, _T("Price"));
  m_list.InsertColumn(3, _T("Option"));
  m_list.SetColumnWidth(1, LVSCW_AUTOSIZE_USEHEADER);
  m_list.SetColumnWidth(2, LVSCW_AUTOSIZE_USEHEADER);
  m_list.SetColumnWidth(3, LVSCW_AUTOSIZE_USEHEADER);

  // Get stored username
  m_user = AfxGetApp()->GetProfileString(_T("OptionServer"), _T("user"));

  // Create popup
  popup = new CDlgPopup();
  popup->Create(IDD_POPUP, this);

  // Completed
  return TRUE;
}

void CUnitOptionServer::OnDestroy()
{
  AfxGetApp()->WriteProfileString(_T("OptionServer"), _T("user"), m_user);
  CDialogEx::OnDestroy();
}

void CUnitOptionServer::PostNcDestroy()
{
  CDialogEx::PostNcDestroy();
  delete this;
}

/////////////////////////////////////////////////////////////////////////////
// Fail safe
/////////////////////////////////////////////////////////////////////////////

void CUnitOptionServer::OnCancel()
{
  // Remove exsisting options
  m_list.DeleteAllItems();
  m_option_id.RemoveAll();
  m_credits = _T("");
  m_selected = 0;
  UpdateData(FALSE);

  // Hide dialog
  ShowWindow(SW_HIDE);
}

void CUnitOptionServer::OnOK()
{
}

/////////////////////////////////////////////////////////////////////////////
// Update list of options
/////////////////////////////////////////////////////////////////////////////

void CUnitOptionServer::OnShowWindow(BOOL bShow, UINT nStatus)
{
  // Show dialog
  CDialogEx::OnShowWindow(bShow, nStatus);
  if (!bShow) return;

  // Authenticate user
  if (m_user.IsEmpty() || m_pass.IsEmpty()) {
    SetTimer(IDC_AUTHENTICATE, 100, NULL);
  } else {
    SetTimer(99099, 100, NULL);
  }
}

void CUnitOptionServer::ConnectChanged(void)
{
  if (!gUnitInfo.Connected) m_update_serial = -1;

  // Verify internal state
  if (!IsWindowVisible()) return;

  // Connection lost
  if (!gUnitInfo.Connected) {
    GetDlgItem(IDC_UPDATE)->EnableWindow(FALSE);
  } else {
    popup->Close();
    // Check target and version
    if (gUnitInfo.TargetID < 100) {
      // Old fujitsu targets is not supported
      GetDlgItem(IDC_UPDATE)->EnableWindow(FALSE);
    } else if (gUnitInfo.FirmwareVer < 262 || (gUnitInfo.FirmwareVer > 8000 && gUnitInfo.FirmwareVer < 8262)) {
      // Old firmware versions is not supported
      GetDlgItem(IDC_UPDATE)->EnableWindow(FALSE);
    } else if (m_serial == (long) gUnitInfo.SerialNumber) {
      // Its the same unit, do nothing
      GetDlgItem(IDC_UPDATE)->EnableWindow((m_selected > 0) ? TRUE : FALSE);
    } else {
      // Check if we need to update option list
      if ((m_serial/1000000) == (long) (gUnitInfo.SerialNumber/1000000)) {
        // Its the same unit type, if any options are selected then start update sequence
        m_serial = gUnitInfo.SerialNumber;  // Save serial number
        GetDlgItem(IDC_UPDATE)->EnableWindow((m_selected > 0) ? TRUE : FALSE);
        if (m_selected > 0) OnClickedUpdate();
      } else {
        // Update list of options
        UpdateList();
      }
    }
  }
}

void CUnitOptionServer::OnClickedAuthenticate()
{
  // Var
  CDlgAuthentication dlg;

  // Ask for authentication
  dlg.userid   = m_user;
  dlg.password = m_pass;
  if (dlg.DoModal() == IDOK) {
    // Store user information
    m_user = dlg.userid;
    m_pass = dlg.password;
    // Update list of options
    if (!m_user.IsEmpty() && !m_pass.IsEmpty()) {
      UpdateList();
    } else {
      // Remove exsisting options
      m_list.DeleteAllItems();
      m_option_id.RemoveAll();
      m_credits = _T("");
      m_selected = 0;
      UpdateData(FALSE);
      SetWindowText(_T("Request unit options"));
      MessageBox(_T("Wrong username or password!"), _T("Authenticate"), MB_OK);
    }
  }
}

void CUnitOptionServer::OnTimer(UINT_PTR nIDEvent)
{
  KillTimer(nIDEvent);
  if (nIDEvent == IDC_AUTHENTICATE)
    PostMessage(WM_COMMAND, MAKELONG(IDC_AUTHENTICATE, BN_CLICKED), (LPARAM)GetDlgItem(IDC_AUTHENTICATE)->GetSafeHwnd());
  else
    UpdateList();
  CDialogEx::OnTimer(nIDEvent);
}

void CUnitOptionServer::UpdateList(void)
{
  // Var
  CString title;
  int     rc;

  // Save serial number
  m_serial = gUnitInfo.SerialNumber;

  // Remove exsisting options
  m_list.DeleteAllItems();
  m_option_id.RemoveAll();
  m_credits = _T("");
  m_selected = 0;
  UpdateData(FALSE);

  BeginWaitCursor();
  m_updating_list = true;

  // Connect to server
  rc = oksConnect((LPTSTR)(LPCTSTR)m_user, (LPTSTR)(LPCTSTR)m_pass);
  if (!rc) {
    // Wait for connections
    while (oksStatus() < 3) Sleep(100);
    if (oksStatus() == 3) {
      // Var
      int profile;

      // Get profile number of connected unit
      profile = m_serial / 1000000;

      // Get list of options
      if (!oksOptionsUpdate(profile)) {
        // Var
        CString temp;
        char    id[6], desc[200];
        int     price, index, idx;

        // Iterate options
        index = 0;
        while (!oksOptionsGet(index++, id, &price, desc, sizeof(desc))) {
          temp.Format(_T("%d"), price);
          idx = m_list.InsertItem(index, _T(""));
          m_list.SetItemText(idx, 1, id);
          m_list.SetItemText(idx, 2, temp);
          m_list.SetItemText(idx, 3, desc);
          m_option_id.Add(CString(id));
        };

        // Update available credits
        m_credits.Format(_T("%d"), oksGetCredits());
      }

      //
      title = _T("Request unit options - ");
      title += m_user;
    } else {
      // Error message
      if (oksStatus() == 7) title = _T("Wrong username or password!");
      else                  title = _T("Could not connect to the option server!");
      MessageBox(title, _T("Update option list"), MB_OK);
      title = _T("Request unit options");
    }
    // Close connection
    oksDisconnect();
  } else {
    MessageBox(_T("Could not connect to the option server."), _T("Update option list"), MB_OK);
    title = _T("Request unit options");
  }

  EndWaitCursor();
  m_updating_list = false;

  // Update view
  SetWindowText(title);
  UpdateData(FALSE);
  GetDlgItem(IDC_UPDATE)->EnableWindow(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// Order options
/////////////////////////////////////////////////////////////////////////////

void CUnitOptionServer::OnClickedUpdate()
{
  // Var
  CString options, text;
  int     rc, cost = 0;
  char    key[12], order = 0;
  long    serial = m_serial;

  // Verify
  if (m_inupdate) return;
  m_inupdate = true;

  // Ask to update
  m_update_serial = m_serial;
  if (MessageBox(_T("Are you sure you want to request these options?"), _T("Request options"), MB_YESNO|MB_ICONQUESTION) != IDYES) {
    m_inupdate = false;
    return;
  }
  if (m_update_serial != m_serial) {
    text.Format(_T("Request is terminated!\nThe unit %d has been removed from PC."), serial);
    MessageBox(text, _T("Request options"), MB_OK|MB_ICONINFORMATION);
    m_inupdate = false;
    return;
  }

  BeginWaitCursor();
  m_updating_list = true;

  // Build list of selected options
  for (int i = 0; i < m_list.GetItemCount(); i++) {
    if (m_list.GetCheck(i)) {
      if (!options.IsEmpty()) options += _T(" ");
      options += m_option_id.GetAt(i);
    }
  }

  // Connect to server
  rc = oksConnect((LPTSTR)(LPCTSTR)m_user, (LPTSTR)(LPCTSTR)m_pass);
  if (!rc) {
    // Wait for connections
    while (oksStatus() < 3) Sleep(100);
    if (oksStatus() == 3) {
      // Purchase the options
      rc = oksOrder(m_serial, (LPTSTR)(LPCTSTR)options, key, &cost);
      if (!rc) {
        order = 1;
        // Update available credits
        m_credits.Format(_T("%d"), oksGetCredits());
        GetDlgItem(IDC_STATUS)->SetWindowText(m_credits);
      } else {
        switch (rc) {
          case -1:  text = _T("Connection to the option server is lost."); break;
          case -2:  text = _T("Timeout waiting for reply from the option server."); break;
          case 1:   text = _T("General error."); break;
          case 2:   text = _T("Unknown option is requested."); break;
          case 3:   text = _T("You do not have enough credits for all the selected options."); break;
          case 4:   text = _T("One of the option(s) require Logic IO approval."); break;
          default:  text.Format(_T("Unknown error (%d)"), rc);
        }
        MessageBox(text, _T("Request options"), MB_OK|MB_ICONINFORMATION);
      }
    } else {
      // Error message
      if (oksStatus() == 7) text = _T("Wrong username or password!");
      else                  text = _T("Could not connect to the option server!");
      MessageBox(text, _T("Request options"), MB_OK);
    }
    // Close connection
    oksDisconnect();
  } else {
    MessageBox(_T("Could not connect to the option server."), _T("Request options"), MB_OK);
  }

  if (order) {
    // Update unit
    rc = rmonFlexOption(hCon, (LPTSTR)(LPCTSTR)key);
    if (rc != rmonOK) {
      switch (rc) {
        case rmonComError:      text = _T("Error communicating with unit!"); break;
        case rmonIllegalHandle: text = _T("Connection closed!"); break;
        case rmonTargetError:   text = _T("Unit does not support Flex Options!"); break;
        case rmonError:         text = _T("Option key is rejected by unit!"); break;
        default:                text.Format(_T("Unknown error (%d)"), rc);
      }
      MessageBox(text, _T("Request options"), MB_OK|MB_ICONINFORMATION);
    } else {
      options.Format(_T("Request confirmed - %d"), m_serial);
      text.Format(_T("Option(s) are enabled! (Cost %d)\n\nPlease note that the requested option(s)\nwill not take effect until the unit is reset."), cost);
      popup->MessageBox(text, options);
    }
  }

  EndWaitCursor();
  m_updating_list = false;
  m_inupdate      = false;
}

void CUnitOptionServer::OnItemChanged(NMHDR *pNMHDR, LRESULT *pResult)
{
  // Default
  LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
  *pResult = 0;

  // This event is also called when inserting new options
  if (m_updating_list) return;

  // Adjust count of selected options
  if (m_list.GetCheck(pNMLV->iItem))
    m_selected++;
  else
    m_selected--;
  GetDlgItem(IDC_UPDATE)->EnableWindow((gUnitInfo.Connected && (m_selected > 0)) ? TRUE : FALSE);
}
