//=============================== RIJNDAEL.H   ===============================

#ifndef H__RIJNDAEL
#define H__RIJNDAEL


int rijndaelSetupEncrypt(unsigned long *rk,              //OUT: The key to use with rijndaelEncrypt()                              
                         const unsigned char *key,       //IN:  Key                                                                
                         int keybits);                   //IN: Number of keybits. 128 (16 bytes) / 192 (24 bytes) / 256 (32 bytes) 
                                                         //RETURN: The 'nrounds' parameter to be used with rijndaelEncrypt()       
                                                                                                                                   
int rijndaelSetupDecrypt(unsigned long *rk,              //OUT: The key to use with rijndaelDecrypt()                              
                         const unsigned char *key,       //IN:  Key                                                                
                         int keybits);                   //IN: Number of keybits. 128 (16 bytes) / 192 (24 bytes) / 256 (32 bytes) 
                                                         //RETURN: The 'nrounds' parameter to be used with rijndaelDecrypt()       

void rijndaelEncrypt(const unsigned long *rk,            //IN: They key returned from rijndaelSetupEncrypt() 
                     int nrounds,                        //IN: Nrounds as returned by rijndaelSetupEncrypt()  
                     const unsigned char plaintext[16],  //IN: The block to encrypt 
                     unsigned char ciphertext[16]);      //OUT: The encrypted data

void rijndaelDecrypt(const unsigned long *rk,            //IN: They key returned from rijndaelSetupDecrypt()
                     int nrounds,                        //IN: Nrounds as returned by rijndaelSetupDecrypt()
                     const unsigned char ciphertext[16], //IN: The block to decrypt                         
                     unsigned char plaintext[16]);       //OUT: The decrypted data                          

void rijndaelEncryptBuffer128(const unsigned char key[16],unsigned char* in_buf,unsigned char* out_buf,int nblocks);
void rijndaelDecryptBuffer128(const unsigned char key[16],unsigned char* in_buf,unsigned char* out_buf,int nblocks);

#define KEYLENGTH(keybits) ((keybits)/8)
#define RKLENGTH(keybits)  ((keybits)/8+28)
#define NROUNDS(keybits)   ((keybits)/32+6)

#endif

